package com.gci.cdnev.smallprogram.annotation;

import org.springframework.core.MethodParameter;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.support.WebDataBinderFactory;
import org.springframework.web.context.request.NativeWebRequest;
import org.springframework.web.context.request.RequestAttributes;
import org.springframework.web.method.support.HandlerMethodArgumentResolver;
import org.springframework.web.method.support.ModelAndViewContainer;

@Component
public class CurrentUserMethodArgumentResolver implements HandlerMethodArgumentResolver {

    /**
     * 拦截参数中含有CurrentUser注解的请求
     * @param parameter 参数
     * @return
     */
    @Override
    public boolean supportsParameter(MethodParameter parameter) {
        // 如果controller中的参数有CurrentUser注解修饰，则执行resolveArgument方法
        return parameter.hasParameterAnnotation(CurrentUser.class);
    }

    /**
     * 实现参数解析，返回CurrentUser中的登录用户信息
     * @param parameter
     * @param mavContainer
     * @param webRequest
     * @param binderFactory
     * @return
     * @throws Exception
     */
    @Override
    public Object resolveArgument(MethodParameter parameter, ModelAndViewContainer mavContainer, NativeWebRequest webRequest, WebDataBinderFactory binderFactory) throws Exception {
        CurrentUser currentUserAnnotation = parameter.getParameterAnnotation(CurrentUser.class);
        return webRequest.getAttribute(currentUserAnnotation.value(), RequestAttributes.SCOPE_REQUEST);
    }
}