package com.gci.cdnev.smallprogram.common.util;

import java.util.ArrayList;
import java.util.Date;
import java.util.Objects;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.gci.cdnev.smallprogram.modules.system.entity.SysDepart;
import com.gci.cdnev.smallprogram.modules.system.entity.SysUser;
import com.gci.cdnev.smallprogram.modules.system.service.ISysDepartService;
import com.gci.cdnev.smallprogram.modules.system.service.ISysUserService;
import com.gci.cdnev.smallprogram.modules.wx.entity.JwtUser;

import cn.hutool.core.util.StrUtil;
import io.jsonwebtoken.Claims;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.SignatureAlgorithm;
import lombok.extern.slf4j.Slf4j;


@Slf4j
@Component
public class JwtUtils {

    public static String header = "X-Access-Token";

    public static long expireTime = 1000 * 60 * 60 * 24 * 60;

    public static String tokenPrefix = "Bearer";

    public static String tokenSecret = "ukc8BDbRigUDaY6pZFfWus2jZWLPHO";

    @Autowired(required = true)
    private static ISysUserService sysUserService;

    @Autowired(required = true)
    private static ISysDepartService sysDepartService;

    @Autowired
    public void setSysUserService(ISysUserService sysUserService) {
        JwtUtils.sysUserService = sysUserService;
    }

    @Autowired
    public void setSysDepartService(ISysDepartService sysDepartService) {
    	JwtUtils.sysDepartService = sysDepartService;
    }

    /**
     * 生成token
     *
     * @param jwtUser
     * @return
     */
    public static String createJwtToken(JwtUser jwtUser) {
        String JwtToken = Jwts.builder()
                .setHeaderParam("typ", "JWT")
                .setHeaderParam("alg", "HS256")
                .setSubject(String.valueOf(jwtUser.getId()))
                .setIssuedAt(new Date())
                .setExpiration(new Date(System.currentTimeMillis() + expireTime))
                .claim("userId", jwtUser.getId())
                .claim("dispatcherId", jwtUser.getDispatcherId())
                .claim("userName", jwtUser.getUserName())
                .claim("employeeCode", jwtUser.getEmployeeCode())
                .claim("sysOrgId", jwtUser.getSysOrgId())
                .claim("sysOrgCode", jwtUser.getSysOrgCode())
                .claim("sysOrgName", jwtUser.getSysOrgName())
                .claim("qualification", jwtUser.getQualification())
                .claim("serviceCode", jwtUser.getServiceCode())
                .claim("traceId", jwtUser.getTenantId())
                .claim("type", jwtUser.getType())
                .claim("employeeId", jwtUser.getEmployeeId())
                .claim("employeeName", jwtUser.getEmployeeName())
                .claim("projectIds", jwtUser.getProjectIds())
                .signWith(SignatureAlgorithm.HS256, tokenSecret)
                .compact();
        return tokenPrefix + JwtToken;
    }

    /**
     * 根据token获取用户信息
     * @param request
     * @return
     */
    public static JwtUser getUserInfoByToken(HttpServletRequest request) {
        //todo 超时续期问题
        String jwtToken = request.getHeader(header);
        if(StrUtil.isEmpty(jwtToken)) {
            return null;
        }
       if (!jwtToken.startsWith(tokenPrefix)) {
            return null;
        } else {
           jwtToken= jwtToken.substring(tokenPrefix.length());
        }
        try {
            Claims claims = Jwts.parser().setSigningKey(tokenSecret).parseClaimsJws(jwtToken).getBody();
            JwtUser jwtUser = new JwtUser();

            jwtUser.setId(claims.get("userId",String.class));

            if(Objects.nonNull(jwtUser.getId())){
                // 根据id获取员工信息
                SysUser sysUser = sysUserService.getUserById(jwtUser.getId());
                jwtUser.setEmployeeId(sysUser.getEmployeeId());
                jwtUser.setEmployeeName(sysUser.getEmployeeName());
                jwtUser.setType(sysUser.getType());
                jwtUser.setUserName(sysUser.getUsername());
                jwtUser.setRealname(sysUser.getRealname());
                jwtUser.setProjectIds(sysUser.getProjectIds());
                //获取所有机构信息
                SysDepart orgDto = sysDepartService.getByUserId(sysUser.getId());
                if(orgDto!=null) {
                	jwtUser.setSysOrgCode(orgDto.getOrgCode());
                	jwtUser.setSysOrgId(orgDto.getId());
                	jwtUser.setSysOrgName(orgDto.getDepartName());
                }
            }else {
                jwtUser.setUserName(claims.get("userName",String.class));
                jwtUser.setRealname(claims.get("realname",String.class));
                jwtUser.setEmployeeCode(claims.get("employeeCode",String.class));
                jwtUser.setSysOrgId(claims.get("sysOrgId",String.class));
                jwtUser.setSysOrgCode( claims.get("sysOrgCode",String.class));
                jwtUser.setSysOrgName(claims.get("sysOrgName",String.class));
                jwtUser.setQualification(claims.get("qualification",String.class));
                jwtUser.setDispatcherId(claims.get("dispatcherId",String.class));
                jwtUser.setType(claims.get("type",String.class));
                jwtUser.setEmployeeId(claims.get("employeeId",String.class));
                jwtUser.setEmployeeName(claims.get("employeeName",String.class));
                jwtUser.setProjectIds(claims.get("projectIds",ArrayList.class));
            }
            jwtUser.setTenantId((String) claims.get("traceId"));
            return jwtUser;
        } catch (Exception e) {
            log.error("token:{}, 异常message:{}", jwtToken, e.getMessage(),e);
            return null;
        }
    }
}
