package com.gci.cdnev.smallprogram.interceptor;


import java.util.Objects;
import java.util.concurrent.TimeUnit;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.gci.cdnev.smallprogram.modules.wx.service.impl.WxLoginServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;
import org.springframework.web.servlet.HandlerInterceptor;

import com.gci.cdnev.smallprogram.common.util.JsonUtil;
import com.gci.cdnev.smallprogram.common.util.JwtUtils;
import com.gci.cdnev.smallprogram.common.vo.Result;
import com.gci.cdnev.smallprogram.config.JwtConfig;
import com.gci.cdnev.smallprogram.constant.CommonConstant;
import com.gci.cdnev.smallprogram.modules.wx.entity.JwtUser;

import lombok.extern.slf4j.Slf4j;

@Component
@Slf4j
public class JwtInterceptor implements HandlerInterceptor {

    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    @Autowired
    private WxLoginServiceImpl wxLoginService;

    @Autowired
    private JwtConfig jwtConfig;

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        // 根据token获取用户信息
        JwtUser jwtUser = JwtUtils.getUserInfoByToken(request);
        //如果为空的话，则看缓存里是否有，有的话则续期。
        String token = request.getHeader(jwtConfig.getHeader());
        if (jwtUser != null) {
            // 将用户信息user保存到request属性中
            request.setAttribute("current_user", jwtUser);
            wxLoginService.jwtTokenRefresh(token, jwtUser);
            return true;
        }
        String tokenKey = String.join("", CommonConstant.SYS_USER_MOBILE_LOGIN, token);
        String redisToken = redisTemplate.opsForValue().get(tokenKey);
        if (StringUtils.isNotBlank(redisToken)) {
            JwtUser jwtUserRedis = JsonUtil.json2Object(redisToken, JwtUser.class);
            request.setAttribute("current_user", jwtUserRedis);
            redisTemplate.opsForValue().set(tokenKey, redisToken, jwtConfig.getExpireTime() * 2, TimeUnit.MILLISECONDS);
            return true;
        }
        //强制跳转登录
        response.setCharacterEncoding("UTF-8");
        response.setContentType("application/json; charset=utf-8");
        Result result = Result.error(50008, "token失效,请重新登录");
        response.getWriter().write(JsonUtil.object2Json(result));
        return false;
    }
}
