package com.gci.cdnev.smallprogram.interceptor;

import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpStatus;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.support.ServletUriComponentsBuilder;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.nio.charset.StandardCharsets;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Arrays;
import java.util.Base64;
import java.util.Objects;

/**
 * @author lyz
 * @date 2023/6/1 13:47
 */
@Slf4j
public class SignatureInterceptor implements HandlerInterceptor {

    private final static String[] DMZ_PATH={"/api/user/login/","/api/rescue/relEventTaskDispatch"};

    private static final String SIGNATURE_HEADER = "X-Signature";
    private static final String SECRET_KEY = "Gsgl@2023-!";

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        ServletUriComponentsBuilder ucb = ServletUriComponentsBuilder.fromRequest(request);
        String ucbString = ucb.toUriString();
        if(isDMZPath(ucbString)){
            return true;
        }
        String signature = request.getHeader(SIGNATURE_HEADER);
        String queryString = request.getQueryString() == null ? "" : "?" + request.getQueryString();
        String message = request.getMethod() + request.getRequestURI() + queryString + SECRET_KEY;
        String expectedSignature = generateSignature(message);
        if (!Objects.equals(expectedSignature, signature)) {
            log.error("签名信息不一致，签名字符串-{}，原始签名-{}，客户端签名-{}", message, expectedSignature, signature);
            response.setStatus(HttpStatus.UNAUTHORIZED.value());
            return false;
        }
        return true;
    }

    private String generateSignature(String message) throws NoSuchAlgorithmException {
        MessageDigest md = MessageDigest.getInstance("SHA-256");
        byte[] digest = md.digest(message.getBytes(StandardCharsets.UTF_8));
        return Base64.getEncoder().encodeToString(digest);
    }

    public static void main(String[] args) throws NoSuchAlgorithmException {
        String message = "ddddddddddddddddddd";
        MessageDigest md = MessageDigest.getInstance("SHA-256");
        byte[] digest = md.digest(message.getBytes(StandardCharsets.UTF_8));
        System.out.println(Base64.getEncoder().encodeToString(digest));
    }

    private boolean isDMZPath(String uri){
        return Arrays.stream(DMZ_PATH).filter((l)->{
            return uri.indexOf(l)>-1;
        }).findFirst().isPresent();
    }

}
