package com.gci.cdnev.smallprogram.modules.api.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gci.cdnev.smallprogram.annotation.CurrentUser;
import com.gci.cdnev.smallprogram.common.log.annotation.AddSysLog;
import com.gci.cdnev.smallprogram.common.util.FormatUtil;
import com.gci.cdnev.smallprogram.common.vo.Result;
import com.gci.cdnev.smallprogram.modules.gis.entity.TbGisFenceAlarm;
import com.gci.cdnev.smallprogram.modules.gis.entity.TbGisSpeedingAlarm;
import com.gci.cdnev.smallprogram.modules.gis.entity.TbGisStoppingAlarm;
import com.gci.cdnev.smallprogram.modules.monitor.service.IMonitorService;
import com.gci.cdnev.smallprogram.modules.monitor.vo.AlarmCountVO;
import com.gci.cdnev.smallprogram.modules.monitor.vo.OverallMonitorVO;
import com.gci.cdnev.smallprogram.modules.monitor.vo.RunningbusVO;
import com.gci.cdnev.smallprogram.modules.wx.entity.JwtUser;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

@Api(tags = "全局监控")
@Slf4j
@RestController
@RequestMapping("/api/monitor")
public class MonitorController {

    /**
     * 查询车辆列表
     *
     * @param statusType
     * @return
     */
    @AddSysLog("全局监控-查询车辆列表")
    @GetMapping(value = "/vehicle/listByView")
    public Result<?> queryVehicleListByView(HttpServletRequest request,
                                            @CurrentUser JwtUser jwtUser,
                                            String statusType,
                                            @RequestParam(value = "projectId", required = false) String projectId,
                                            @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                            @RequestParam(name = "pageSize", defaultValue = "500") Integer pageSize) {
        List<String> projectIds = new ArrayList<>();
        //判断是否有传项目id，如果有传，则查询所选项目，否则查询用户权限范围内所有项目数据
        if (StringUtils.isNotBlank(projectId)) {
            projectIds.add(projectId);
        } else {
            projectIds.addAll(jwtUser.getProjectIds());
        }
        Page<OverallMonitorVO> page = new Page<>(pageNo, pageSize);
        List<OverallMonitorVO> list = monitorService.getOverallVehicleByListView(page, statusType, projectIds);
        page.setRecords(list);
        return Result.ok(page);
    }


    /**
     * 查询权限范围内所有车辆列表，支持模糊搜索
     *
     * @return
     */
    @AddSysLog("全局监控-查询权限范围内所有车辆列表，支持模糊搜索")
    @GetMapping(value = "/vehicle/listByAll")
    public Result<?> queryVehicleListByAll(HttpServletRequest request,
                                           String projectId,
                                           @CurrentUser JwtUser jwtUser,
                                           String plateNumber) {
        List<String> projectIds = new ArrayList<>();
        //判断是否有传项目id，如果有传，则查询所选项目，否则查询用户权限范围内所有项目数据
        if (StringUtils.isNotBlank(projectId)) {
            projectIds.add(projectId);
        } else {
            projectIds.addAll(jwtUser.getProjectIds());
        }
        List<OverallMonitorVO> list = monitorService.getOverallVehicleByAll(plateNumber, projectIds);
        return Result.ok(list);
    }

    /**
     * 查询车辆列表 用于地图
     *
     * @param statusType
     * @return
     */
    @AddSysLog("全局监控-查询车辆列表 用于地图")
    @GetMapping(value = "/vehicle/listByMap")
    public Result<?> queryVehicleListByMap(HttpServletRequest request,
                                           @CurrentUser JwtUser jwtUser,
                                           String statusType,
                                           @RequestParam(value = "projectId", required = false) String projectId,
                                           @RequestParam(name = "lon", defaultValue = "104.062353") double lon,
                                           @RequestParam(name = "lat", defaultValue = "30.658941") double lat,
                                           @RequestParam(name = "mapLevel", defaultValue = "3.00") double mapLevel) {
        List<String> projectIds = new ArrayList<>();
        //判断是否有传项目id，如果有传，则查询所选项目，否则查询用户权限范围内所有项目数据
        if (StringUtils.isNotBlank(projectId)) {
            projectIds.add(projectId);
        } else {
            projectIds.addAll(jwtUser.getProjectIds());
        }
        List<OverallMonitorVO> list = monitorService.getOverallVehicleByMap(statusType, lon, lat, mapLevel, projectIds);
        return Result.ok(list);
    }


    /**
     * 获取各种状态下的车辆数量
     *
     * @return
     */
    @AddSysLog("全局监控-获取各种状态下的车辆数量")
    @GetMapping(value = "/vehicle/count")
    public Result<?> getVehicleCount(HttpServletRequest request,
                                     @RequestParam(value = "projectId", required = false) String projectId,
                                     @CurrentUser JwtUser jwtUser) {
        List<String> projectIds = new ArrayList<>();
        //判断是否有传项目id，如果有传，则查询所选项目，否则查询用户权限范围内所有项目数据
        if (StringUtils.isNotBlank(projectId)) {
            projectIds.add(projectId);
        } else {
            projectIds.addAll(jwtUser.getProjectIds());
        }
        List<AlarmCountVO> list = monitorService.getVehicleCount(projectIds);
        return Result.ok(list);
    }

    /**
     * 实时监控--获取车辆详情
     *
     * @return
     */
    @AddSysLog("实时监控-获取车辆详情")
    @GetMapping(value = "/vehicle/realInfo")
    public Result<?> getVehicleRealInfo(HttpServletRequest request, String plateNumber) {
        if (StringUtils.isBlank(plateNumber)) {
            return Result.error(101, "无效参数");
        }
        RunningbusVO vo = monitorService.getRunningbusInfo(plateNumber);
        return Result.ok(vo);
    }


    /**
     * 实时监控-获取电子围栏告警信息列表
     *
     * @return
     */
    @AddSysLog("实时监控-获取电子围栏告警信息列表")
    @GetMapping(value = "/fenceAlarm/list")
    public Result<?> getFenceAlarmList(HttpServletRequest request,
                                       @CurrentUser JwtUser jwtUser,
                                       @RequestParam(value = "projectId", required = false) String projectId,
                                       @RequestParam(value = "startTime", required = false) String startTime,
                                       @RequestParam(value = "endTime", required = false) String endTime,
                                       @RequestParam(value = "plateNumber", required = false) String plateNumber,
                                       @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                       @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                       @RequestParam(name = "status", defaultValue = "10") Integer status) {
        List<String> projectIds = new ArrayList<>();
        //判断是否有传项目id，如果有传，则查询所选项目，否则查询用户权限范围内所有项目数据
        if (StringUtils.isNotBlank(projectId)) {
            projectIds.add(projectId);
        } else {
            projectIds.addAll(jwtUser.getProjectIds());
        }
        IPage<TbGisFenceAlarm> pageList = monitorService.pageFenceList(projectIds, plateNumber, startTime, endTime, status, pageNo, pageSize);
        for (TbGisFenceAlarm record : pageList.getRecords()) {
            String duration = FormatUtil.formattedTimeDifference(record.getStartTime(), record.getEndTime());
            record.setDurTime(duration);
            if(Objects.nonNull(record.getEndTime())){
                record.setStatus(1);
            }else{
                record.setStatus(0);
            }
        }
        return Result.ok(pageList);
    }

    /**
     * 实时监控-获取超速告警信息列表
     *
     * @return
     */
    @AddSysLog("实时监控-获取超速告警信息列表")
    @GetMapping(value = "/speedAlarm/list")
    public Result<?> getSpeedAlarmList(HttpServletRequest request, @CurrentUser JwtUser jwtUser,
                                       @RequestParam(value = "projectId", required = false) String projectId,
                                       @RequestParam(value = "startTime", required = false) String startTime,
                                       @RequestParam(value = "endTime", required = false) String endTime,
                                       @RequestParam(value = "plateNumber", required = false) String plateNumber,
                                       @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                       @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                       @RequestParam(name = "status", defaultValue = "10") Integer status) {

        List<String> projectIds = new ArrayList<>();
        //判断是否有传项目id，如果有传，则查询所选项目，否则查询用户权限范围内所有项目数据
        if (StringUtils.isNotBlank(projectId)) {
            projectIds.add(projectId);
        } else {
            projectIds.addAll(jwtUser.getProjectIds());
        }
        IPage<TbGisSpeedingAlarm> speedingAlarms = monitorService.pageSpeedAlarmList(projectIds, plateNumber, startTime, endTime, status, pageNo, pageSize);
        for (TbGisSpeedingAlarm record : speedingAlarms.getRecords()) {
            String duration = FormatUtil.formattedTimeDifference(record.getStartTime(), record.getEndTime());
            record.setDurTime(duration);
            if(Objects.nonNull(record.getEndTime())){
                record.setStatus(1);
            }else{
                record.setStatus(0);
            }
        }
        return Result.ok(speedingAlarms);
    }

    /**
     * 实时监控-获取超速告警信息列表
     *
     * @return
     */
    @AddSysLog("实时监控-获取超速告警信息列表")
    @GetMapping(value = "/stopAlarm/list")
    public Result<?> getStopAlarmList(HttpServletRequest request, @CurrentUser JwtUser jwtUser,
                                       @RequestParam(value = "projectId", required = false) String projectId,
                                       @RequestParam(value = "startTime", required = false) String startTime,
                                       @RequestParam(value = "endTime", required = false) String endTime,
                                       @RequestParam(value = "plateNumber", required = false) String plateNumber,
                                       @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                       @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                       @RequestParam(name = "status", defaultValue = "10") Integer status) {

        List<String> projectIds = new ArrayList<>();
        //判断是否有传项目id，如果有传，则查询所选项目，否则查询用户权限范围内所有项目数据
        if (StringUtils.isNotBlank(projectId)) {
            projectIds.add(projectId);
        } else {
            projectIds.addAll(jwtUser.getProjectIds());
        }
        IPage<TbGisStoppingAlarm> stoppingAlarms = monitorService.pageStopAlarmList(projectIds, plateNumber, startTime, endTime, status, pageNo, pageSize);
        for (TbGisStoppingAlarm record : stoppingAlarms.getRecords()) {
            String duration = FormatUtil.formattedTimeDifference(record.getStartTime(), record.getEndTime());
            record.setDurTime(duration);
            if(Objects.nonNull(record.getEndTime())){
                record.setStatus(1);
            }else{
                record.setStatus(0);
            }
        }
        return Result.ok(stoppingAlarms);
    }

    @Autowired
    private IMonitorService monitorService;
}
