package com.gci.cdnev.smallprogram.modules.basedata.controller;


import java.util.ArrayList;
import java.util.Comparator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import com.gci.cdnev.smallprogram.common.log.annotation.AddSysLog;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gci.cdnev.smallprogram.annotation.Query.handle.QueryGenerator;
import com.gci.cdnev.smallprogram.common.vo.Result;
import com.gci.cdnev.smallprogram.constant.CommonConstant;
import com.gci.cdnev.smallprogram.modules.basedata.entity.TbExternalAddressBook;
import com.gci.cdnev.smallprogram.modules.basedata.service.ITbExternalAddressBookService;

import lombok.extern.slf4j.Slf4j;
import net.sourceforge.pinyin4j.PinyinHelper;
import net.sourceforge.pinyin4j.format.HanyuPinyinCaseType;
import net.sourceforge.pinyin4j.format.HanyuPinyinOutputFormat;
import net.sourceforge.pinyin4j.format.HanyuPinyinToneType;
import net.sourceforge.pinyin4j.format.HanyuPinyinVCharType;
import net.sourceforge.pinyin4j.format.exception.BadHanyuPinyinOutputFormatCombination;

/**
 * <p>
 *  外部通讯录 前端控制器
 * </p>
 *
 * @author xushuhan
 * @since 2023-08-24
 */
@Slf4j
@RestController
@RequestMapping("/api/externalAddressBook")
public class TbExternalAddressBookController {

	@Autowired
	private ITbExternalAddressBookService tbExternalAddressBookService;


	@AddSysLog(value = "外部通讯录-列表查询")
	@GetMapping("/list")
	public Result<?> list(TbExternalAddressBook item) throws BadHanyuPinyinOutputFormatCombination {
		try {
			LambdaQueryWrapper<TbExternalAddressBook> queryWrapper = QueryGenerator.initQueryWrapper(TbExternalAddressBook.class, item);
			queryWrapper.eq(TbExternalAddressBook::getDelFlag, CommonConstant.DEL_FLAG_0);
			queryWrapper.orderByDesc(TbExternalAddressBook::getUserName);
			List<TbExternalAddressBook> list = tbExternalAddressBookService.list(queryWrapper);
			//t3是全部的拼音，不带声调
			HanyuPinyinOutputFormat t3 = new HanyuPinyinOutputFormat();
			t3.setCaseType(HanyuPinyinCaseType.UPPERCASE);
			t3.setToneType(HanyuPinyinToneType.WITHOUT_TONE);
			t3.setVCharType(HanyuPinyinVCharType.WITH_V);
			for (TbExternalAddressBook book : list) {
				book.setUserNamePy(PinyinHelper.toHanYuPinyinString(book.getUserName(),t3,"",true));
				book.setUserNameFrist(book.getUserNamePy().toCharArray()[0]+"");
			}
			list.sort(new Comparator<TbExternalAddressBook>() {//使用List接口的方法排序
	            @Override
	            public int compare(TbExternalAddressBook o1, TbExternalAddressBook o2) {
	                return o1.getUserNamePy().compareTo(o2.getUserNamePy());
	            }
	        });
			Map<String,List<TbExternalAddressBook>> map = new LinkedHashMap<>();
			for (TbExternalAddressBook book : list) {
				List<TbExternalAddressBook> temps = map.getOrDefault(book.getUserNameFrist(), new ArrayList<>());
				temps.add(book);
				map.put(book.getUserNameFrist(), temps);
			}
			return Result.ok(map);
		} catch (Exception e) {
			log.error("获取通讯录失败", e);
			return Result.error("获取通讯录失败");
		}
        
	}

	@AddSysLog(value = "外部通讯录-分页查询")
	@GetMapping("/page")
	public Result<?> page(TbExternalAddressBook item,@RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
						  @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize) {
        LambdaQueryWrapper<TbExternalAddressBook> queryWrapper = QueryGenerator.initQueryWrapper(TbExternalAddressBook.class, item);
		queryWrapper.eq(TbExternalAddressBook::getDelFlag, CommonConstant.DEL_FLAG_0);
		queryWrapper.orderByDesc(TbExternalAddressBook::getUserName);
		Page<TbExternalAddressBook> page = new Page<TbExternalAddressBook>(pageNo, pageSize);
		IPage<TbExternalAddressBook> pageList = tbExternalAddressBookService.page(page,queryWrapper);
		return Result.ok(pageList);
	}

}
