package com.gci.cdnev.smallprogram.modules.basedata.controller;


import java.util.List;

import com.gci.cdnev.smallprogram.common.log.annotation.AddSysLog;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gci.cdnev.smallprogram.annotation.Query.handle.QueryGenerator;
import com.gci.cdnev.smallprogram.common.vo.Result;
import com.gci.cdnev.smallprogram.constant.CommonConstant;
import com.gci.cdnev.smallprogram.modules.basedata.entity.TbVehicleCheckTemplate;
import com.gci.cdnev.smallprogram.modules.basedata.service.ITbVehicleCheckTemplateProjectService;
import com.gci.cdnev.smallprogram.modules.basedata.service.ITbVehicleCheckTemplateService;

import lombok.extern.slf4j.Slf4j;

/**
 * <p>
 * 车辆验收模板 前端控制器
 * </p>
 *
 * @author xushuhan
 * @since 2023-09-07
 */
@Slf4j
@RestController
@RequestMapping("/api/vehicleCheckTemplate")
public class TbVehicleCheckTemplateController {

	@Autowired
	private ITbVehicleCheckTemplateService tbVehicleCheckTemplateService;
	@Autowired
	private ITbVehicleCheckTemplateProjectService tbVehicleCheckTemplateProjectService;


	@AddSysLog(value = "车辆验收模板-列表查询")
	@GetMapping("/list")
	public Result<?> list(TbVehicleCheckTemplate item) {
		if(!StringUtils.hasLength(item.getCheckType())) {
			item.setCheckType("1");
		}
		LambdaQueryWrapper<TbVehicleCheckTemplate> queryWrapper = QueryGenerator.initQueryWrapper(TbVehicleCheckTemplate.class, item);
		//通过类型区分不同模板
		queryWrapper.eq(TbVehicleCheckTemplate::getCheckType, item.getCheckType());
		queryWrapper.eq(TbVehicleCheckTemplate::getDelFlag, CommonConstant.DEL_FLAG_0);
		queryWrapper.orderByDesc(TbVehicleCheckTemplate::getCreateTime);
		List<TbVehicleCheckTemplate> list = tbVehicleCheckTemplateService.list(queryWrapper);
		/*for(TbVehicleCheckTemplate template:list) {
			tbVehicleCheckTemplateProjectService.getProjectsByTemplateId(template);
		}*/
		return Result.ok(list);
	}

	@AddSysLog(value = "车辆验收模板-分页查询")
	@GetMapping("/page")
	public Result<?> page(TbVehicleCheckTemplate item,@RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
						  @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize) {
		if(!StringUtils.hasLength(item.getCheckType())) {
			item.setCheckType("1");
		}
		LambdaQueryWrapper<TbVehicleCheckTemplate> queryWrapper = QueryGenerator.initQueryWrapper(TbVehicleCheckTemplate.class, item);
		//通过类型区分不同模板
		queryWrapper.eq(TbVehicleCheckTemplate::getCheckType, item.getCheckType());
		queryWrapper.eq(TbVehicleCheckTemplate::getDelFlag, CommonConstant.DEL_FLAG_0);
		queryWrapper.orderByDesc(TbVehicleCheckTemplate::getCreateTime);
		Page<TbVehicleCheckTemplate> page = new Page<TbVehicleCheckTemplate>(pageNo, pageSize);
		IPage<TbVehicleCheckTemplate> pageList = tbVehicleCheckTemplateService.page(page,queryWrapper);
		/*for(TbVehicleCheckTemplate template:pageList.getRecords()) {
			tbVehicleCheckTemplateProjectService.getProjectsByTemplateId(template);
		}*/
		return Result.ok(pageList);
	}

}
