package com.gci.cdnev.smallprogram.modules.basedata.controller;


import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import cn.hutool.core.collection.CollectionUtil;
import com.gci.cdnev.smallprogram.common.log.annotation.AddSysLog;
import com.gci.cdnev.smallprogram.modules.projects.service.IProjectsService;
import com.gci.cdnev.smallprogram.modules.vehicles.entity.TbRefuelChargeRecord;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gci.cdnev.smallprogram.annotation.CurrentUser;
import com.gci.cdnev.smallprogram.annotation.Query.handle.QueryGenerator;
import com.gci.cdnev.smallprogram.common.util.MinioUtils;
import com.gci.cdnev.smallprogram.common.vo.Result;
import com.gci.cdnev.smallprogram.constant.CommonConstant;
import com.gci.cdnev.smallprogram.modules.basedata.entity.TbVehicle;
import com.gci.cdnev.smallprogram.modules.basedata.query.ProjectVehicleListQuery;
import com.gci.cdnev.smallprogram.modules.basedata.service.ITbVehicleService;
import com.gci.cdnev.smallprogram.modules.wx.entity.JwtUser;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author 2464
 * @since 2023-08-25
 */
@RestController
@RequestMapping("/api/vehicle")
public class TbVehicleController {
    @Autowired
    private ITbVehicleService iTbVehicleService;
	@Autowired
	private IProjectsService projectsService;
    @Autowired
    private MinioUtils minioUtils;

	@AddSysLog("车辆档案-根据项目查询")
	@GetMapping("/listByProjectId")
	public Result<?> listByProjectId(@CurrentUser JwtUser jwtUser,TbVehicle item) {
		if(!StringUtils.hasLength(item.getProjectId())) {
			return Result.ok();
		}
		LambdaQueryWrapper<TbVehicle> queryWrapper = QueryGenerator.initQueryWrapper(TbVehicle.class, item);
		queryWrapper.eq(TbVehicle::getCancelFlag, CommonConstant.DEL_FLAG_0);
		queryWrapper.orderByDesc(TbVehicle::getCreateTime);
		List<TbVehicle> list = iTbVehicleService.list(queryWrapper);
		return Result.ok(list);
	}

	@AddSysLog("车辆档案-获取有权限的车辆列表")
	@GetMapping("/getVehicleList")
	public Result<?> getVehicleList(@CurrentUser JwtUser jwtUser,TbVehicle item) {
		//数据权限
		Integer departCategory = projectsService.judgeDepartCategory(jwtUser);
		//超级管理员查询全部数据
		List<String> projectIds = new ArrayList<>();
		if(departCategory != 5){
			projectIds = projectsService.queryProjectListOfUser(jwtUser);
			if(CollectionUtil.isEmpty(projectIds) || projectIds.size() == 0 ){
				projectIds.add("no permission");
			}
		}
		LambdaQueryWrapper<TbVehicle> queryWrapper = QueryGenerator.initQueryWrapper(TbVehicle.class, item);
		queryWrapper.eq(TbVehicle::getCancelFlag, CommonConstant.DEL_FLAG_0);
		if(CollectionUtil.isNotEmpty(projectIds) && projectIds.size()>0){
			queryWrapper.in(TbVehicle::getProjectId, projectIds);
		}
		queryWrapper.orderByDesc(TbVehicle::getCreateTime);
		List<TbVehicle> list = iTbVehicleService.list(queryWrapper);
		return Result.ok(list);
	}
	/**
	 * 分页列表查询
	 *
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@AddSysLog("车辆档案-分页列表查询")
	@GetMapping(value = "/list")
	public Result<?> queryPageList(TbVehicle tbVehicle,
								   @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
								   @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
								   HttpServletRequest req) {
		Page<TbVehicle> page = new Page<TbVehicle>(pageNo, pageSize);
		LambdaQueryWrapper<TbVehicle> lambdaQueryWrapper = QueryGenerator.initQueryWrapper(TbVehicle.class, tbVehicle);
		IPage<TbVehicle> pageList = iTbVehicleService.page(page, lambdaQueryWrapper);
		return Result.ok(pageList);
	}
}
