package com.gci.cdnev.smallprogram.modules.basedata.entity;

import com.alibaba.excel.annotation.ExcelIgnore;
import com.alibaba.excel.annotation.ExcelProperty;
import com.alibaba.excel.annotation.write.style.ColumnWidth;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.gci.cdnev.smallprogram.common.config.CancelConverter;
import com.gci.cdnev.smallprogram.common.validated.SaveValidated;
import com.gci.cdnev.smallprogram.common.validated.UpdateValidated;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;
import org.hibernate.validator.constraints.Length;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.Pattern;
import java.io.Serializable;
import java.util.Date;

@Data
@TableName("tb_customer_archives")
@Accessors(chain = true)
@EqualsAndHashCode(callSuper = false)
@ApiModel(value="客户档案表对象", description="客户档案表对象")
public class TbCustomerArchives implements Serializable {
    private static final long serialVersionUID = 1L;
    /**主键id*/
    @TableId(type = IdType.ASSIGN_ID)
    @ApiModelProperty(value = "主键id")
    @ExcelIgnore
    private String id;
    @NotBlank(message = "全称不能为空",groups = {SaveValidated.class, UpdateValidated.class})
    @Length(max = 128,groups = {SaveValidated.class,UpdateValidated.class},message = "全称长度不能超过128")
    @ApiModelProperty(value = "全称")
    @ExcelProperty("全称")
    @ColumnWidth(value = 30)
    private String name;
    @NotBlank(message = "简称不能为空",groups = {SaveValidated.class, UpdateValidated.class})
    @Length(max = 32,groups = {SaveValidated.class,UpdateValidated.class},message = "简称长度不能超过32")
    @ApiModelProperty(value = "简称")
    @ExcelProperty("简称")
    @ColumnWidth(value = 15)
    private String shortened;
    @ApiModelProperty(value = "联系人")
    @NotBlank(message = "联系人不能为空！",groups = {SaveValidated.class,UpdateValidated.class})
    @Length(max = 32,groups = {SaveValidated.class,UpdateValidated.class},message = "联系人长度不能超过32")
    @ExcelProperty("联系人")
    @ColumnWidth(value = 15)
    private String contact;
    @ApiModelProperty(value = "联系电话")
    @NotBlank(message = "联系电话不能为空！",groups = {SaveValidated.class,UpdateValidated.class})
    @Pattern(regexp = "^(?:(?:\\+|00)86)?1\\d{10}$",message = "联系电话格式错误!",groups = {SaveValidated.class,UpdateValidated.class})
    @ExcelProperty("联系电话")
    @ColumnWidth(value = 15)
    private String phone;
    @ApiModelProperty(value = "地址")
    @NotBlank(message = "地址不能为空！",groups = {SaveValidated.class,UpdateValidated.class})
    @Length(max = 128,groups = {SaveValidated.class,UpdateValidated.class},message = "地址长度不能超过128")
    @ExcelProperty("地址")
    @ColumnWidth(value = 30)
    private String address;
    /**所属机构*/
    @ApiModelProperty(value = "所属部门")
    @ExcelIgnore
    private String sysOrgCode;
    /**创建日期*/
    @JsonFormat(timezone = "GMT+8",pattern = "yyyy-MM-dd HH:mm:ss")
    @DateTimeFormat(pattern="yyyy-MM-dd HH:mm:ss")
    @ApiModelProperty(value = "创建日期")
    @ExcelIgnore
    private Date createTime;
    /**更新日期*/
    @JsonFormat(timezone = "GMT+8",pattern = "yyyy-MM-dd HH:mm:ss")
    @DateTimeFormat(pattern="yyyy-MM-dd HH:mm:ss")
    @ApiModelProperty(value = "更新日期")
    @ExcelIgnore
    private Date updateTime;
    /**创建人*/
    @ApiModelProperty(value = "创建人")
    @ExcelIgnore
    private String createBy;
    /**更新人*/
    @ApiModelProperty(value = "更新人")
    @ExcelIgnore
    private String updateBy;
    /**作废标志--默认0-正常，1-作废*/
    @ApiModelProperty(value = "作废标志")
    @ExcelProperty(value = "数据状态", converter = CancelConverter.class)
    @ColumnWidth(value = 15)
    private Integer cancelFlag;
}
