package com.gci.cdnev.smallprogram.modules.basedata.entity;

import com.alibaba.excel.annotation.ExcelIgnore;
import com.alibaba.excel.annotation.ExcelProperty;
import com.baomidou.mybatisplus.annotation.*;

import java.io.Serializable;
import java.util.Date;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonPropertyDescription;
import com.gci.cdnev.smallprogram.annotation.Query.OrderQuery;
import com.gci.cdnev.smallprogram.annotation.Query.SelectQuery;
import com.gci.cdnev.smallprogram.annotation.repeat.FieldRepeat;
import com.gci.cdnev.smallprogram.annotation.repeat.TableRepeat;
import com.gci.cdnev.smallprogram.common.dict.annotation.Dict;
import com.gci.cdnev.smallprogram.common.dict.excel.DictConverter;
import com.gci.cdnev.smallprogram.common.validated.SaveValidated;
import com.gci.cdnev.smallprogram.common.validated.UpdateValidated;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.hibernate.validator.constraints.Length;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.Max;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Pattern;

/**
 * <p>
 *
 * </p>
 *
 * @author 2464
 * @since 2023-08-25
 */
@Data
@TableName("tb_vehicle")
@ApiModel(value = "TbVehicle对象", description = "")
@TableRepeat(aClass = TbVehicle.class,groups = {SaveValidated.class, UpdateValidated.class})
public class TbVehicle implements Serializable {

    private static final long serialVersionUID = 1L;

    @ApiModelProperty("主键id")
    @TableId(value = "id", type = IdType.ASSIGN_ID)
    @NotBlank(message = "id不能为空！",groups = {UpdateValidated.class})
    @ExcelIgnore
    private String id;

    @ApiModelProperty("所属项目")
    @SelectQuery
    @Length(max = 32,message = "所属项目数据错误!",groups = {SaveValidated.class,UpdateValidated.class})
    @Dict(dicCode = "id",dictTable = "tb_projects",dicText = "project_name")
    @ExcelProperty(value = "所属项目",converter = DictConverter.class)
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private String projectId;

    @ApiModelProperty("车牌号")
    @NotBlank(message = "车牌号不能为空！",groups = {SaveValidated.class,UpdateValidated.class})
    @FieldRepeat(message = "车牌号码不能重复！")
    @SelectQuery
    @Pattern(message = "车牌号格式错误",regexp = "^[京津沪渝冀豫云辽黑湘皖鲁新苏浙赣鄂桂甘晋蒙陕吉闽贵粤青藏川宁琼使领][A-HJ-NP-Z][A-HJ-NP-Z0-9]{4,5}[A-HJ-NP-Z0-9挂学警港澳]$",groups = {SaveValidated.class,UpdateValidated.class})
    @ExcelProperty(value = "车牌号")
    private String plateNumber;

    @ApiModelProperty("车架号")
    @NotBlank(message = "车架号不能为空！",groups = {SaveValidated.class,UpdateValidated.class})
    @SelectQuery
    @FieldRepeat(message = "车架号不能重复!")
    @Length(max = 20,message = "车架号不能超过20位",groups = {SaveValidated.class,UpdateValidated.class})
    @ExcelProperty(value = "车架号")
    private String frameNumber;

    @ApiModelProperty("所属车商")
    @NotBlank(message = "所属车商不能为空！",groups = {SaveValidated.class,UpdateValidated.class})
    @Length(max = 32,message = "所属车商数据错误！",groups = {SaveValidated.class,UpdateValidated.class})
    @Dict(dicCode = "id",dictTable = "tb_vehicle_supplier_archives",dicText = "name")
    @ExcelProperty(value = "所属车商",converter = DictConverter.class)
    private String dealer;

    @ApiModelProperty("交付时间")
    @JsonFormat(timezone = "GMT+8",pattern = "yyyy-MM-dd")
    @DateTimeFormat(pattern="yyyy-MM-dd")
    @NotNull(message = "交付时间不能为空!",groups = {SaveValidated.class,UpdateValidated.class})
    @SelectQuery(Type = SelectQuery.SelectType.EQ)
    @com.alibaba.excel.annotation.format.DateTimeFormat("yyyy年MM月dd日HH时mm分ss秒")
    @ExcelProperty(value = "交付时间")
    private Date deliveryTime;

    @ApiModelProperty("车辆品牌")
    @NotBlank(message = "车辆品牌不能为空！",groups = {SaveValidated.class,UpdateValidated.class})
    @Dict(dicCode = "vehicle_brand")
    @ExcelProperty(value = "车辆品牌",converter = DictConverter.class)
    private String vehicleBrand;

    @ApiModelProperty("车辆型号")
    @NotBlank(message = "车辆型号不能为空！",groups = {SaveValidated.class,UpdateValidated.class})
    @ExcelProperty(value = "车辆型号")
    private String vehicleModel;


    @ApiModelProperty("车辆种类")
    @NotBlank(message = "车辆种类不能为空！",groups = {SaveValidated.class,UpdateValidated.class})
    @Dict(dicCode = "vehicle_type")
    @ExcelProperty(value = "车辆种类",converter = DictConverter.class)
    @SelectQuery(Type = SelectQuery.SelectType.EQ)
    private String vehicleType;

    @ApiModelProperty("车辆识别代码（VIN码）")
    @NotBlank(message = "VIN码不能为空",groups = {SaveValidated.class,UpdateValidated.class})
    @FieldRepeat(message = "VIN码不能重复！")
    @Length(max = 20,message = "VIN码不能超过20位！",groups = {SaveValidated.class,UpdateValidated.class})
    @ExcelIgnore
    private String vinCode;

    @ApiModelProperty("发动机号码")
    @NotBlank(message = "发动机号码不能为空！",groups = {SaveValidated.class,UpdateValidated.class})
    @Length(max = 20,message = "发动机号码不能超过20位！",groups = {SaveValidated.class,UpdateValidated.class})
    @ExcelIgnore
    private String engineNumber;

    @ApiModelProperty("车身颜色")
    @Dict(dicCode = "body_color")
    @ExcelIgnore
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private String bodyColor;

    @ApiModelProperty("燃料类型")
    @Dict(dicCode = "fuel_type")
    @ExcelIgnore
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private String fuelType;

    @ApiModelProperty("纯电续航里程（单位：公里）")
    @Max(value = 2147483647,message = "续航里程不能超过2147483647！",groups = {SaveValidated.class,UpdateValidated.class})
    @ExcelIgnore
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private Integer electricRange;

    @ApiModelProperty("电池容量（单位：千瓦时）")
    @Max(value = 2147483647,message = "电池容量不能超过2147483647！",groups = {SaveValidated.class,UpdateValidated.class})
    @ExcelIgnore
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private Integer batteryCapacity;

    @ApiModelProperty("主机设备号")
    @FieldRepeat(message = "主机设备号不能重复!")
    @Length(max = 20,message = "主机设备号不能超过20位！",groups = {SaveValidated.class,UpdateValidated.class})
    @ExcelIgnore
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private String hostDeviceNumber;

    @ApiModelProperty("注册日期")
    @JsonFormat(timezone = "GMT+8",pattern = "yyyy-MM-dd")
    @DateTimeFormat(pattern="yyyy-MM-dd")
    @ExcelIgnore
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private Date registrationDate;

    @ApiModelProperty("发证日期")
    @JsonFormat(timezone = "GMT+8",pattern = "yyyy-MM-dd")
    @DateTimeFormat(pattern="yyyy-MM-dd")
    @ExcelIgnore
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private Date issuanceDate;

    @ApiModelProperty("年审有效期")
    @JsonFormat(timezone = "GMT+8",pattern = "yyyy-MM-dd")
    @DateTimeFormat(pattern="yyyy-MM-dd")
    @ExcelIgnore
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private Date annualInspectionPeriod;

    @ApiModelProperty("使用性质")
    @Dict(dicCode = "usage_nature")
    @ExcelIgnore
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private String usageNature;

    @ApiModelProperty("定位终端ID")
    @NotBlank(message = "定位终端ID不能为空!",groups = {SaveValidated.class,UpdateValidated.class})
    @FieldRepeat(message = "定位终端ID不能重复！")
    @Length(max = 20,message = "定位终端ID不能超过20位！",groups = {SaveValidated.class,UpdateValidated.class})
    @ExcelIgnore
    private String locationTerminalId;

    @ApiModelProperty("主动安全设备ID")
    @Length(max = 20,message = "主动安全设备ID不能超过20位！",groups = {SaveValidated.class,UpdateValidated.class})
    @ExcelIgnore
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private String activeSafetyDeviceId;

    @ApiModelProperty("视频设备ID")
    @Length(max = 20,message = "视频设备ID不能超过20位！",groups = {SaveValidated.class,UpdateValidated.class})
    @ExcelIgnore
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private String videoDeviceId;

    @ApiModelProperty("备注")
    @ExcelIgnore
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private String remark;

    @ApiModelProperty("车辆照片的url地址")
    @Length(max = 128,message = "车辆照片文件名过长!",groups = {SaveValidated.class,UpdateValidated.class})
    @ExcelIgnore
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private String vehiclePhoto;

    @ApiModelProperty("行驶证照片的url地址")
    @Length(max = 128,message = "行驶证文件名过长!",groups = {SaveValidated.class,UpdateValidated.class})
    @ExcelIgnore
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private String drivingLicensePhoto;

    @ApiModelProperty("所属机构")
    @ExcelIgnore
    private String sysOrgCode;

    @ApiModelProperty("创建时间")
    @JsonFormat(timezone = "GMT+8",pattern = "yyyy-MM-dd HH:mm:ss")
    @DateTimeFormat(pattern="yyyy-MM-dd HH:mm:ss")
    @OrderQuery(sort = 1)
    @ExcelIgnore
    private Date createTime;

    @ApiModelProperty("更新时间")
    @JsonFormat(timezone = "GMT+8",pattern = "yyyy-MM-dd HH:mm:ss")
    @DateTimeFormat(pattern="yyyy-MM-dd HH:mm:ss")
    @ExcelIgnore
    private Date updateTime;

    @ApiModelProperty("创建人")
    @ExcelIgnore
    private String createBy;

    @ApiModelProperty("更新人")
    @ExcelIgnore
    private String updateBy;

    @ApiModelProperty("作废标志")
    @Dict(dicCode = "cancel_flag")
    @ExcelProperty(value = "状态",converter = DictConverter.class)
    private Integer cancelFlag;
    
    @ApiModelProperty("来源（1车辆档案，2采购交车）")
    @ExcelIgnore
    private String sourceType;

    @JsonPropertyDescription("油卡卡号")
    @SelectQuery(Type = SelectQuery.SelectType.LIKE)
    @Length(max = 32,message = "油卡卡号不能超过32位！",groups = {SaveValidated.class,UpdateValidated.class})
    @ExcelIgnore
    private String oilCard;

}
