package com.gci.cdnev.smallprogram.modules.customer.controller;


import java.io.IOException;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gci.cdnev.smallprogram.annotation.Query.handle.QueryGenerator;
import com.gci.cdnev.smallprogram.common.log.annotation.AddSysLog;
import com.gci.cdnev.smallprogram.common.util.DateUtils;
import com.gci.cdnev.smallprogram.common.util.MinioUtils;
import com.gci.cdnev.smallprogram.common.validated.SaveValidated;
import com.gci.cdnev.smallprogram.common.validated.UpdateValidated;
import com.gci.cdnev.smallprogram.common.vo.Result;
import com.gci.cdnev.smallprogram.enums.FileExtension;
import com.gci.cdnev.smallprogram.modules.customer.entity.TbCustomerFollow;
import com.gci.cdnev.smallprogram.modules.customer.service.ITbCustomerFollowPartService;
import com.gci.cdnev.smallprogram.modules.customer.service.ITbCustomerFollowService;
import com.gci.cdnev.smallprogram.modules.customer.service.ITbCustomerOpinionService;

import cn.hutool.core.util.ObjectUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * <p>
 * 客户回访台账 前端控制器
 * </p>
 *
 * @author xushuhan
 * @since 2023-10-14
 */
@Slf4j
@RestController
@RequestMapping("/api/customerFollow")
public class TbCustomerFollowController {

	@Autowired
	private ITbCustomerFollowService tbCustomerFollowService;
	@Autowired
	private ITbCustomerFollowPartService tbCustomerFollowPartService;
	@Autowired
	private ITbCustomerOpinionService tbCustomerOpinionService;
	@Autowired
    private MinioUtils minioUtils;

	@AddSysLog(value = "客户回访台账-列表查询")
	@GetMapping("/list")
	public Result<?> list(TbCustomerFollow item) {
		LambdaQueryWrapper<TbCustomerFollow> queryWrapper = QueryGenerator.initQueryWrapper(TbCustomerFollow.class, item);
		queryWrapper.ge(item.getReviewTimeStart()!=null,TbCustomerFollow::getReviewTime, item.getReviewTimeStart());
		queryWrapper.le(item.getReviewTimeEnd()!=null,TbCustomerFollow::getReviewTime, DateUtils.getDayEnd(item.getReviewTimeEnd()));
		queryWrapper.orderByDesc(TbCustomerFollow::getCreateTime);
		List<TbCustomerFollow> list = tbCustomerFollowService.list(queryWrapper);
		tbCustomerOpinionService.initHasOpinion(list);
		tbCustomerFollowService.initHasInspectionTask(list);
		return Result.ok(list);
	}

	@AddSysLog(value = "客户回访台账-分页查询")
	@GetMapping("/page")
	public Result<?> page(TbCustomerFollow item,@RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
						  @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize) {
		LambdaQueryWrapper<TbCustomerFollow> queryWrapper = QueryGenerator.initQueryWrapper(TbCustomerFollow.class, item);
		queryWrapper.ge(item.getReviewTimeStart()!=null,TbCustomerFollow::getReviewTime, item.getReviewTimeStart());
		queryWrapper.le(item.getReviewTimeEnd()!=null,TbCustomerFollow::getReviewTime, DateUtils.getDayEnd(item.getReviewTimeEnd()));
		queryWrapper.orderByDesc(TbCustomerFollow::getCreateTime);
		Page<TbCustomerFollow> page = new Page<TbCustomerFollow>(pageNo, pageSize);
		IPage<TbCustomerFollow> pageList = tbCustomerFollowService.page(page,queryWrapper);
		for(TbCustomerFollow template:pageList.getRecords()) {
			tbCustomerFollowPartService.getPartsByFollowId(template);
			tbCustomerOpinionService.getOpinionByFollowId(template);
			tbCustomerFollowService.getInspectionTaskByFollowId(template);
		}
		return Result.ok(pageList);
	}

	@AddSysLog(value = "客户回访台账-新增")
	@PostMapping("/add")
	public Result<?> add(@RequestBody @Validated(SaveValidated.class) TbCustomerFollow item){
		item.setSourceType("1");
		tbCustomerFollowService.saveItem(item);
		return Result.ok();
	}

	@AddSysLog(value = "客户回访台账-编辑")
	@PutMapping("/edit")
	public Result<?> edit(@RequestBody @Validated(UpdateValidated.class) TbCustomerFollow item){
		item.setSourceType("1");
		tbCustomerFollowService.saveItem(item);
		return Result.ok();
	}

	@AddSysLog(value = "客户回访台账-删除")
	@DeleteMapping("/delete")
	public Result<?> delete(TbCustomerFollow item){
		tbCustomerFollowService.removeById(item.getId());
		return Result.ok();
	}

    /**
     * 文件上传
     */
    @PostMapping("/upload")
    public Result<?> upload(HttpServletRequest request) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        MultipartFile file = multipartRequest.getFile("file");
        if (ObjectUtil.isEmpty(file)) {
            return Result.error("请正确上传文件！");
        }
        return Result.ok(minioUtils.uploadFileValid(file, FileExtension.getCommonFileTypes()));
    }

	/**
	 *  simpleWrite在数据量不大的情况下可以使用（5000以内，具体也要看实际情况），数据量大时要 重复多次写入
	 * @param response
	 */
    @AddSysLog(value = "客户回访台账-导出")
	@GetMapping("/simple/export")
	public Result<?> export(TbCustomerFollow item,HttpServletResponse response){
		try (OutputStream output = response.getOutputStream()){

			String fileName = "客户回访台账.xlsx";
			try {
				fileName = URLEncoder.encode(fileName, "UTF-8");
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
			}
			response.setHeader("Content-disposition", "attachment;filename=" + fileName + ";" + "filename*=utf-8''" + fileName);

			Result<?> result = list(item);
			List<TbCustomerFollow> list = (List<TbCustomerFollow>)result.getResult();
			EasyExcel.write(output, TbCustomerFollow.class).sheet("sheet1").doWrite(list);

		} catch (IOException e) {
			log.error(e.getMessage(), e);
			return Result.error("导出错误！");
		}
		return Result.ok();
	}

    @AddSysLog(value = "客户回访台账-分页导出")
	@GetMapping("/multi/export")
	public Result<?> multiExport(TbCustomerFollow item,HttpServletResponse response){
		try (OutputStream output = response.getOutputStream()){
			String fileName = "客户回访台账.xlsx";
			try {
				fileName = URLEncoder.encode(fileName, "UTF-8");
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
			}
			response.setHeader("Content-disposition", "attachment;filename=" + fileName + ";" + "filename*=utf-8''" + fileName);

			int pages = 1;
			try (ExcelWriter excelWriter = EasyExcel.write(output, TbCustomerFollow.class).build()) {
				WriteSheet writeSheet = EasyExcel.writerSheet("sheet1").build();
				for (int i = 0; i < pages; i++) {
					Result<?> result = page(item,i+1, 10);
					IPage<TbCustomerFollow> page = (IPage<TbCustomerFollow>)result.getResult();
					List<TbCustomerFollow> data = page.getRecords();
					int temp = (int)page.getPages();
					if(pages<temp) {
						pages = temp;
					}
					excelWriter.write(data, writeSheet);
				}
			}
		} catch (IOException e) {
			log.error(e.getMessage(), e);
			return Result.error("导出错误！");
		}
		return Result.ok();
	}

}
