package com.gci.cdnev.smallprogram.modules.driver.controller;


import java.util.Date;
import java.util.List;

import com.gci.cdnev.smallprogram.common.log.annotation.AddSysLog;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gci.cdnev.smallprogram.annotation.CurrentUser;
import com.gci.cdnev.smallprogram.annotation.Query.handle.QueryGenerator;
import com.gci.cdnev.smallprogram.common.util.DateUtils;
import com.gci.cdnev.smallprogram.common.vo.Result;
import com.gci.cdnev.smallprogram.modules.driver.entity.TbDriverSignIn;
import com.gci.cdnev.smallprogram.modules.driver.service.ITbDriverSignInService;
import com.gci.cdnev.smallprogram.modules.wx.entity.JwtUser;

/**
 * <p>
 * 司机打卡 前端控制器
 * </p>
 *
 * @author xushuhan
 * @since 2023-09-13
 */
@RestController
@RequestMapping("/api/driverSignIn")
public class TbDriverSignInController {

	@Autowired
	private ITbDriverSignInService driverService;

	@AddSysLog(value = "司机档案-列表查询")
	@GetMapping("/list")
	public Result<?> list(@CurrentUser JwtUser jwtUser,TbDriverSignIn item) {
		LambdaQueryWrapper<TbDriverSignIn> queryWrapper = QueryGenerator.initQueryWrapper(TbDriverSignIn.class, item);
		queryWrapper.eq(TbDriverSignIn::getDriverId, jwtUser.getEmployeeId());
		//queryWrapper.eq(TbDriverSignIn::getCancelFlag, CommonConstant.STATUS_1);
		if(item.getSignInDate()==null) {
			queryWrapper.eq(TbDriverSignIn::getSignInDate, DateUtils.getToday());
		}
		queryWrapper.orderByDesc(TbDriverSignIn::getSignInTime);
		List<TbDriverSignIn> list = driverService.list(queryWrapper);
		return Result.ok(list);
	}

	@AddSysLog(value = "司机档案-分页查询")
	@GetMapping("/page")
	public Result<?> page(@CurrentUser JwtUser jwtUser,TbDriverSignIn item,@RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
						  @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize) {

		LambdaQueryWrapper<TbDriverSignIn> queryWrapper = QueryGenerator.initQueryWrapper(TbDriverSignIn.class, item);
		queryWrapper.eq(TbDriverSignIn::getDriverId, jwtUser.getEmployeeId());
		queryWrapper.orderByDesc(TbDriverSignIn::getSignInTime);
		Page<TbDriverSignIn> page = new Page<TbDriverSignIn>(pageNo, pageSize);
		IPage<TbDriverSignIn> pageList = driverService.page(page,queryWrapper);
		return Result.ok(pageList);
	}

	@AddSysLog(value = "司机档案-新增")
	@PostMapping("/add")
	public Result<?> add(@CurrentUser JwtUser jwtUser,@RequestBody TbDriverSignIn item){
		item.setDriverId(jwtUser.getEmployeeId());
		item.setDriverName(jwtUser.getEmployeeName());
		item.setOpType("1");
		item.setSignInTime(new Date());
		item.setSignInDate(DateUtils.getDayStart(item.getSignInTime()));
		item.setSysOrgCode(jwtUser.getSysOrgCode());
		item.setCreateBy(jwtUser.getUserName());
		String error = driverService.sign(item);;
		if(StringUtils.hasLength(error)) {
			return Result.error(error);
		}
		return Result.ok(item);
	}

	/**
	 * 获取最后签到车辆
	 * @param jwtUser
	 * @return
	 */
	@AddSysLog(value = "司机档案-获取最后签到车辆")
	@GetMapping("/getLastSignIn")
	public Result<?> getLastSignIn(@CurrentUser JwtUser jwtUser) {
		TbDriverSignIn last = driverService.getLastSignIn(jwtUser.getEmployeeId());
		if(last!=null&&"1".equals(last.getSignInType())) {
			return Result.ok(last);
		}
		return Result.ok();
	}

	/**
	 * 获取打卡日历
	 * @param jwtUser
	 * @param item
	 * @return
	 */
	@AddSysLog(value = "司机档案-获取打卡日历")
	@GetMapping("/monthList")
	public Result<?> monthList(@CurrentUser JwtUser jwtUser,TbDriverSignIn item) {
		item.setDriverId(jwtUser.getEmployeeId());
		return Result.ok(driverService.monthList(item));
	}

	/*@PutMapping("/edit")
	public Result<?> edit(@CurrentUser JwtUser jwtUser,@RequestBody TbDriverSignIn item){
		item.setDriverId(jwtUser.getEmployeeId());
		driverService.updateById(item);
		return Result.ok();
	}*/

	/*@DeleteMapping("/delete")
	public Result<?> delete(TbDriverSignIn item){
		driverService.removeById(item.getId());
		return Result.ok();
	}*/

}
