package com.gci.cdnev.smallprogram.modules.driver.entity;

import com.alibaba.excel.annotation.ExcelIgnore;
import com.alibaba.excel.annotation.ExcelProperty;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.annotation.JsonFormat;

import java.io.Serializable;
import java.util.Date;

import com.gci.cdnev.smallprogram.annotation.Query.SelectQuery;
import com.gci.cdnev.smallprogram.common.dict.annotation.Dict;
import com.gci.cdnev.smallprogram.common.dict.excel.DictConverter;
import org.springframework.format.annotation.DateTimeFormat;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Getter;
import lombok.Setter;

/**
 * <p>
 * 司机档案
 * </p>
 *
 * @author xushuhan
 * @since 2023-09-04
 */
@Getter
@Setter
@TableName("tb_driver")
@ApiModel(value = "TbDriver对象", description = "")
public class TbDriver implements Serializable {

    private static final long serialVersionUID = 1L;

    @TableId(value = "id", type = IdType.ASSIGN_ID)
    @ExcelIgnore
    private String id;

    @ApiModelProperty("所属项目")
    @SelectQuery(Type = SelectQuery.SelectType.EQ)
    @ExcelProperty(value = "所属项目", converter = DictConverter.class, index = 0)
    @Dict(dictTable = "tb_projects", dicText = "project_name", dicCode = "id")
    private String projectId;

    @ApiModelProperty("姓名")
    @SelectQuery
    @ExcelProperty(value = "姓名", index = 1)
    private String name;

    @ApiModelProperty("性别（1男，0女）")
    @ExcelProperty(value = "性别", converter = DictConverter.class, index = 2)
    @Dict(dicCode = "sex")
    private String sex;

    @ApiModelProperty("电话")
    @ExcelProperty(value = "联系电话", index = 3)
    private String phone;

    @ApiModelProperty("身份证号码")
    @ExcelProperty(value = "身份证号码", index = 4)
    private String idCard;

    @ApiModelProperty("入职时间")
    @JsonFormat(timezone = "GMT+8",pattern = "yyyy-MM-dd")
    @DateTimeFormat(pattern="yyyy-MM-dd")
    @ExcelProperty(value = "入职日期", index = 5)
    private Date entryTime;

    @ApiModelProperty("入职状态（1在职，0离职）")
    @SelectQuery(Type = SelectQuery.SelectType.EQ)
    @ExcelProperty(value = "入职状态", converter = DictConverter.class, index = 6)
    @Dict(dicCode = "entry_status")
    private String entryStatus;

    @ApiModelProperty("地址")
    @ExcelIgnore
    private String address;

    @ApiModelProperty("司机照片")
    @ExcelIgnore
    private String fileUrl;

    @ApiModelProperty("备注")
    @ExcelIgnore
    private String remark;

    @ApiModelProperty("所属机构")
    @ExcelIgnore
    private String sysOrgCode;

    @ApiModelProperty("创建时间")
    @JsonFormat(timezone = "GMT+8",pattern = "yyyy-MM-dd HH:mm:ss")
    @DateTimeFormat(pattern="yyyy-MM-dd HH:mm:ss")
    @ExcelIgnore
    private Date createTime;

    @ApiModelProperty("更新时间")
    @JsonFormat(timezone = "GMT+8",pattern = "yyyy-MM-dd HH:mm:ss")
    @DateTimeFormat(pattern="yyyy-MM-dd HH:mm:ss")
    @ExcelIgnore
    private Date updateTime;

    @ApiModelProperty("创建人")
    @ExcelIgnore
    private String createBy;

    @ApiModelProperty("更新人")
    @ExcelIgnore
    private String updateBy;

    @ExcelIgnore
    @ApiModelProperty("删除标志")
    private Integer delFlag;

    @ApiModelProperty("状态(1启用，0禁用)")
    @Dict(dicCode = "status")
    @ExcelIgnore
    private String status;

    @JsonFormat(timezone = "GMT+8",pattern = "yyyy-MM-dd")
    @DateTimeFormat(pattern="yyyy-MM-dd")
    @TableField(exist = false)
    @ExcelIgnore
    private Date entryTimeStart;

    @JsonFormat(timezone = "GMT+8",pattern = "yyyy-MM-dd")
    @DateTimeFormat(pattern="yyyy-MM-dd")
    @TableField(exist = false)
    @ExcelIgnore
    private Date entryTimeEnd;

    @ApiModelProperty("系统用户")
    @ExcelIgnore
    private String userId;
    
}
