package com.gci.cdnev.smallprogram.modules.driver.service.impl;

import java.util.Date;
import java.util.List;

import cn.hutool.core.util.StrUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gci.cdnev.smallprogram.common.util.DateUtils;
import com.gci.cdnev.smallprogram.common.util.JsonUtil;
import com.gci.cdnev.smallprogram.constant.CommonConstant;
import com.gci.cdnev.smallprogram.modules.driver.entity.TbDriverSignIn;
import com.gci.cdnev.smallprogram.modules.driver.mapper.TbDriverSignInMapper;
import com.gci.cdnev.smallprogram.modules.driver.service.ITbDriverSignInService;

/**
 * <p>
 * 司机打卡 服务实现类
 * </p>
 *
 * @author xushuhan
 * @since 2023-09-13
 */
@Service
public class TbDriverSignInServiceImpl extends ServiceImpl<TbDriverSignInMapper, TbDriverSignIn> implements ITbDriverSignInService {

	@Autowired
	private StringRedisTemplate redisTemplate;
	@Autowired
	private TbDriverSignInMapper driverSignInMapper;
	
	@Override
	public TbDriverSignIn getLastSignIn(String driverId) {
		Date today = new Date();
		today = DateUtils.addHours(today, -12);
		List<TbDriverSignIn> list = lambdaQuery().eq(TbDriverSignIn::getDriverId, driverId)
				.gt(TbDriverSignIn::getSignInTime, today)
				.orderByDesc(TbDriverSignIn::getCreateTime).last("limit 1").list();
		return list.size()>0?list.get(0):null;
	}
	
	public TbDriverSignIn getVehicleLastSignIn(String vehicleId) {
		Date today = new Date();
		today = DateUtils.addHours(today, -12);
		List<TbDriverSignIn> list = lambdaQuery().eq(TbDriverSignIn::getVehicleId, vehicleId)
				.gt(TbDriverSignIn::getSignInTime, today)
				.orderByDesc(TbDriverSignIn::getCreateTime).last("limit 1").list();
		return list.size()>0?list.get(0):null;
	}
	
	@Override
	public String sign(TbDriverSignIn sign) {

		TbDriverSignIn last = getLastSignIn(sign.getDriverId());

		//签退车辆与签到车辆不一致
		if(last!=null&&"2".equals(sign.getSignInType())&&"1".equals(last.getSignInType())) {
			if(!sign.getVehicleId().equals(last.getVehicleId())) {
				return "签退车辆与签到车辆不一致"+last.getPlateNumber();
			}
		}
		
		if(last!=null&&last.getSignInType().equals(sign.getSignInType())) {
			if("1".equals(sign.getSignInType())) {
				return "请先签退车辆"+last.getPlateNumber();
			}else {
				return "请先签到车辆"+sign.getPlateNumber();
			}
		}
		
		TbDriverSignIn vehicleLast = getVehicleLastSignIn(sign.getVehicleId());
		//车辆已被其他司机签到
		if(vehicleLast!=null&&"1".equals(vehicleLast.getSignInType())) {
			if("1".equals(sign.getSignInType())&&!sign.getDriverId().equals(vehicleLast.getDriverId())) {
				return "车辆"+vehicleLast.getPlateNumber()+"已被司机"+vehicleLast.getDriverName()+"签到";
			}
		}
		
		this.save(sign);
		
		//司机签退，查询上次签到记录，保存签退id
		if("2".equals(sign.getSignInType())&&last!=null&&"1".equals(last.getSignInType())&&!StringUtils.hasLength(last.getSignOutId())) {
			last.setSignOutId(sign.getId());
			this.updateById(last);
		}
		//签到
		if("1".equals(sign.getSignInType())){
			redisTemplate.opsForValue().set(CommonConstant.VEHICLE_BIND_DRIVER+sign.getPlateNumber(), JsonUtil.object2Json(sign));
		}else{
			redisTemplate.delete(CommonConstant.VEHICLE_BIND_DRIVER+sign.getPlateNumber());
		}
		return "";
	}
	
	@Override
	public List<TbDriverSignIn> monthList(TbDriverSignIn sign) {
		if(!StringUtils.hasLength(sign.getMonth())) {
			sign.setMonth(DateUtils.formatDate(new Date(), DateUtils.HYPHENS_MONTH));
		}
		return baseMapper.monthList(sign);
	}

	@Override
	public String queryDriverByPlateNumberAndTime(String plateNumber, String time) {
		if(StrUtil.isBlank(plateNumber) || StrUtil.isBlank(time)){
			return null;
		}
		return driverSignInMapper.queryDriverByPlateNumberAndTime(plateNumber,time);
	}

}
