package com.gci.cdnev.smallprogram.modules.event.controller;


import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gci.cdnev.smallprogram.annotation.CurrentUser;
import com.gci.cdnev.smallprogram.common.log.annotation.AddSysLog;
import com.gci.cdnev.smallprogram.common.util.MinioUtils;
import com.gci.cdnev.smallprogram.common.validated.SaveValidated;
import com.gci.cdnev.smallprogram.common.validated.UpdateValidated;
import com.gci.cdnev.smallprogram.common.vo.Result;
import com.gci.cdnev.smallprogram.enums.FileExtension;
import com.gci.cdnev.smallprogram.modules.event.entity.EventInfo;
import com.gci.cdnev.smallprogram.modules.event.entity.EventProgress;
import com.gci.cdnev.smallprogram.modules.event.query.EventInfoQuery;
import com.gci.cdnev.smallprogram.modules.event.service.IEventInfoService;
import com.gci.cdnev.smallprogram.modules.wx.entity.JwtUser;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;

/**
 * <p>
 * 事故信息基础表 前端控制器
 * </p>
 *
 * @author 2464
 * @since 2023-09-13
 */
@RestController
@RequestMapping("/api/event/info")
public class EventInfoController {
    @Autowired
    private IEventInfoService iEventInfoService;
    @Autowired
    private MinioUtils minioUtils;

    /**
     * 分页列表查询
     *
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AddSysLog("事故信息-列表查询")
    @GetMapping(value = "/list")
    public Result<?> queryPageList(EventInfoQuery query,@CurrentUser JwtUser user,
                                   @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                   @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                   HttpServletRequest req) {
        Page<EventInfo> page = new Page<EventInfo>(pageNo, pageSize);
        IPage<EventInfo> pageList = iEventInfoService.pageBuild(page, query,user);
        return Result.ok(pageList);
    }

    /**
     * 待办列表
     */
    //@GetMapping(value = "/todoList")
    public Result<?> todoList(
                                   @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                   @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                   HttpServletRequest req) {
        Page<EventInfo> page = new Page<EventInfo>(pageNo, pageSize);
        IPage<EventInfo> pageList = iEventInfoService.todoList(page);
        return Result.ok(pageList);
    }

    /**
     * 添加
     */
    @AddSysLog("事故信息-添加")
    @PostMapping(value = "/add")
    public Result<?> add(@CurrentUser JwtUser user,@RequestBody @Validated(SaveValidated.class) EventInfo eventInfo) {
        eventInfo.setReportBy(user.getUserName());
        eventInfo.setCreateBy(user.getUserName());
        eventInfo.setUpdateBy(user.getUserName());
        eventInfo.setSysOrgCode(user.getSysOrgCode());
        eventInfo.setIsHandle(0);
        iEventInfoService.saveItem(eventInfo);
        return Result.ok("添加成功！");
    }

    /**
     * 编辑
     */
    @AddSysLog("事故信息-编辑")
    @PutMapping(value = "/edit")
    public Result<?> edit(@CurrentUser JwtUser user,@RequestBody @Validated(UpdateValidated.class) EventInfo eventInfo) {
        eventInfo.setUpdateBy(user.getUserName());
        iEventInfoService.updateItem(eventInfo,user);
        return Result.ok("编辑成功！");
    }

    @AddSysLog("事故信息-处理")
    @PostMapping(value = "/handle")
    public Result<?> handle(@CurrentUser JwtUser user,@RequestBody @Validated(UpdateValidated.class) EventProgress progress) {
        iEventInfoService.handle(progress,user);
        return Result.ok("处理成功！");
    }

    /**
     * 删除
     */
    @AddSysLog("事故信息-删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        iEventInfoService.removeItem(id);
        return Result.ok("删除成功！");
    }

    /**
     * 批量删除
     */
//    @DeleteMapping(value = "/deleteBatch")
    public Result<?> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        iEventInfoService.removeByIds(Arrays.asList(ids.split(",")));
        return Result.ok("批量删除成功！");
    }

    /**
     * 文件上传
     */
    @AddSysLog("事故信息-文件上传")
    @PostMapping("/upload")
    public Result<?> upload(HttpServletRequest request) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        MultipartFile file = multipartRequest.getFile("file");
        if (ObjectUtil.isEmpty(file)) {
            return Result.error("请正确上传文件！");
        }
        List<String> fileList = new ArrayList<>();
        fileList.addAll(FileExtension.PIC.getNameList());
        fileList.addAll(FileExtension.DOC.getNameList());
        return Result.ok(minioUtils.uploadFileV2Valid(file,fileList));
    }


    /**
     * 获取单个
     */
    @AddSysLog("事故信息-查询详情")
    @GetMapping(value = "/getById")
    public Result<?> getById(String id,@CurrentUser JwtUser user) {
        EventInfo info = iEventInfoService.getEventInfoById(id);
        if(Objects.isNull(info.getEventProgress())){
            //只要是运营维保且未处理，则显示去处理
            if(user.isMaintenance()){
                info.setCanHandle(true);
            }
            //未处理且为自己上报
            if(user.getUserName().equals(info.getCreateBy())){
                info.setCanDelete(true);
            }
        }
        return Result.ok(info);
    }
}
