package com.gci.cdnev.smallprogram.modules.event.entity;

import com.baomidou.mybatisplus.annotation.*;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonPropertyDescription;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.gci.cdnev.smallprogram.annotation.fileUrl.FileUrl;
import com.gci.cdnev.smallprogram.annotation.fileUrl.handle.FileUrlHandle;
import com.gci.cdnev.smallprogram.common.dict.annotation.Dict;
import com.gci.cdnev.smallprogram.common.validated.SaveValidated;
import com.gci.cdnev.smallprogram.common.validated.UpdateValidated;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.apache.ibatis.annotations.One;
import org.apache.ibatis.mapping.FetchType;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Pattern;
import javax.validation.constraints.Size;
import java.io.Serializable;
import java.util.Date;

/**
 * <p>
 * 事件信息基础表
 * </p>
 *
 * @author 2464
 * @since 2023-09-13
 */
@Data
@TableName("tb_event_info")
@ApiModel(value = "EventInfo对象", description = "事件信息基础表")
public class EventInfo implements Serializable {

    private static final long serialVersionUID = 1L;

    @JsonPropertyDescription("id")
    @TableId(value = "id", type = IdType.ASSIGN_ID)
    private String id;

    @ApiModelProperty("所属机构")
    private String sysOrgCode;

    @ApiModelProperty("创建时间")
    private Date createTime;

    @ApiModelProperty("创建人")
    private String createBy;

    @ApiModelProperty("更新时间")
    private Date updateTime;

    @ApiModelProperty("更新人")
    private String updateBy;

    @ApiModelProperty("所属项目")
    @NotBlank(message = "所属项目不能为空！",groups = {SaveValidated.class, UpdateValidated.class})
    @Dict(dicCode = "id",dictTable = "tb_projects",dicText = "project_name")
    private String projectId;

    @ApiModelProperty("事件主题")
    @NotBlank(message = "事件主题不能为空！",groups = {SaveValidated.class, UpdateValidated.class})
    @Size(max = 32,message = "事件主题不能超过32位字符！",groups = {SaveValidated.class, UpdateValidated.class})
    private String eventTheme;

    @ApiModelProperty("事件分类")
    @Dict(dicCode = "event_category")
    @NotBlank(message = "事件分类不能为空！",groups = {SaveValidated.class, UpdateValidated.class})
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private String eventCategory;

    @ApiModelProperty("上报时间")
    @JsonFormat(timezone = "GMT+8",pattern = "yyyy-MM-dd HH:mm")
    @DateTimeFormat(pattern="yyyy-MM-dd HH:mm")
    @NotNull(message = "上报时间不能为空！",groups = {SaveValidated.class, UpdateValidated.class})
    private Date reportTime;

    @ApiModelProperty("上报人")
    @Dict(dictTable = "sys_user",dicCode = "username",dicText = "realname")
    @Size(max = 32,message = "上报人不能超过32位字符！",groups = {SaveValidated.class, UpdateValidated.class})
    private String reportBy;


    @ApiModelProperty("联系电话")
    @Pattern(regexp = "^(?:(?:\\+|00)86)?1\\d{10}$",message = "联系电话格式错误!",groups = {SaveValidated.class,UpdateValidated.class})
    @NotBlank(message = "联系电话不能为空！",groups = {SaveValidated.class, UpdateValidated.class})
    private String phoneNumber;

    @ApiModelProperty("事件内容")
    @Size(max = 1024,message = "事件内容不能超过1024位字符！")
    @NotBlank(message = "事件内容不能为空！",groups = {SaveValidated.class, UpdateValidated.class})
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private String eventText;

    @ApiModelProperty("备注")
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    @Size(max = 1024,message = "备注不能超过1024位字符！")
    private String remark;

    @ApiModelProperty("事件附件")
    @Size(max = 1024,message = "事件附件文件名过长请修改！",groups = {SaveValidated.class, UpdateValidated.class})
    @JsonDeserialize(using = FileUrlHandle.class)
    @FileUrl
    private String fileUrl;

    @ApiModelProperty("是否已处理")
    @Dict(dicCode = "yes_no")
    private Integer isHandle;

    @ApiModelProperty("事件进度")
    @TableField(exist = false)
    @Valid
    private EventProgress eventProgress;

    @TableField(exist = false)
    private boolean canHandle;

    @TableField(exist = false)
    private boolean canDelete;


}
