package com.gci.cdnev.smallprogram.modules.event.entity;

import com.baomidou.mybatisplus.annotation.*;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.gci.cdnev.smallprogram.annotation.fileUrl.FileUrl;
import com.gci.cdnev.smallprogram.annotation.fileUrl.handle.FileUrlHandle;
import com.gci.cdnev.smallprogram.common.dict.annotation.Dict;
import com.gci.cdnev.smallprogram.common.validated.SaveValidated;
import com.gci.cdnev.smallprogram.common.validated.UpdateValidated;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import java.io.Serializable;
import java.util.Date;

/**
 * <p>
 * 事件进度信息
 * </p>
 *
 * @author 2464
 * @since 2023-09-13
 */
@Data
@AllArgsConstructor
@NoArgsConstructor
@TableName("tb_event_progress")
@ApiModel(value = "EventProgress对象", description = "事件进度信息")
public class EventProgress implements Serializable {

    private static final long serialVersionUID = 1L;

    @ApiModelProperty("id")
    @TableId(value = "id", type = IdType.ASSIGN_ID)
    private String id;

    @ApiModelProperty("事件信息主表id")
    private String eventId;

    @ApiModelProperty("处理人")
    @Dict(dictTable = "sys_user",dicCode = "username",dicText = "realname")
    @Size(max = 32,message = "处理人字段过长请修改",groups = {SaveValidated.class, UpdateValidated.class})
    private String handleBy;

    @ApiModelProperty("处理时间")
    @NotNull(message = "处理时间不能为空！",groups = {SaveValidated.class, UpdateValidated.class})
    @JsonFormat(timezone = "GMT+8",pattern = "yyyy-MM-dd HH:mm")
    @DateTimeFormat(pattern="yyyy-MM-dd HH:mm")
    private Date handleTime;

    @ApiModelProperty("处理结论")
    @NotBlank(message = "处理结论不能为空！",groups = {SaveValidated.class, UpdateValidated.class})
    @Size(max = 1024,message = "处理结论过长请修改！",groups = {SaveValidated.class, UpdateValidated.class})
    private String handleResult;

    @ApiModelProperty("附件")
    @Size(max = 1024,message = "附件文件名过长请修改！",groups = {SaveValidated.class, UpdateValidated.class})
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    @JsonDeserialize(using = FileUrlHandle.class)
    @FileUrl
    private String handleFile;

    @ApiModelProperty("备注")
    @Size(max = 512,message = "备注不能超过512位字符！",groups = {SaveValidated.class, UpdateValidated.class})
    private String remark;

    @ApiModelProperty("处理状态")
    private Integer handleStatus;







}
