package com.gci.cdnev.smallprogram.modules.gis.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.BetweenFormatter;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import com.gci.cdnev.smallprogram.annotation.Query.handle.QueryGenerator;
import com.gci.cdnev.smallprogram.common.util.CoordinateConverterUtil;
import com.gci.cdnev.smallprogram.common.util.JsonUtil;
import com.gci.cdnev.smallprogram.common.util.LatLonPoint;
import com.gci.cdnev.smallprogram.exception.GciBootException;
import com.gci.cdnev.smallprogram.modules.basedata.entity.TbVehicle;
import com.gci.cdnev.smallprogram.modules.basedata.service.ITbVehicleService;
import com.gci.cdnev.smallprogram.modules.gis.entity.TbGisChargeLog;
import com.gci.cdnev.smallprogram.modules.gis.entity.TbGisGps;
import com.gci.cdnev.smallprogram.modules.gis.mapper.TbGisGpsMapper;
import com.gci.cdnev.smallprogram.modules.gis.query.GpsHistoryQuery;
import com.gci.cdnev.smallprogram.modules.gis.service.ITbGisChargeLogService;
import com.gci.cdnev.smallprogram.modules.gis.service.ITbGisGpsService;
import com.gci.cdnev.smallprogram.modules.gis.vo.GisHistoryVo;
import com.gci.cdnev.smallprogram.modules.gis.vo.PointListDTO;
import com.gci.cdnev.smallprogram.modules.gis.vo.TimeSlotDTO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author 2464
 * @since 2023-09-01
 */
@Service
@Slf4j
public class TbGisGpsServiceImpl extends ServiceImpl<TbGisGpsMapper, TbGisGps> implements ITbGisGpsService {

    @Autowired
    private ITbVehicleService vehicleService;
    @Autowired
    private ITbGisChargeLogService chargeLogService;

    @Override
    public List<GisHistoryVo> getGpsHistory(GpsHistoryQuery vo) {
        List<Date> time = vo.getTime();
        if (DateUtil.between(time.get(0), time.get(1), DateUnit.MS, false) < 0) {
            throw new GciBootException("开始时间不能大于结束时间");
        }

        TbVehicle one = vehicleService.getOne(new LambdaQueryWrapper<TbVehicle>().eq(TbVehicle::getPlateNumber, vo.getPlateNumber()));
        Integer batteryCapacity = one.getBatteryCapacity();


        LambdaQueryWrapper<TbGisGps> lambdaQueryWrapper = QueryGenerator.initQueryWrapper(TbGisGps.class, vo);
        List<TbGisGps> list = list(lambdaQueryWrapper);


        //返回给前端的list
        List<GisHistoryVo> gisHistoryVos = new ArrayList<>();

        //行驶数据
        drivingData(batteryCapacity, list,gisHistoryVos);
        //充电数据 只查询结束时间在查询条件时间条件区间内
        powerData(time,gisHistoryVos);

        //再排序两遍
        if (ObjectUtil.isNotEmpty(gisHistoryVos)) {
            for (GisHistoryVo gisHistoryVo : gisHistoryVos) {
                List<TimeSlotDTO> timeSlot = gisHistoryVo.getTimeSlot();
                timeSlot.sort(Comparator.comparing(TimeSlotDTO::getSlotTime).reversed());
            }
        }
        gisHistoryVos.stream().sorted(Comparator.comparing(GisHistoryVo::getDayTime).reversed()).collect(Collectors.toList());
        return gisHistoryVos;
    }

    private void powerData(List<Date> time, List<GisHistoryVo> gisHistoryVos) {
        List<TbGisChargeLog> tbGisChargeLogs = chargeLogService.list(new LambdaQueryWrapper<TbGisChargeLog>().between(TbGisChargeLog::getEndTime, time.get(0), time.get(1)));
        if (ObjectUtil.isNotEmpty(tbGisChargeLogs)) {
            for (TbGisChargeLog tbGisChargeLog : tbGisChargeLogs) {
                Date startTime = tbGisChargeLog.getStartTime();
                Date endTime = tbGisChargeLog.getEndTime();

                String day = DateUtil.format(startTime, "yyyy-MM-dd");
                DateTime dayTime = DateUtil.beginOfDay(startTime);
                String startFormat = DateUtil.format(startTime, "mm:ss");
                String endFormat = DateUtil.format(endTime, "mm:ss");
                String formatBetween = DateUtil.formatBetween(startTime, endTime, BetweenFormatter.Level.MINUTE);
                boolean sameDay = DateUtil.isSameDay(startTime, endTime);


                TimeSlotDTO timeSlotDTO = new TimeSlotDTO();
                //充电
                timeSlotDTO.setType("power");
                timeSlotDTO.setSlotTime(startTime);
                timeSlotDTO.setTimeCount(formatBetween);
                timeSlotDTO.setIsOvernight(sameDay ? 0 : 1);
                timeSlotDTO.setTimeRange(startFormat + " - " + endFormat);
                timeSlotDTO.setChargedEnergy(tbGisChargeLog.getChargingEnergy());
                timeSlotDTO.setChargedEfficiency(tbGisChargeLog.getChargingEfficiency());

                ArrayList<PointListDTO> pointListDTOS = new ArrayList<>();
                PointListDTO pointListDTO = new PointListDTO();
                ArrayList<Double> doubles = new ArrayList<>();
                doubles.add(tbGisChargeLog.getGdLat().doubleValue());
                doubles.add(tbGisChargeLog.getGdLon().doubleValue());
                pointListDTO.setLocation(doubles);
                pointListDTOS.add(pointListDTO);
                timeSlotDTO.setPointList(pointListDTOS);
                GisHistoryVo gisHistoryVo = gisHistoryVos.stream().filter(gisHistoryVo1 -> gisHistoryVo1.getDay().equals(day)).findFirst().orElse(new GisHistoryVo());
                if (BeanUtil.isEmpty(gisHistoryVo)) {
                    gisHistoryVo.setDay(day);
                    gisHistoryVo.setDayTime(dayTime);
                    ArrayList<TimeSlotDTO> timeSlotDTOS = new ArrayList<>();
                    timeSlotDTOS.add(timeSlotDTO);
                    gisHistoryVo.setTimeSlot(timeSlotDTOS);
                    gisHistoryVos.add(gisHistoryVo);
                } else {
                    gisHistoryVo.getTimeSlot().add(timeSlotDTO);
                }
            }
        }
    }

    private void drivingData(Integer batteryCapacity, List<TbGisGps> list, List<GisHistoryVo> gisHistoryVos) {
        List<List<TbGisGps>> lists = groupByStatusAndTime(list);
        if (ObjectUtil.isNotEmpty(lists)) {
            for (List<TbGisGps> tbGisGps : lists) {
                TbGisGps startGisGps = tbGisGps.get(0);
                TbGisGps endGisGps = tbGisGps.get(tbGisGps.size() - 1);
                Date startGisGpsGpsTime = startGisGps.getGpsTime();
                Date endGisGpsGpsTime = endGisGps.getGpsTime();
                BigDecimal startGisGpsTotalMileage = startGisGps.getTotalMileage();
                BigDecimal endGisGpsTotalMileage = endGisGps.getTotalMileage();
                Integer startGisGpsStateOfCharge = startGisGps.getStateOfCharge();
                Integer endGisGpsStateOfCharge = endGisGps.getStateOfCharge();

                String day = DateUtil.format(startGisGpsGpsTime, "yyyy-MM-dd");
                DateTime dayTime = DateUtil.beginOfDay(startGisGpsGpsTime);
                String startFormat = DateUtil.format(startGisGpsGpsTime, "mm:ss");
                String endFormat = DateUtil.format(endGisGpsGpsTime, "mm:ss");
                String formatBetween = DateUtil.formatBetween(startGisGpsGpsTime, endGisGpsGpsTime, BetweenFormatter.Level.MINUTE);
                boolean sameDay = DateUtil.isSameDay(startGisGpsGpsTime, endGisGpsGpsTime);

                TimeSlotDTO timeSlotDTO = new TimeSlotDTO();
                //行驶
                timeSlotDTO.setType("driving");
                timeSlotDTO.setSlotTime(startGisGpsGpsTime);
                timeSlotDTO.setTimeCount(formatBetween);
                timeSlotDTO.setIsOvernight(sameDay ? 0 : 1);
                timeSlotDTO.setTimeRange(startFormat + " - " + endFormat);

                if (ObjectUtil.isAllNotEmpty(startGisGpsTotalMileage, endGisGpsTotalMileage)) {
                    timeSlotDTO.setMileage(NumberUtil.sub(endGisGpsTotalMileage, startGisGpsTotalMileage));
                }
                if (ObjectUtil.isAllNotEmpty(startGisGpsStateOfCharge,endGisGpsStateOfCharge, batteryCapacity)) {
                    long hour = DateUtil.between(startGisGpsGpsTime, endGisGpsGpsTime, DateUnit.HOUR);
                    BigDecimal sub = NumberUtil.sub(startGisGpsStateOfCharge, endGisGpsStateOfCharge);
                    BigDecimal div = NumberUtil.div(sub, 100);
                    BigDecimal mul = NumberUtil.mul(div, batteryCapacity);
                    timeSlotDTO.setPowerConsumption(NumberUtil.div(mul,hour));
                }
                ArrayList<PointListDTO> pointListDTOS = new ArrayList<>();
                ArrayList<String> strings = new ArrayList<>();
                for (TbGisGps tbGisGp : tbGisGps) {
                    PointListDTO pointListDTO = new PointListDTO();
                    String speed = tbGisGp.getSpeed();
                    strings.add(speed);

                    BigDecimal lat = tbGisGp.getLat();
                    BigDecimal lon = tbGisGp.getLon();
                    LatLonPoint latLonPoint = CoordinateConverterUtil.toWGSPoint(lat.doubleValue(), lon.doubleValue());
                    ArrayList<Double> doubles = new ArrayList<>();
                    doubles.add(latLonPoint.getLatitude());
                    doubles.add(latLonPoint.getLongitude());

                    pointListDTO.setLocation(doubles);
                    pointListDTO.setStateOfCharge(tbGisGp.getStateOfCharge());
                    pointListDTO.setSpeed(NumberUtil.toBigDecimal(tbGisGp.getSpeed()));
                    pointListDTO.setTotalMileage(tbGisGp.getTotalMileage());
                    pointListDTO.setTotalVoltage(tbGisGp.getTotalVoltage());
                    pointListDTO.setGpsTime(DateUtil.formatDateTime(tbGisGp.getGpsTime()));
                    pointListDTO.setAcc(tbGisGp.getAcc());
                    pointListDTO.setDcStatus(tbGisGp.getDcStatus());
                    pointListDTO.setOperatingMode(tbGisGp.getOperatingMode());
                    pointListDTOS.add(pointListDTO);
                }
                timeSlotDTO.setPointList(pointListDTOS);
                OptionalDouble average = strings.stream().filter(StrUtil::isNotBlank).mapToInt(Integer::parseInt).average();
                timeSlotDTO.setAverageSpeed(average.orElse(0));
                GisHistoryVo gisHistoryVo = gisHistoryVos.stream().filter(gisHistoryVo1 -> gisHistoryVo1.getDay().equals(day)).findFirst().orElse(new GisHistoryVo());
                if (BeanUtil.isEmpty(gisHistoryVo)) {
                    gisHistoryVo.setDay(day);
                    gisHistoryVo.setDayTime(dayTime);
                    ArrayList<TimeSlotDTO> timeSlotDTOS = new ArrayList<>();
                    timeSlotDTOS.add(timeSlotDTO);
                    gisHistoryVo.setTimeSlot(timeSlotDTOS);
                    gisHistoryVos.add(gisHistoryVo);
                } else {
                    gisHistoryVo.getTimeSlot().add(timeSlotDTO);
                }
            }
        }
    }

    private List<List<TbGisGps>> groupByStatusAndTime(List<TbGisGps> list) {
        // 创建一个空的结果列表
        List<List<TbGisGps>> result = new ArrayList<>();
        // 如果输入列表为空，直接返回结果
        if (list == null || list.isEmpty()) {
            return result;
        }
        // 创建一个临时的子列表，用于存放符合条件的元素
        List<TbGisGps> subList = new ArrayList<>();
        // 遍历输入列表中的每个元素
        for (int i = 0; i < list.size(); i++) {
            // 获取当前元素
            TbGisGps current = list.get(i);
            // 获取当前状态 1-点火 0-熄火
            int currentStatus = getCurrentStatus(current);
            current.setCerStatus(currentStatus);
            // 点火状态时
            if (currentStatus == 1) {
                //临时列表为空并且上一个也是点火状态
                if (subList.isEmpty() || subList.get(subList.size() - 1).getCerStatus() == currentStatus) {
                    subList.add(current);
                } else {
                    log.warn("出现了意料之外的情况，什么情况会到这里？list数据为:{}", JsonUtil.object2Json(list));
                }
            } else if (currentStatus == 0) {
                //熄火状态时
                //临时列表不为空
                if (!subList.isEmpty()) {
                    //加入结束点
                    subList.add(current);
                    result.add(subList);
                    //新建临时列表
                    subList = new ArrayList<>();
                }
            }
        }
        // 遍历结束后，判断临时列表是否为空
        if (!subList.isEmpty()) {
            result.add(subList);
        }
        // 返回结果列表
        return result;
    }

    /**
     * 获取准确的是否在线
     */
    private int getCurrentStatus(TbGisGps current) {
        if (current.getRecordSource() == 1) {
            return current.getStatus() == 1 ? 1 : 0;
        } else if (current.getRecordSource() == 2) {
            return current.getAcc();
        }
        return 3;
    }
}
