package com.gci.cdnev.smallprogram.modules.monitor.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gci.cdnev.smallprogram.common.util.CoordinateConverterUtil;
import com.gci.cdnev.smallprogram.common.util.FormatUtil;
import com.gci.cdnev.smallprogram.modules.basedata.entity.TbVehicle;
import com.gci.cdnev.smallprogram.modules.basedata.mapper.TbVehicleMapper;
import com.gci.cdnev.smallprogram.modules.gis.entity.TbGisFenceAlarm;
import com.gci.cdnev.smallprogram.modules.gis.entity.TbGisRunningbus;
import com.gci.cdnev.smallprogram.modules.gis.entity.TbGisSpeedingAlarm;
import com.gci.cdnev.smallprogram.modules.gis.entity.TbGisStoppingAlarm;
import com.gci.cdnev.smallprogram.modules.gis.mapper.TbGisFenceAlarmMapper;
import com.gci.cdnev.smallprogram.modules.gis.mapper.TbGisRunningbusMapper;
import com.gci.cdnev.smallprogram.modules.gis.service.ITbGisFenceAlarmService;
import com.gci.cdnev.smallprogram.modules.gis.service.ITbGisRunningbusService;
import com.gci.cdnev.smallprogram.modules.gis.service.ITbGisSpeedingAlarmService;
import com.gci.cdnev.smallprogram.modules.gis.service.ITbGisStoppingAlarmService;
import com.gci.cdnev.smallprogram.modules.monitor.enums.DirectionEnum;
import com.gci.cdnev.smallprogram.modules.monitor.service.IMonitorService;
import com.gci.cdnev.smallprogram.modules.monitor.vo.AlarmCountVO;
import com.gci.cdnev.smallprogram.modules.monitor.vo.OverallMonitorVO;
import com.gci.cdnev.smallprogram.modules.monitor.vo.RunningbusVO;
import com.gci.cdnev.smallprogram.modules.monitor.vo.VehicleTreeVO;
import com.gci.cdnev.smallprogram.modules.vehicles.entity.TbInsurance;
import com.gci.cdnev.smallprogram.modules.vehicles.service.ITbInsuranceService;
import io.swagger.models.auth.In;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

@Service
@Slf4j
public class MonitorServiceImpl implements IMonitorService {

    @Autowired
    private ITbGisRunningbusService runningbusService;
    @Autowired
    private TbGisRunningbusMapper runningbusMapper;
    @Autowired
    private TbVehicleMapper tbVehicleMapper;

    @Autowired
    private ITbGisFenceAlarmService tbGisFenceAlarmService;
    @Autowired
    private ITbGisSpeedingAlarmService tbGisSpeedingAlarmService;
    @Autowired
    private ITbGisStoppingAlarmService tbGisStoppingAlarmService;

    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private ITbInsuranceService iTbInsuranceService;

    /**
     * 获取全局监控车辆数据
     *
     * @param statusType
     * @return
     */
    @Override
    public List<OverallMonitorVO> getOverallVehicleByListView(Page<OverallMonitorVO> page, String statusType, List<String> projectIds) {
        //status -1在线，2-离线，3-故障告警，4-围栏告警，5-超速告警，6-主动安全告警
        List<OverallMonitorVO> list = runningbusMapper.queryOverallVehicle(page, statusType, projectIds);
        return list;
    }

    @Override
    public List<OverallMonitorVO> getOverallVehicleByMap(String statusType, double lon, double lat, double mapLevel, List<String> projectIds) {
        List<OverallMonitorVO> resultList = runningbusMapper.queryOverallVehicleByMap(statusType, projectIds);
        resultList = resultList.stream().filter(r->{
            return r.getLon().compareTo(BigDecimal.ZERO)>0&&r.getLat().compareTo(BigDecimal.ZERO)>0;
        }).collect(Collectors.toList());
        //计算距离
//        double radius = CoordinateConverterUtil.calculateVisibleAreaRadius(mapLevel);
////        log.info("根据层级{}计算出来的半径为{}米", mapLevel, radius);
//        //计算变化量
//        double div = CoordinateConverterUtil.anglediv(radius);
//        double startLat = lat - div;
//        double endLat = lat + div;
//        double startLon = lon - div;
//        double endLon = lon + div;
//        List<OverallMonitorVO> resultList = list.stream().filter(l -> {
//            return l.getLat().doubleValue() > startLat && l.getLat().doubleValue() < endLat &&
//                    l.getLon().doubleValue() > startLon && l.getLon().doubleValue() < endLon;
//        }).collect(Collectors.toList());
        return resultList;
    }

    @Override
    public List<OverallMonitorVO> getOverallVehicleByAll(String plateNumber, List<String> projectIds) {
        List<OverallMonitorVO> list = runningbusMapper.queryOverallVehicleByAll(plateNumber, projectIds);
        return list;
    }

    /**
     * 获取所有车辆对应项目，为空的归到其它
     *
     * @param plateNumber
     * @return
     */
    @Override
    public List<VehicleTreeVO> getVehicleTree(String plateNumber) {
        List<VehicleTreeVO> treeVO = runningbusMapper.queryProjrctVehicle(plateNumber);
        return treeVO;
    }

    /**
     * 获取各状态车辆数量
     *
     * @return
     */
    @Override
    public List<AlarmCountVO> getVehicleCount(List<String> projectIds) {
        LambdaQueryWrapper<TbGisRunningbus> busQuery = new LambdaQueryWrapper<>();
        if (!projectIds.isEmpty()) {
            busQuery.in(TbGisRunningbus::getProjectId, projectIds);
        }
        List<TbGisRunningbus> busList = runningbusMapper.selectList(busQuery);
        List<AlarmCountVO> list = new ArrayList<>();
        AlarmCountVO vo = new AlarmCountVO("全部车辆", "", busList.stream().count());
        AlarmCountVO vo1 = new AlarmCountVO("在线", "1",
                busList.stream().filter(item -> Objects.nonNull(item.getIsOnline()) && 1 == item.getIsOnline()).count());
        AlarmCountVO vo2 = new AlarmCountVO("离线", "2",
                busList.stream().filter(item -> Objects.nonNull(item.getIsOnline()) && 0 == item.getIsOnline()).count());
        AlarmCountVO vo3 = new AlarmCountVO("故障告警", "3",
                busList.stream().filter(item -> Objects.nonNull(item.getFaultAlarm()) && 1 == item.getFaultAlarm()).count());
        AlarmCountVO vo4 = new AlarmCountVO("围栏告警", "4",
                busList.stream().filter(item -> Objects.nonNull(item.getFenceAlarm()) && 1 == item.getFenceAlarm()).count());
        AlarmCountVO vo5 = new AlarmCountVO("超速告警", "5",
                busList.stream().filter(item -> Objects.nonNull(item.getSpeedingAlarm()) && 1 == item.getSpeedingAlarm()).count());
        AlarmCountVO vo6 = new AlarmCountVO("主动安全告警", "6",
                busList.stream().filter(item -> Objects.nonNull(item.getSafetyAlarm()) && 1 == item.getSafetyAlarm()).count());
        list.add(vo);
        list.add(vo1);
        list.add(vo2);
        list.add(vo3);
        list.add(vo4);
        list.add(vo5);
        list.add(vo6);
        return list;
    }

    /**
     * 获取车辆详情
     *
     * @param plateNumber
     * @return
     */
    @Override
    public OverallMonitorVO getVehicleInfo(String plateNumber) {
        OverallMonitorVO vo = runningbusMapper.queryVehicleInfo(plateNumber);
        if (StrUtil.isNotBlank(vo.getDir())) {
            vo.setDirection(DirectionEnum.get(vo.getDir()));
        }
        return vo;
    }

    @Override
    public RunningbusVO getRunningbusInfo(String plateNumber) {
        LambdaQueryWrapper<TbGisRunningbus> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(TbGisRunningbus::getPlateNumber, plateNumber).last(" limit 1 ");
        TbGisRunningbus tbGisRunningbus = runningbusMapper.selectOne(queryWrapper);
        if (Objects.nonNull(tbGisRunningbus)) {
            RunningbusVO vo = new RunningbusVO();
            BeanUtil.copyProperties(tbGisRunningbus, vo);

            LambdaQueryWrapper<TbVehicle> queryVehicleWrapper = new LambdaQueryWrapper<>();
            queryVehicleWrapper.eq(TbVehicle::getPlateNumber, plateNumber);
            TbVehicle vehicle = tbVehicleMapper.selectOne(queryVehicleWrapper);
            if (Objects.nonNull(vehicle)) {
                vo.setVehicleType(vehicle.getVehicleType());
                vo.setVinCode(vehicle.getVinCode());
            }
            if (StrUtil.isNotBlank(tbGisRunningbus.getVehicleJson())) {
                JSONObject object = JSONUtil.parseObj(tbGisRunningbus.getVehicleJson());
                object.set("totalMileage", String.format("%.2f",object.get("totalMileage")));
                object.set("speed", String.format("%.2f",object.get("speed")));
                object.set("totalVoltage", String.format("%.2f",object.get("totalVoltage")));
                object.set("totalCurrent", String.format("%.2f",object.get("totalCurrent")));
                vo.setVehicleJSONObject(object);
            }
            if (StrUtil.isNotBlank(tbGisRunningbus.getAlarmDataJson())) {
                vo.setAlarmDataJSONObject(JSONUtil.parseObj(tbGisRunningbus.getAlarmDataJson()));
            }
            if (StrUtil.isNotBlank(tbGisRunningbus.getElectricMotorJson()) && !"null".equals(tbGisRunningbus.getElectricMotorJson())) {
                String handleElectricMotorJson = FormatUtil.replaceStatusValues("status",tbGisRunningbus.getElectricMotorJson());
                vo.setElectricMotorJSONArray(JSONUtil.parseArray(handleElectricMotorJson));
                //转速取最大值
                for (int i = 0; i < vo.getElectricMotorJSONArray().size(); i++) {
                    if ((Integer) vo.getElectricMotorJSONArray().getJSONObject(i).get("rpm") > vo.getMaxRpm()) {
                        vo.setMaxRpm((Integer) vo.getElectricMotorJSONArray().getJSONObject(i).get("rpm"));
                    }
                }
            }
            //补充保险信息
            LambdaQueryWrapper<TbInsurance> insuranceQuery = new LambdaQueryWrapper<>();
            insuranceQuery.eq(TbInsurance::getPlateNumber, plateNumber).orderByDesc(TbInsurance::getCompulsoryValidTime)
                    .last("limit 1");
            TbInsurance insurance = iTbInsuranceService.getOne(insuranceQuery);
            if(Objects.nonNull(insurance)){
                vo.setInsuranceInfo(insurance);
            }else{
                vo.setInsuranceInfo(new TbInsurance());
            }
            return vo;
        }
        return null;
    }

    @Override
    public IPage<TbGisFenceAlarm> pageFenceList(List<String> projectIds, String plateNumber, String startTime, String endTime, Integer status,
                                                Integer pageNo, Integer pageSize) {

        Page<TbGisFenceAlarm> page = new Page<>(pageNo, pageSize);
        LambdaQueryWrapper<TbGisFenceAlarm> lambdaQueryWrapper = new LambdaQueryWrapper<TbGisFenceAlarm>();
        if (!projectIds.isEmpty()) {
            lambdaQueryWrapper.in(TbGisFenceAlarm::getProjectId, projectIds);
        }
        if (StringUtils.isNotBlank(startTime) && StringUtils.isNotBlank(endTime)) {
            // 如果 startTime 和 endTime 都提供了，使用 BETWEEN 条件
            lambdaQueryWrapper.between(TbGisFenceAlarm::getStartTime, DateUtil.parse(startTime), DateUtil.parse(endTime));
        } else if (StringUtils.isNotBlank(startTime)) {
            // 如果只提供了 startTime，使用 大于等于
            lambdaQueryWrapper.ge(TbGisFenceAlarm::getStartTime, DateUtil.parse(startTime));
        } else if (StringUtils.isNotBlank(endTime)) {
            // 如果只提供了 endTime，使用 小于等于
            lambdaQueryWrapper.le(TbGisFenceAlarm::getStartTime, DateUtil.parse(endTime));
        }
        if (StringUtils.isNotBlank(plateNumber)) {
            lambdaQueryWrapper.eq(TbGisFenceAlarm::getPlateNumber, plateNumber);
        }
        //没有结束时间的就是未结束
        if (Objects.nonNull(status)) {
            if(status == 0){
                //未结束
                lambdaQueryWrapper.isNull(TbGisFenceAlarm::getEndTime);
            }else if(status == 1){
                //已结束
                lambdaQueryWrapper.isNotNull(TbGisFenceAlarm::getEndTime);
            }
        }
        lambdaQueryWrapper.orderByDesc(TbGisFenceAlarm::getStartTime);
        IPage<TbGisFenceAlarm> pageList = tbGisFenceAlarmService.page(page, lambdaQueryWrapper);
        return pageList;
    }

    @Override
    public IPage<TbGisSpeedingAlarm> pageSpeedAlarmList(List<String> projectIds, String plateNumber, String startTime, String endTime, Integer status,
                                                        Integer pageNo, Integer pageSize) {
        Page<TbGisSpeedingAlarm> page = new Page<>(pageNo, pageSize);
        LambdaQueryWrapper<TbGisSpeedingAlarm> lambdaQueryWrapper = new LambdaQueryWrapper<TbGisSpeedingAlarm>();
        if (!projectIds.isEmpty()) {
            lambdaQueryWrapper.in(TbGisSpeedingAlarm::getProjectId, projectIds);
        }
        if (StringUtils.isNotBlank(startTime) && StringUtils.isNotBlank(endTime)) {
            // 如果 startTime 和 endTime 都提供了，使用 BETWEEN 条件
            lambdaQueryWrapper.between(TbGisSpeedingAlarm::getStartTime, DateUtil.parse(startTime), DateUtil.parse(endTime));
        } else if (StringUtils.isNotBlank(startTime)) {
            // 如果只提供了 startTime，使用 大于等于
            lambdaQueryWrapper.ge(TbGisSpeedingAlarm::getStartTime, DateUtil.parse(startTime));
        } else if (StringUtils.isNotBlank(endTime)) {
            // 如果只提供了 endTime，使用 小于等于
            lambdaQueryWrapper.le(TbGisSpeedingAlarm::getStartTime, DateUtil.parse(endTime));
        }
        if (StringUtils.isNotBlank(plateNumber)) {
            lambdaQueryWrapper.eq(TbGisSpeedingAlarm::getPlateNumber, plateNumber);
        }
        //没有结束时间的就是未结束
        if (Objects.nonNull(status)) {
            if(status == 0){
                //未结束
                lambdaQueryWrapper.isNull(TbGisSpeedingAlarm::getEndTime);
            }else if(status == 1){
                //已结束
                lambdaQueryWrapper.isNotNull(TbGisSpeedingAlarm::getEndTime);
            }
        }
        lambdaQueryWrapper.orderByDesc(TbGisSpeedingAlarm::getStartTime);
        IPage<TbGisSpeedingAlarm> pageList = tbGisSpeedingAlarmService.page(page, lambdaQueryWrapper);
        return pageList;
    }

    @Override
    public IPage<TbGisStoppingAlarm> pageStopAlarmList(List<String> projectIds, String plateNumber, String startTime, String endTime, Integer status,
                                                       Integer pageNo, Integer pageSize) {
        Page<TbGisStoppingAlarm> page = new Page<>(pageNo, pageSize);
        LambdaQueryWrapper<TbGisStoppingAlarm> lambdaQueryWrapper = new LambdaQueryWrapper<TbGisStoppingAlarm>();
        if (!projectIds.isEmpty()) {
            lambdaQueryWrapper.in(TbGisStoppingAlarm::getProjectId, projectIds);
        }
        if (StringUtils.isNotBlank(startTime) && StringUtils.isNotBlank(endTime)) {
            // 如果 startTime 和 endTime 都提供了，使用 BETWEEN 条件
            lambdaQueryWrapper.between(TbGisStoppingAlarm::getStartTime, DateUtil.parse(startTime), DateUtil.parse(endTime));
        } else if (StringUtils.isNotBlank(startTime)) {
            // 如果只提供了 startTime，使用 大于等于
            lambdaQueryWrapper.ge(TbGisStoppingAlarm::getStartTime, DateUtil.parse(startTime));
        } else if (StringUtils.isNotBlank(endTime)) {
            // 如果只提供了 endTime，使用 小于等于
            lambdaQueryWrapper.le(TbGisStoppingAlarm::getStartTime, DateUtil.parse(endTime));
        }
        if (StringUtils.isNotBlank(plateNumber)) {
            lambdaQueryWrapper.eq(TbGisStoppingAlarm::getPlateNumber, plateNumber);
        }
        //没有结束时间的就是未结束
        if (Objects.nonNull(status)) {
            if(status == 0){
                //未结束
                lambdaQueryWrapper.isNull(TbGisStoppingAlarm::getEndTime);
            }else if(status == 1){
                //已结束
                lambdaQueryWrapper.isNotNull(TbGisStoppingAlarm::getEndTime);
            }
        }
        lambdaQueryWrapper.orderByDesc(TbGisStoppingAlarm::getStartTime);
        IPage<TbGisStoppingAlarm> pageList = tbGisStoppingAlarmService.page(page, lambdaQueryWrapper);
        return pageList;
    }

}
