package com.gci.cdnev.smallprogram.modules.projects.controller;


import java.util.List;

import com.gci.cdnev.smallprogram.common.log.annotation.AddSysLog;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gci.cdnev.smallprogram.annotation.CurrentUser;
import com.gci.cdnev.smallprogram.annotation.Query.handle.QueryGenerator;
import com.gci.cdnev.smallprogram.common.vo.Result;
import com.gci.cdnev.smallprogram.constant.CommonConstant;
import com.gci.cdnev.smallprogram.modules.driver.service.ITbDriverService;
import com.gci.cdnev.smallprogram.modules.projects.entity.Projects;
import com.gci.cdnev.smallprogram.modules.projects.service.IProjectsService;
import com.gci.cdnev.smallprogram.modules.wx.entity.JwtUser;

import io.jsonwebtoken.lang.Collections;
import lombok.extern.slf4j.Slf4j;

/**
 * <p>
 * 项目基础信息表 前端控制器
 * </p>
 *
 * @author xushuhan
 * @since 2023-08-25
 */
@Slf4j
@RestController
@RequestMapping("/api/projects")
public class ProjectsController {

	@Autowired
	private IProjectsService projectsService;
	@Autowired
	private ITbDriverService tbDriverService;


	@AddSysLog(value = "项目档案-列表查询")
	@GetMapping("/list")
	public Result<?> list(@CurrentUser JwtUser jwtUser,Projects item) {
		LambdaQueryWrapper<Projects> queryWrapper = QueryGenerator.initQueryWrapper(Projects.class, item);
		queryWrapper.in(!Collections.isEmpty(jwtUser.getProjectIds()),Projects::getId, jwtUser.getProjectIds());
		queryWrapper.eq(Projects::getStatus, CommonConstant.STATUS_1);
		queryWrapper.orderByDesc(Projects::getCreateTime);
		List<Projects> list = projectsService.list(queryWrapper);
		return Result.ok(list);
	}

	@AddSysLog(value = "项目档案-分页查询")
	@GetMapping("/page")
	public Result<?> page(@CurrentUser JwtUser jwtUser,Projects item,@RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
						  @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize) {

		LambdaQueryWrapper<Projects> queryWrapper = QueryGenerator.initQueryWrapper(Projects.class, item);
		queryWrapper.in(!Collections.isEmpty(jwtUser.getProjectIds()),Projects::getId, jwtUser.getProjectIds());
		queryWrapper.eq(Projects::getStatus, CommonConstant.STATUS_1);
		queryWrapper.orderByDesc(Projects::getCreateTime);
		Page<Projects> page = new Page<Projects>(pageNo, pageSize);
		IPage<Projects> pageList = projectsService.page(page,queryWrapper);
		for(Projects pro:pageList.getRecords()) {
			projectsService.initBusId(pro);
		}
		return Result.ok(pageList);
	}


}
