package com.gci.cdnev.smallprogram.modules.projects.controller;


import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gci.cdnev.smallprogram.annotation.CurrentUser;
import com.gci.cdnev.smallprogram.annotation.Query.handle.QueryGenerator;
import com.gci.cdnev.smallprogram.common.log.annotation.AddSysLog;
import com.gci.cdnev.smallprogram.common.util.MinioUtils;
import com.gci.cdnev.smallprogram.common.validated.SaveValidated;
import com.gci.cdnev.smallprogram.common.validated.UpdateValidated;
import com.gci.cdnev.smallprogram.common.vo.Result;
import com.gci.cdnev.smallprogram.enums.FileExtension;
import com.gci.cdnev.smallprogram.modules.projects.entity.TbVehicleCheck;
import com.gci.cdnev.smallprogram.modules.projects.service.ITbVehicleCheckDetailService;
import com.gci.cdnev.smallprogram.modules.projects.service.ITbVehicleCheckProjectService;
import com.gci.cdnev.smallprogram.modules.projects.service.ITbVehicleCheckService;
import com.gci.cdnev.smallprogram.modules.wx.entity.JwtUser;

import cn.hutool.core.util.ObjectUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * <p>
 * 车辆验收管理 前端控制器
 * </p>
 *
 * @author xushuhan
 * @since 2023-09-11
 */
@Slf4j
@RestController
@RequestMapping("/api/vehicleCheck")
public class TbVehicleCheckController {

	@Autowired
	private ITbVehicleCheckService tbVehicleCheckService;
	@Autowired
	private ITbVehicleCheckProjectService tbVehicleCheckProjectService;
	@Autowired
	private ITbVehicleCheckDetailService tbVehicleCheckDetailService;
	@Autowired
    private MinioUtils minioUtils;

	@AddSysLog(value = "车辆验收管理-列表查询")
	@GetMapping("/list")
	public Result<?> list(TbVehicleCheck item) {
		if(!StringUtils.hasLength(item.getCheckType())) {
			item.setCheckType("1");
		}
		LambdaQueryWrapper<TbVehicleCheck> queryWrapper = QueryGenerator.initQueryWrapper(TbVehicleCheck.class, item);
		if(StringUtils.hasLength(item.getPlateNumber())||StringUtils.hasLength(item.getFrameNumber())) {
			String where = "";
			if(StringUtils.hasLength(item.getPlateNumber())) {
				where += " and cd.plate_number like '%"+item.getPlateNumber()+"%'";
			}
			if(StringUtils.hasLength(item.getFrameNumber())) {
				where += " and cd.frame_number like '%"+item.getFrameNumber()+"%'";
			}
			String sql = "select 1 from tb_vehicle_check_detail cd where tb_vehicle_check.id = cd.check_id "+where;
			queryWrapper.exists(sql);
		}
		if("1".equals(item.getCheckType())) {
			queryWrapper.orderByAsc(TbVehicleCheck::getStatus);
		}else {
			queryWrapper.orderByAsc(TbVehicleCheck::getReceiveStatus);
		}
		queryWrapper.orderByDesc(TbVehicleCheck::getCreateTime);
		List<TbVehicleCheck> list = tbVehicleCheckService.list(queryWrapper);
		//tbVehicleCheckService.initItem(list);
		for(TbVehicleCheck template:list) {
			tbVehicleCheckDetailService.getVehiclesByCheckId(template);
			tbVehicleCheckProjectService.getProjectsByCheckId(template);
			template.setVehicleCount(template.getVehicleList().size());
		}
		return Result.ok(list);
	}

	@AddSysLog(value = "车辆验收管理-分页查询")
	@GetMapping("/page")
	public Result<?> page(TbVehicleCheck item,@RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
						  @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize) {
		if(!StringUtils.hasLength(item.getCheckType())) {
			item.setCheckType("1");
		}
		LambdaQueryWrapper<TbVehicleCheck> queryWrapper = QueryGenerator.initQueryWrapper(TbVehicleCheck.class, item);
		if(StringUtils.hasLength(item.getPlateNumber())||StringUtils.hasLength(item.getFrameNumber())) {
			String where = "";
			if(StringUtils.hasLength(item.getPlateNumber())) {
				where += " and cd.plate_number like '%"+item.getPlateNumber()+"%'";
			}
			if(StringUtils.hasLength(item.getFrameNumber())) {
				where += " and cd.frame_number like '%"+item.getFrameNumber()+"%'";
			}
			String sql = "select 1 from tb_vehicle_check_detail cd where tb_vehicle_check.id = cd.check_id "+where;
			queryWrapper.exists(sql);
		}
		if("1".equals(item.getCheckType())) {
			queryWrapper.orderByAsc(TbVehicleCheck::getStatus);
		}else {
			queryWrapper.orderByAsc(TbVehicleCheck::getReceiveStatus);
		}
		queryWrapper.orderByDesc(TbVehicleCheck::getCreateTime);
		Page<TbVehicleCheck> page = new Page<TbVehicleCheck>(pageNo, pageSize);
		IPage<TbVehicleCheck> pageList = tbVehicleCheckService.page(page,queryWrapper);
		//tbVehicleCheckService.initItem(pageList.getRecords());
		for(TbVehicleCheck template:pageList.getRecords()) {
			//获取租赁收车
			tbVehicleCheckService.getChildren(template);
			tbVehicleCheckDetailService.getVehiclesByCheckId(template);
			tbVehicleCheckProjectService.getProjectsByCheckId(template);
			template.setVehicleCount(template.getVehicleList().size());
		}
		return Result.ok(pageList);
	}

	@AddSysLog(value = "车辆验收管理-新增")
	@PostMapping("/add")
	public Result<?> add(@CurrentUser JwtUser jwtUser,@RequestBody @Validated(SaveValidated.class) TbVehicleCheck item){
		if(!StringUtils.hasLength(item.getCheckType())) {
			return Result.error("验收类型不能为空");
		}
		item.setSysOrgCode(jwtUser.getSysOrgCode());
		item.setCreateBy(jwtUser.getUserName());
		item.setSourceType("2");
		String error = tbVehicleCheckService.saveItem(item);
		return Result.ok("操作成功。"+error);
	}

	@PutMapping("/edit")
	public Result<?> edit(@CurrentUser JwtUser jwtUser,@RequestBody @Validated(UpdateValidated.class) TbVehicleCheck item){
		if(!StringUtils.hasLength(item.getCheckType())) {
			return Result.error("验收类型不能为空");
		}
		item.setSysOrgCode(jwtUser.getSysOrgCode());
		item.setCreateBy(jwtUser.getUserName());
		item.setSourceType("2");
		String error = tbVehicleCheckService.saveItem(item);
		return Result.ok("操作成功。"+error);
	}

    /**
     * 文件上传
     */
	@AddSysLog(value = "车辆验收管理-编辑")
    @PostMapping("/upload")
    public Result<?> upload(HttpServletRequest request) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        MultipartFile file = multipartRequest.getFile("file");
        if (ObjectUtil.isEmpty(file)) {
            return Result.error("请正确上传文件！");
        }
        return Result.ok(minioUtils.uploadFileV2Valid(file, FileExtension.getCommonFileTypes()));
    }

	@AddSysLog(value = "车辆验收管理-删除")
	@DeleteMapping("/delete")
	public Result<?> delete(TbVehicleCheck item){
		tbVehicleCheckService.removeById(item.getId());
		return Result.ok();
	}

}
