package com.gci.cdnev.smallprogram.modules.projects.entity;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

import com.gci.cdnev.smallprogram.annotation.Query.SelectQuery;
import com.gci.cdnev.smallprogram.common.dict.annotation.Dict;
import com.gci.cdnev.smallprogram.common.dict.excel.DictConverter;
import org.springframework.format.annotation.DateTimeFormat;

import com.alibaba.excel.annotation.ExcelIgnore;
import com.alibaba.excel.annotation.ExcelProperty;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.annotation.JsonFormat;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Getter;
import lombok.Setter;

/**
 * <p>
 * 项目基础信息表
 * </p>
 *
 * @author xushuhan
 * @since 2023-08-25
 */
@Getter
@Setter
@TableName("tb_projects")
@ApiModel(value = "Projects对象", description = "项目基础信息表")
public class Projects implements Serializable {

    private static final long serialVersionUID = 1L;

    @TableId(value = "id", type = IdType.ASSIGN_ID)
    @ExcelIgnore
    private String id;

    @ExcelProperty(value = "项目名称", index = 0)
    @ApiModelProperty("项目名称，必填且唯一")
    @SelectQuery(Type = SelectQuery.SelectType.LIKE)
    private String projectName;

    @ExcelProperty(value = "项目负责人", index = 1)
    @ApiModelProperty("负责人的姓名")
    @SelectQuery(Type = SelectQuery.SelectType.LIKE)
    private String projectManager;

    @ExcelIgnore
    @ApiModelProperty("对方单位的名称")
    private String counterparty;

    @ExcelIgnore
    @ApiModelProperty("联系人的姓名")
    private String contactPerson;

    @ExcelProperty(value = "电话号码", index = 2)
    @ApiModelProperty("联系人的电话号码")
    @SelectQuery(Type = SelectQuery.SelectType.LIKE)
    private String contactPhone;

    @ExcelProperty(value = "开始日期", index = 3)
    @ApiModelProperty("项目开始日期")
    @JsonFormat(timezone = "GMT+8",pattern = "yyyy-MM-dd")
    @DateTimeFormat(pattern="yyyy-MM-dd")
    private Date startDate;

    @ExcelProperty(value = "结束日期", index = 4)
    @ApiModelProperty("项目结束日期")
    @JsonFormat(timezone = "GMT+8",pattern = "yyyy-MM-dd")
    @DateTimeFormat(pattern="yyyy-MM-dd")
    private Date endDate;

    @ExcelIgnore
    @ApiModelProperty("所属机构编码")
    private String sysOrgCode;

    @ExcelIgnore
    @ApiModelProperty("创建时间")
    @JsonFormat(timezone = "GMT+8",pattern = "yyyy-MM-dd HH:mm:ss")
    @DateTimeFormat(pattern="yyyy-MM-dd HH:mm:ss")
    private Date createTime;

    @ExcelIgnore
    @ApiModelProperty("创建人")
    private String createBy;

    @ExcelIgnore
    @ApiModelProperty("更新时间")
    @JsonFormat(timezone = "GMT+8",pattern = "yyyy-MM-dd HH:mm:ss")
    @DateTimeFormat(pattern="yyyy-MM-dd HH:mm:ss")
    private Date updateTime;

    @ExcelIgnore
    @ApiModelProperty("更新人")
    private String updateBy;

    @ApiModelProperty("废弃状态 0-正常 1-废弃")
    private Integer cancelFlag;

    @ApiModelProperty("历史车辆")
    @ExcelIgnore
    private String hisVehicleId;

    @ExcelProperty(value = "状态", converter = DictConverter.class, index = 5)
    @ApiModelProperty("状态(1启用，0禁用)")
    @Dict(dicCode = "status")
    private String status;

    @ExcelIgnore
    @ApiModelProperty("删除标志")
    private Integer delFlag;

    @ExcelProperty(value = "车牌号", converter = DictConverter.class, index = 6)
    @TableField(exist = false)
    @Dict(dictTable = "tb_vehicle", dicText = "plate_number", dicCode = "id")
    public String busIds;

}
