package com.gci.cdnev.smallprogram.modules.projects.service.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import com.gci.cdnev.smallprogram.modules.projects.entity.ProjectUserRel;
import com.gci.cdnev.smallprogram.modules.projects.mapper.ProjectUserRelMapper;
import com.gci.cdnev.smallprogram.modules.system.entity.SysRole;
import com.gci.cdnev.smallprogram.modules.wx.entity.JwtUser;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.ArrayUtils;
import org.apache.shiro.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gci.cdnev.smallprogram.constant.CommonConstant;
import com.gci.cdnev.smallprogram.exception.GciBootException;
import com.gci.cdnev.smallprogram.modules.basedata.entity.TbVehicle;
import com.gci.cdnev.smallprogram.modules.basedata.mapper.TbVehicleMapper;
import com.gci.cdnev.smallprogram.modules.basedata.service.ITbVehicleService;
import com.gci.cdnev.smallprogram.modules.driver.entity.TbDriver;
import com.gci.cdnev.smallprogram.modules.driver.service.ITbDriverService;
import com.gci.cdnev.smallprogram.modules.projects.entity.Projects;
import com.gci.cdnev.smallprogram.modules.projects.mapper.ProjectsMapper;
import com.gci.cdnev.smallprogram.modules.projects.service.IProjectsService;
import com.gci.cdnev.smallprogram.modules.system.entity.SysDepart;
import com.gci.cdnev.smallprogram.modules.system.entity.SysUser;
import com.gci.cdnev.smallprogram.modules.system.service.ISysDepartService;
import com.gci.cdnev.smallprogram.modules.system.service.ISysRoleService;


/**
 * <p>
 * 项目基础信息表 服务实现类
 * </p>
 *
 * @author xushuhan
 * @since 2023-08-25
 */
@Service
public class ProjectsServiceImpl extends ServiceImpl<ProjectsMapper, Projects> implements IProjectsService {

	@Autowired
    private ITbVehicleService iTbVehicleService;

	@Autowired
	private TbVehicleMapper tbVehicleMapper;
	@Autowired
	private ISysDepartService sysDepartService;
	@Autowired
	private ISysRoleService sysRoleService;
	@Autowired
	private ITbDriverService tbDriverService;
	@Autowired
	private ProjectsMapper projectsMapper;
	@Autowired
	private ProjectUserRelMapper projectUserRelMapper;

	@Override
	public void saveItem(Projects item) {
		checkItem(item);
		String busIds = item.getBusIds();
		saveOrUpdate(item);

		tbVehicleMapper.clearProductId(item.getId());
		if(StringUtils.hasLength(busIds)) {
			QueryWrapper<TbVehicle> queryWrapper = new QueryWrapper<>();
			queryWrapper.in("id", busIds.split(","));
			//queryWrapper.isNull("project_id");
			queryWrapper.and(w->w.eq("project_id", "").or().isNull("project_id"));
			TbVehicle bus = new TbVehicle();
			bus.setProjectId(item.getId());
			tbVehicleMapper.update(bus, queryWrapper);
		}
	}

	public void checkItem(Projects item) {
		QueryWrapper<Projects> queryWrapper = new QueryWrapper<>();
		queryWrapper.ne(StringUtils.hasLength(item.getId()),"id", item.getId());
		queryWrapper.eq("project_name", item.getProjectName());
		int count = count(queryWrapper);
		if(count>0) {
			throw new GciBootException("项目名称不能重复");
		}
	}

	@Override
	public void initBusId(Projects item) {
		QueryWrapper<TbVehicle> queryWrapper = new QueryWrapper<>();
		queryWrapper.eq("project_id", item.getId());
		List<TbVehicle> list = tbVehicleMapper.selectList(queryWrapper);
		String busIds = null;
		for (TbVehicle tbVehicle : list) {
			if(busIds==null) {
				busIds = tbVehicle.getId();
			}else {
				busIds += ","+tbVehicle.getId();
			}
		}
		item.setBusIds(busIds);
	}

	@Override
	public void updateStatus(Projects item) {
		if(CommonConstant.STATUS_1.equals(item.getStatus())) {
			initBusId(item);
			if(StringUtils.hasLength(item.getBusIds())) {
				item.setHisVehicleId(item.getBusIds());
			}
			item.setStatus(CommonConstant.STATUS_0);
			updateById(item);
			tbVehicleMapper.clearProductId(item.getId());
		}else if(CommonConstant.STATUS_0.equals(item.getStatus())) {
			item.setStatus(CommonConstant.STATUS_1);
			updateById(item);
		}
	}


	@Override
	public void initUserPower(SysUser sysUser) {
		List<String> projectIds = new ArrayList<>();
		Set<String> roleCodes = sysRoleService.getRoleCodeSetByUserId(sysUser.getId());
		List<String> listDepart = new ArrayList<>();
		String userType = null;

		List<SysDepart> departList = sysDepartService.getRoleByUserId(sysUser.getId());
		if(CollectionUtils.isNotEmpty(departList)){
			for(SysDepart depart: departList){
				listDepart.add(depart.getOrgCode());
			}
		}
		if(roleCodes.contains(CommonConstant.MAINTENANCE_ADMIN_ROLE_CODE)|| roleCodes.contains(CommonConstant.MAINTENANCE_LEADER_ROLE_CODE)) {
			userType = "4";
		}else if(roleCodes.contains(CommonConstant.DRIVER_ROLE_CODE)) {
			userType = "1";
		}else if(roleCodes.contains(CommonConstant.CUSTOM_ADMIN_ROLE_CODE)) {
			userType = "3";
		}else if(roleCodes.contains(CommonConstant.REPAIR_ADMIN_ROLE_CODE)) {
			userType = "2";
		}

		//userType 1司机、2维修厂、3租赁方（客户）、4维保部
		sysUser.setType(userType);
		if("1".equals(userType)) {
			List<TbDriver> driver = tbDriverService.lambdaQuery().eq(TbDriver::getUserId, sysUser.getId()).list();
	    	if(driver.size()>0) {
	    		sysUser.setEmployeeId(driver.get(0).getId());
	    		sysUser.setEmployeeName(driver.get(0).getName());
	    		projectIds = tbDriverService.getProjectIds(sysUser.getEmployeeId());
	    	}
		}else if("3".equals(userType)) {
			//查询租赁客户项目id列表
			Set<String> projectsByDepartCodes = baseMapper.getCustomerProjectsByDepartCodes(listDepart);
			if(CollectionUtils.isNotEmpty(projectsByDepartCodes)){
				projectIds.addAll(projectsByDepartCodes);
			}
		}
		if(CollectionUtils.isEmpty(projectIds)) {
			projectIds = new ArrayList<>();
			//无权限给一个临时值
			projectIds.add("0000");
		}
		if("2".equals(userType)) {
			//查询维修厂权限 项目权限是全部，维修单只能查自己的
			//全部权限
			projectIds = new ArrayList<>();
		}else if("4".equals(userType)) {
			//全部权限
			projectIds = new ArrayList<>();
		}
		
		sysUser.setProjectIds(projectIds);

	}

	@Override
	public List<String> queryProjectListOfUser(JwtUser jwtUser) {
		final List<String> result = new ArrayList<>();
		// 获取登录用户信息
		if(jwtUser==null || org.apache.commons.lang3.StringUtils.isEmpty(jwtUser.getId())){
			throw new RuntimeException("用户未登录");
		}
		List<SysDepart> departList = sysDepartService.getDepartByUserId(jwtUser.getId());
		//flag标识是否是客户公司身份 1_普通部门,2_维修厂,3_客户公司
		int departCategory = 0;
		boolean flag = false;
		List<String> listDepart = new ArrayList<>();
		if(CollectionUtils.isNotEmpty(departList)){
			for(SysDepart depart: departList){
				if(depart.getOrgCategory()==1){
					departCategory = 1;
					break;
				}else if(depart.getOrgCategory()==2){
					if(departCategory == 0){
						departCategory = 2;
					}
					return result;
				}else if(depart.getOrgCategory()==3){
					departCategory = 3;
					listDepart.add(depart.getOrgCode());
				}
			}
		}
		//按照部门类型依次处理：1、内部员工，2、客户人员（司机、客户管理员），3、其他人员
		if(departCategory == 1){
			List<ProjectUserRel> listProjectUserRel = projectUserRelMapper.selectList(new QueryWrapper<ProjectUserRel>().eq("user_id", jwtUser.getId()));
			listProjectUserRel.stream().forEach(rel -> {
				result.add(rel.getProjectId());
			});
		}else if(departCategory == 3){
			TbDriver tbDiver = tbDriverService.getOne(new QueryWrapper<TbDriver>().eq("user_id", jwtUser.getId()));
			if(tbDiver != null && tbDiver.getId() != null && org.apache.commons.lang3.StringUtils.isNotEmpty(tbDiver.getProjectId())){
				//处理司机的情况
				String[] projectIdStr = tbDiver.getProjectId().split(",");
				if(ArrayUtils.isNotEmpty(projectIdStr)){
					for(String projectIdDriver: projectIdStr){
						result.add(projectIdDriver);
					}
				}
			}else{
				//处理客户其他人员的情况
				List<Projects> projectsByDepartCodes = projectsMapper.getProjectsByDepartCodes(listDepart);
				projectsByDepartCodes.stream().forEach(proj -> {
					result.add(proj.getId());
				});
			}
		}
		return result;
	}


	/**
	 * 处理逻辑：有效看是否有普通部门，作为内部员工处理，其次判断客户公司和维修厂
	 * @return
	 */
	@Override
	public Integer judgeDepartCategory(JwtUser jwtUser) {
		// 获取登录用户信息
		if(jwtUser==null || org.apache.commons.lang3.StringUtils.isEmpty(jwtUser.getId())){
			throw new RuntimeException("用户未登录");
		}
		List<SysDepart> departList = sysDepartService.getDepartByUserId(jwtUser.getId());
		//判断用户身份： 1_普通部门,2_维修厂,3_司机，4_客户公司其他人员，5_超级管理员，0、其他身份
		int departCategory = 0;
		//获取角色，判断是否包含超级管理员
		List<SysRole> listRole = sysRoleService.getRoleByUserId(jwtUser.getId());
		if(CollectionUtils.isEmpty(listRole)){
			//角色配置为空
			throw new RuntimeException("角色配置为空");
		}
		StringBuilder sbRole = new StringBuilder();
		listRole.forEach(lRole -> {
			sbRole.append(","+lRole.getRoleCode());
		});
		String strRole = sbRole.toString();
		if(strRole.contains("PM0000")){
			return 5;
		}

		List<String> listDepart = new ArrayList<>();
		if(CollectionUtils.isNotEmpty(departList)){
			for(SysDepart depart: departList){
				if(depart.getOrgCategory()==1){
					departCategory = 1;
					break;
				}else if(depart.getOrgCategory()==2){
					if(departCategory == 0){
						departCategory = 2;
					}
				}else if(depart.getOrgCategory()==3){
					TbDriver tbDiver = tbDriverService.getOne(new QueryWrapper<TbDriver>().eq("user_id", jwtUser.getId()));
					if(tbDiver != null && tbDiver.getId() != null){
						//处理司机的情况
						departCategory = 3;
					}else{
						//公司的其他人员
						if(departCategory == 0 || departCategory == 2){
							departCategory = 4;
						}
					}
				}
			}
		}
		return departCategory;
	}


}
