package com.gci.cdnev.smallprogram.modules.projects.service.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gci.cdnev.smallprogram.modules.basedata.entity.TbCustomerArchives;
import com.gci.cdnev.smallprogram.modules.basedata.entity.TbInsuranceCompanyArchives;
import com.gci.cdnev.smallprogram.modules.basedata.entity.TbRepairShopArchives;
import com.gci.cdnev.smallprogram.modules.basedata.entity.TbVehicleSupplierArchives;
import com.gci.cdnev.smallprogram.modules.basedata.service.ITbCustomerArchivesService;
import com.gci.cdnev.smallprogram.modules.basedata.service.ITbInsuranceCompanyArchivesService;
import com.gci.cdnev.smallprogram.modules.basedata.service.ITbRepairShopArchivesService;
import com.gci.cdnev.smallprogram.modules.basedata.service.ITbVehicleSupplierArchivesService;
import com.gci.cdnev.smallprogram.modules.projects.entity.TbContract;
import com.gci.cdnev.smallprogram.modules.projects.entity.TbContractVehicle;
import com.gci.cdnev.smallprogram.modules.projects.entity.TbVehicleCheckDetail;
import com.gci.cdnev.smallprogram.modules.projects.mapper.TbContractMapper;
import com.gci.cdnev.smallprogram.modules.projects.service.ITbContractService;
import com.gci.cdnev.smallprogram.modules.projects.service.ITbContractVehicleService;
import com.gci.cdnev.smallprogram.modules.projects.service.ITbVehicleCheckDetailService;


/**
 * <p>
 * 合同档案 服务实现类
 * </p>
 *
 * @author xushuhan
 * @since 2023-09-01
 */
@Service
public class TbContractServiceImpl extends ServiceImpl<TbContractMapper, TbContract> implements ITbContractService {

	@Autowired
    private ITbCustomerArchivesService tbCustomerArchivesService;
	@Autowired
    private ITbInsuranceCompanyArchivesService tbInsuranceCompanyArchivesService;
	@Autowired
    private ITbRepairShopArchivesService tbRepairShopArchivesService;
	@Autowired
    private ITbVehicleSupplierArchivesService tbVehicleSupplierArchivesService;
	@Autowired
	private ITbContractVehicleService contractVehicleService;
	@Autowired
	private ITbVehicleCheckDetailService vehicleCheckDetailService;

	//根据合同类型获取签约方列表
	@Override
	public List<TbContract> getSigningList(String contractType,String sysOrgCode) {
		List<TbContract> dataList = new ArrayList<>();
		if("1".equals(contractType)) {
			QueryWrapper<TbCustomerArchives> queryWrapper = new QueryWrapper<>();
			//queryWrapper.likeRight(StringUtils.hasLength(sysOrgCode),"sys_org_code", sysOrgCode);
			List<TbCustomerArchives> list = tbCustomerArchivesService.list(queryWrapper);
			for(TbCustomerArchives item:list) {
				TbContract con = new TbContract();
				con.setContractType(contractType);
				con.setId(item.getId());
				con.setName(item.getName());
				dataList.add(con);
			}
		}else if("2".equals(contractType)) {
			QueryWrapper<TbInsuranceCompanyArchives> queryWrapper = new QueryWrapper<>();
			//queryWrapper.likeRight(StringUtils.hasLength(sysOrgCode),"sys_org_code", sysOrgCode);
			List<TbInsuranceCompanyArchives> list = tbInsuranceCompanyArchivesService.list(queryWrapper);
			for(TbInsuranceCompanyArchives item:list) {
				TbContract con = new TbContract();
				con.setContractType(contractType);
				con.setId(item.getId());
				con.setName(item.getName());
				dataList.add(con);
			}
		}else if("3".equals(contractType)) {
			QueryWrapper<TbRepairShopArchives> queryWrapper = new QueryWrapper<>();
			//queryWrapper.likeRight(StringUtils.hasLength(sysOrgCode),"sys_org_code", sysOrgCode);
			List<TbRepairShopArchives> list = tbRepairShopArchivesService.list(queryWrapper);
			for(TbRepairShopArchives item:list) {
				TbContract con = new TbContract();
				con.setContractType(contractType);
				con.setId(item.getId());
				con.setName(item.getName());
				dataList.add(con);
			}
		}else if("4".equals(contractType)) {
			QueryWrapper<TbVehicleSupplierArchives> queryWrapper = new QueryWrapper<>();
			//queryWrapper.likeRight(StringUtils.hasLength(sysOrgCode),"sys_org_code", sysOrgCode);
			List<TbVehicleSupplierArchives> list = tbVehicleSupplierArchivesService.list(queryWrapper);
			for(TbVehicleSupplierArchives item:list) {
				TbContract con = new TbContract();
				con.setContractType(contractType);
				con.setId(item.getId());
				con.setName(item.getName());
				dataList.add(con);
			}
		}else {
			return null;
		}
		return dataList;
	}

	//TODO 交接单状态判断(已提交)，合同id
	@Override
	public void updateStatus(String id) {
		LambdaQueryWrapper<TbContract> queryWrapper = new LambdaQueryWrapper<>();
		queryWrapper.eq(TbContract::getId,id);
		queryWrapper.eq(TbContract::getStatus, "0");
		List<TbContract> list = list(queryWrapper);
		for(TbContract item:list) {
			//获取车辆明细
			vehicleCheckDetailService.initVehicleCheckDetail(item);
			int tatol = 0;
			Map<String,Integer> map = new HashMap<>();
			for(TbVehicleCheckDetail d:item.getVehicleList()) {
				if(StringUtils.hasLength(d.getPlateNumber())) {
					int count = map.getOrDefault(d.getVehicleModel(), 0);
					map.put(d.getVehicleModel(), count+1);
					tatol++;
				}
			}
			//1客户租赁合同、4车辆采购合同
			if("4".equals(item.getContractType())) {
				//交接单车辆大于采购合同车辆数
				if(tatol>=item.getVehicleCount()&&item.getVehicleCount()!=0) {
					item.setStatus("1");
					updateById(item);
				}
			}else if("1".equals(item.getContractType())) {
				boolean pass = true;
				//交接单车辆大于租赁合同车型明细车辆数
				List<TbContractVehicle> vehicleList = contractVehicleService.getByContractId(item.getId());
				if(vehicleList.size()==0) {
					pass = false;
				}
				for (TbContractVehicle type : vehicleList) {
					if(!map.containsKey(type.getVehicleModel())) {
						pass = false;
						break;
					}
					int count = map.get(type.getVehicleModel());
					if(count<type.getQuantity()) {
						pass = false;
						break;
					}
				}
				if(pass) {
					item.setStatus("1");
					updateById(item);
				}
			}
		}
	}
	
}
