package com.gci.cdnev.smallprogram.modules.statis.service.impl;


import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.gci.cdnev.smallprogram.exception.GciBootException;
import com.gci.cdnev.smallprogram.modules.projects.entity.Projects;
import com.gci.cdnev.smallprogram.modules.projects.service.IProjectsService;
import com.gci.cdnev.smallprogram.modules.statis.service.ReportService;
import com.gci.cdnev.smallprogram.modules.statis.vo.ReportChartVo;
import com.gci.cdnev.smallprogram.modules.statis.vo.ReportListVo;
import com.gci.cdnev.smallprogram.modules.statis.vo.ReportProjectVo;
import com.gci.cdnev.smallprogram.modules.statis.vo.StatisParamVO;
import com.gci.cdnev.smallprogram.modules.system.service.ISysDictService;
import com.gci.cdnev.smallprogram.modules.vehicles.mapper.TbAccidentInfoMapper;
import com.gci.cdnev.smallprogram.modules.vehicles.mapper.TbRepairInfoMapper;
import com.gci.cdnev.smallprogram.modules.vehicles.mapper.TbUpkeepBillMapper;
import com.gci.cdnev.smallprogram.modules.vehicles.mapper.TbViolationRecordMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

@Service
@Slf4j
public class ReportServiceImpl implements ReportService {
    @Autowired
    private IProjectsService projectsService;
    @Autowired
    private ISysDictService sysDictService;
    @Autowired
    private TbRepairInfoMapper repairInfoMapper;
    @Autowired
    private TbUpkeepBillMapper upkeepBillMapper;
    @Autowired
    private TbViolationRecordMapper violationRecordMapper;
    @Autowired
    private TbAccidentInfoMapper accidentInfoMapper;

    @Override
    public List<ReportChartVo> reportChart(StatisParamVO vo) {
        ArrayList<ReportChartVo> reportChartVos = new ArrayList<>();
        if (StrUtil.isEmpty(vo.getStatisType())) {
            return reportChartVos;
        }
        if ("1".equals(vo.getStatisType())) {
            return reportChartVos;
        } else if ("2".equals(vo.getStatisType())) {
            String startTime = vo.getStartTime();
            try {
                DateTime parse = DateUtil.parse(startTime);
                DateTime beginOfMonth = DateUtil.beginOfMonth(parse);
                DateTime endOfMonth = DateUtil.endOfMonth(parse);
                vo.setStartTime(DateUtil.formatDate(beginOfMonth));
                vo.setEndTime(DateUtil.formatDate(endOfMonth));
            } catch (Exception e) {
                log.error("时间格式化错误:",e);
                throw new GciBootException("时间格式错误！");
            }
        } else if ("3".equals(vo.getStatisType())) {
            String startTime = vo.getStartTime();
            try {
                DateTime parse = DateUtil.parse(startTime);
                DateTime beginOfMonth = DateUtil.beginOfYear(parse);
                DateTime endOfMonth = DateUtil.endOfYear(parse);
                vo.setStartTime(DateUtil.formatDate(beginOfMonth));
                vo.setEndTime(DateUtil.formatDate(endOfMonth));
            } catch (Exception e) {
                log.error("时间格式化错误:",e);
                throw new GciBootException("时间格式错误！");
            }
        } else if ("4".equals(vo.getStatisType())) {
            String startTime = vo.getStartTime();
            try {
                DateTime parse = DateUtil.parse(startTime);
                DateTime beginOfMonth = DateUtil.beginOfQuarter(parse);
                DateTime endOfMonth = DateUtil.endOfQuarter(parse);
                vo.setStartTime(DateUtil.formatDate(beginOfMonth));
                vo.setEndTime(DateUtil.formatDate(endOfMonth));
            } catch (Exception e) {
                log.error("时间格式化错误:",e);
                throw new GciBootException("时间格式错误！");
            }
        }
        //查询有效的项目
        List<Projects> projectsList = projectsService.list();
        if (projectsList.isEmpty()) {
            return reportChartVos;
        }
        List<ReportProjectVo> reportProjectVos = new ArrayList<>();
        projectsList.forEach(i -> {
            ReportProjectVo reportProjectVo = new ReportProjectVo();
            reportProjectVo.setProjectId(i.getId());
            reportProjectVo.setProjectName(i.getProjectName());
            reportProjectVo.setCount(0);
            reportProjectVos.add(reportProjectVo);
        });
        //维修
        List<ReportProjectVo> countReport = repairInfoMapper.countRepair(vo);
        List<ReportProjectVo> repairEdit = getReportProjectVos(reportProjectVos, countReport);
        ReportChartVo reportChartVo = new ReportChartVo();
        reportChartVo.setType("维修");
        reportChartVo.setList(CollUtil.sort(repairEdit, Comparator.comparingInt(ReportProjectVo::getCount).reversed()));
        reportChartVo.setTotalCount(repairEdit.stream().mapToInt(ReportProjectVo::getCount).sum());
        reportChartVos.add(reportChartVo);
        //保养
        List<ReportProjectVo> countUpkeep = upkeepBillMapper.countUpkeep(vo);
        List<ReportProjectVo> upKeepEdit = getReportProjectVos(reportProjectVos, countUpkeep);
        ReportChartVo upKeepChartVo = new ReportChartVo();
        upKeepChartVo.setType("保养");
        upKeepChartVo.setList(CollUtil.sort(upKeepEdit, Comparator.comparingInt(ReportProjectVo::getCount).reversed()));
        upKeepChartVo.setTotalCount(upKeepEdit.stream().mapToInt(ReportProjectVo::getCount).sum());
        reportChartVos.add(upKeepChartVo);
        //违章
        List<ReportProjectVo> countViolation = violationRecordMapper.countViolation(vo);
        List<ReportProjectVo> violationEdit = getReportProjectVos(reportProjectVos, countViolation);
        ReportChartVo violationChartVo = new ReportChartVo();
        violationChartVo.setType("违章");
        violationChartVo.setList(CollUtil.sort(violationEdit, Comparator.comparingInt(ReportProjectVo::getCount).reversed()));
        violationChartVo.setTotalCount(violationEdit.stream().mapToInt(ReportProjectVo::getCount).sum());
        reportChartVos.add(violationChartVo);
        //事故
        List<ReportProjectVo> countAccident = accidentInfoMapper.countAccident(vo);
        List<ReportProjectVo> accidentEdit = getReportProjectVos(reportProjectVos, countAccident);
        ReportChartVo accidentChartVo = new ReportChartVo();
        accidentChartVo.setType("事故");
        accidentChartVo.setList(CollUtil.sort(accidentEdit, Comparator.comparingInt(ReportProjectVo::getCount).reversed()));
        accidentChartVo.setTotalCount(accidentEdit.stream().mapToInt(ReportProjectVo::getCount).sum());
        reportChartVos.add(accidentChartVo);
        return reportChartVos;
    }

    @Override
    public List<ReportListVo> reportList(StatisParamVO vo) {
        if (StrUtil.isEmpty(vo.getStatisType())) {
            return new ArrayList<ReportListVo>();
        }
        if ("1".equals(vo.getStatisType())) {
            return new ArrayList<ReportListVo>();
        } else if ("2".equals(vo.getStatisType())) {
            String startTime = vo.getStartTime();
            try {
                DateTime parse = DateUtil.parse(startTime);
                DateTime beginOfMonth = DateUtil.beginOfMonth(parse);
                DateTime endOfMonth = DateUtil.endOfMonth(parse);
                vo.setStartTime(DateUtil.formatDate(beginOfMonth));
                vo.setEndTime(DateUtil.formatDate(endOfMonth));
            } catch (Exception e) {
                log.error("时间格式化错误:",e);
                throw new GciBootException("时间格式错误！");
            }
        } else if ("3".equals(vo.getStatisType())) {
            String startTime = vo.getStartTime();
            try {
                DateTime parse = DateUtil.parse(startTime);
                DateTime beginOfMonth = DateUtil.beginOfYear(parse);
                DateTime endOfMonth = DateUtil.endOfYear(parse);
                vo.setStartTime(DateUtil.formatDate(beginOfMonth));
                vo.setEndTime(DateUtil.formatDate(endOfMonth));
            } catch (Exception e) {
                log.error("时间格式化错误:",e);
                throw new GciBootException("时间格式错误！");
            }
        } else if ("4".equals(vo.getStatisType())) {
            String startTime = vo.getStartTime();
            try {
                DateTime parse = DateUtil.parse(startTime);
                DateTime beginOfMonth = DateUtil.beginOfQuarter(parse);
                DateTime endOfMonth = DateUtil.endOfQuarter(parse);
                vo.setStartTime(DateUtil.formatDate(beginOfMonth));
                vo.setEndTime(DateUtil.formatDate(endOfMonth));
            } catch (Exception e) {
                log.error("时间格式化错误:",e);
                throw new GciBootException("时间格式错误！");
            }
        }
        //维修
        List<ReportProjectVo> countReport = repairInfoMapper.countRepair(vo);
        //保养
        List<ReportProjectVo> countUpkeep = upkeepBillMapper.countUpkeep(vo);
        //违章
        List<ReportProjectVo> countViolation = violationRecordMapper.countViolation(vo);
        //事故
        List<ReportProjectVo> countAccident = accidentInfoMapper.countAccident(vo);
        //创建一个Map，用于存储项目Id，项目名称和车辆类型为键，ReportListVo为值
        Map<String, ReportListVo> map = new HashMap<>();

        //遍历维修列表，将每个ReportProjectVo转换为ReportListVo，并放入Map中
        for (ReportProjectVo rp : countReport) {
            //生成一个键，由项目Id，项目名称和车辆类型组成，用逗号分隔
            String key = rp.getProjectId() + "," + rp.getProjectName() + "," + rp.getVehicleType();
            //如果Map中已经存在该键，说明已经有一个ReportListVo对象，那么只需要更新其维修统计字段
            if (map.containsKey(key)) {
                ReportListVo rl = map.get(key);
                rl.setRepairCount(rp.getCount());
            } else {
                //否则，创建一个新的ReportListVo对象，并设置其项目Id，项目名称，车辆类型和维修统计字段
                ReportListVo rl = new ReportListVo();
                rl.setProjectId(rp.getProjectId());
                rl.setProjectName(rp.getProjectName());
                rl.setVehicleType(rp.getVehicleType());
                rl.setRepairCount(rp.getCount());
                //将新的ReportListVo对象放入Map中
                map.put(key, rl);
            }
        }

        //遍历保养列表，将每个ReportProjectVo转换为ReportListVo，并放入Map中
        for (ReportProjectVo rp : countUpkeep) {
            //生成一个键，由项目Id，项目名称和车辆类型组成，用逗号分隔
            String key = rp.getProjectId() + "," + rp.getProjectName() + "," + rp.getVehicleType();
            //如果Map中已经存在该键，说明已经有一个ReportListVo对象，那么只需要更新其保养统计字段
            if (map.containsKey(key)) {
                ReportListVo rl = map.get(key);
                rl.setUpKeepCount(rp.getCount());
            } else {
                //否则，创建一个新的ReportListVo对象，并设置其项目Id，项目名称，车辆类型和保养统计字段
                ReportListVo rl = new ReportListVo();
                rl.setProjectId(rp.getProjectId());
                rl.setProjectName(rp.getProjectName());
                rl.setVehicleType(rp.getVehicleType());
                rl.setUpKeepCount(rp.getCount());
                //将新的ReportListVo对象放入Map中
                map.put(key, rl);
            }
        }

        //遍历违章列表，将每个ReportProjectVo转换为ReportListVo，并放入Map中
        for (ReportProjectVo rp : countViolation) {
            //生成一个键，由项目Id，项目名称和车辆类型组成，用逗号分隔
            String key = rp.getProjectId() + "," + rp.getProjectName() + "," + rp.getVehicleType();
            //如果Map中已经存在该键，说明已经有一个ReportListVo对象，那么只需要更新其违章统计字段
            if (map.containsKey(key)) {
                ReportListVo rl = map.get(key);
                rl.setViolationCount(rp.getCount());
            } else {
                //否则，创建一个新的ReportListVo对象，并设置其项目Id，项目名称，车辆类型和违章统计字段
                ReportListVo rl = new ReportListVo();
                rl.setProjectId(rp.getProjectId());
                rl.setProjectName(rp.getProjectName());
                rl.setVehicleType(rp.getVehicleType());
                rl.setViolationCount(rp.getCount());
                //将新的ReportListVo对象放入Map中
                map.put(key, rl);
            }
        }

        //遍历事故列表，将每个ReportProjectVo转换为ReportListVo，并放入Map中
        for (ReportProjectVo rp : countAccident) {
            //生成一个键，由项目Id，项目名称和车辆类型组成，用逗号分隔
            String key = rp.getProjectId() + "," + rp.getProjectName() + "," + rp.getVehicleType();
            //如果Map中已经存在该键，说明已经有一个ReportListVo对象，那么只需要更新其事故统计字段
            if (map.containsKey(key)) {
                ReportListVo rl = map.get(key);
                rl.setAccidentCount(rp.getCount());
            } else {
                //否则，创建一个新的ReportListVo对象，并设置其项目Id，项目名称，车辆类型和事故统计字段
                ReportListVo rl = new ReportListVo();
                rl.setProjectId(rp.getProjectId());
                rl.setProjectName(rp.getProjectName());
                rl.setVehicleType(rp.getVehicleType());
                rl.setAccidentCount(rp.getCount());
                //将新的ReportListVo对象放入Map中
                map.put(key, rl);
            }
        }

        //创建一个List，用于存储最终的结果

        //返回结果
        return new ArrayList<>(map.values());
    }

    /**
     * 获取根据项目的统计总数
     *
     * @param reportProjectVos
     * @param countReport
     * @return
     */
    private List<ReportProjectVo> getReportProjectVos(List<ReportProjectVo> reportProjectVos, List<ReportProjectVo> countReport) {
        Map<String, Map<String, ReportProjectVo>> reportColl = CollStreamUtil.group2Map(countReport, ReportProjectVo::getProjectId, ReportProjectVo::getVehicleType);
        return ListUtil.list(false, CollUtil.edit(BeanUtil.copyToList(reportProjectVos, ReportProjectVo.class), reportProjectVo -> {
            Map<String, ReportProjectVo> stringReportProjectVoMap = reportColl.get(reportProjectVo.getProjectId());
            if (ObjectUtil.isNotEmpty(stringReportProjectVoMap)) {
                Collection<ReportProjectVo> values = stringReportProjectVoMap.values();
                reportProjectVo.setCount(values.stream().mapToInt(ReportProjectVo::getCount).sum());
            }
            return reportProjectVo;
        }));
    }
}
