package com.gci.cdnev.smallprogram.modules.system.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gci.cdnev.smallprogram.common.dict.excel.DictParser;
import com.gci.cdnev.smallprogram.constant.CacheConstant;
import com.gci.cdnev.smallprogram.modules.system.entity.SysDict;
import com.gci.cdnev.smallprogram.modules.system.mapper.SysDictMapper;
import com.gci.cdnev.smallprogram.modules.system.service.ISysDictService;
import com.gci.cdnev.smallprogram.modules.system.vo.DictModel;
import org.apache.commons.lang3.StringUtils;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author chenzhiwei
 * @since 2023-08-25
 */
@Service
public class SysDictServiceImpl extends ServiceImpl<SysDictMapper, SysDict> implements ISysDictService, DictParser {

    @Override
    public List<DictModel> queryDictItemsByCode(String code) {
        return baseMapper.queryDictItemsByCode(code);
    }

    @Override
    public List<DictModel> queryTableDictItemsByCodeAndFilter(String table, String text, String code, String filterSql) {
        return baseMapper.queryTableDictItemsByCode(table, text, code, filterSql);
    }
    @Override
    public List<DictModel> queryTableDictItemsByCode(String table, String text, String code) {
        return baseMapper.queryTableDictItemsByCode(table, text, code, null);
    }
    /**
     * 通过查询指定table的 text code 获取字典值text
     * dictTableCache采用redis缓存有效期10分钟
     *
     * @param table
     * @param text
     * @param code
     * @param key
     * @return
     */
    @Override
    @Cacheable(value = CacheConstant.SYS_DICT_TABLE_CACHE, key = "#table+':'+#code+':'+#key")
    public String queryTableDictTextByKey(String table, String text, String code, String key) {
        if (StringUtils.isBlank(key)) {
            return null;
        }
        List<DictModel> dictModels = baseMapper.queryTableDictByValue(table, text, code, key);
        if (dictModels.size() == 0) {
            return null;
        }
        return dictModels.get(0).getText();
    }

    @Override
    @Cacheable(value = CacheConstant.SYS_DICT_CACHE, key = "#code+':'+#key")
    public String queryDictTextByKey(String code, String key) {
        if (StringUtils.isBlank(key)) {
            return null;
        }
        List<DictModel> dictModels = baseMapper.queryDictByValue(code, key);
        if (dictModels.size() == 0) {
            return null;
        }
        return dictModels.get(0).getText();
    }

    @Override
    @Cacheable(value = CacheConstant.SYS_DICT_CACHE_TEXTKEY, key = "#code+':'+#key")
    public String queryDictValueByText(String code, String key) {
        if (StringUtils.isBlank(key)) {
            return null;
        }
        List<DictModel> dictModels = baseMapper.queryDictByText(code, key);
        if (dictModels.size() == 0) {
            return null;
        }
        return dictModels.get(0).getValue();
    }

    @Override
    @Cacheable(value = CacheConstant.SYS_DICT_TABLE_CACHE_TEXTKEY, key = "#table+':'+#text+':'+#key")
    public String queryTableDictValueByText(String table, String text, String code, String key) {
        if (StringUtils.isBlank(key)) {
            return null;
        }
        List<DictModel> dictModels = baseMapper.queryTableDictByText(table, text, code, key);
        if (dictModels.size() == 0) {
            return null;
        }
        return dictModels.get(0).getValue();
    }

}
