package com.gci.cdnev.smallprogram.modules.vehicles.controller;

import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateUtil;
import com.gci.cdnev.smallprogram.annotation.CurrentUser;
import com.gci.cdnev.smallprogram.common.log.annotation.AddSysLog;
import com.gci.cdnev.smallprogram.common.vo.Result;
import com.gci.cdnev.smallprogram.modules.customer.entity.TbCustomerFollow;
import com.gci.cdnev.smallprogram.modules.customer.service.ITbCustomerFollowPartService;
import com.gci.cdnev.smallprogram.modules.customer.service.ITbCustomerFollowService;
import com.gci.cdnev.smallprogram.modules.customer.service.ITbCustomerOpinionService;
import com.gci.cdnev.smallprogram.modules.system.service.ISysDictService;
import com.gci.cdnev.smallprogram.modules.vehicles.entity.TbAccidentInfo;
import com.gci.cdnev.smallprogram.modules.vehicles.entity.TbAnnualInspectionInfo;
import com.gci.cdnev.smallprogram.modules.vehicles.entity.TbInsuranceRecord;
import com.gci.cdnev.smallprogram.modules.vehicles.entity.TbViolationRecord;
import com.gci.cdnev.smallprogram.modules.vehicles.service.*;
import com.gci.cdnev.smallprogram.modules.vehicles.vo.*;
import com.gci.cdnev.smallprogram.modules.wx.entity.JwtUser;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.*;

@Api(tags="待办")
@Slf4j
@RestController
@RequestMapping("/api/vehicles/backlog")
public class BacklogController {

    @Autowired
    private BacklogService backlogService;
    @Autowired
    private ITbViolationRecordService violationRecordService;
    @Autowired
    private ITbRepairInfoService repairInfoService;
    @Autowired
    private ITbUpkeepBillService upkeepBillService;
    @Autowired
    private ITbAccidentInfoService iTbAccidentInfoService;
    @Autowired
    private ITbInsuranceRecordService tbInsuranceRecordService;
    @Autowired
    private ITbAnnualInspectionInfoService iTbAnnualInspectionInfoService;
    @Autowired
    private ISysDictService sysDictService;
    @Autowired
    private ITbCustomerFollowPartService tbCustomerFollowPartService;
    @Autowired
    private ITbCustomerOpinionService tbCustomerOpinionService;
    @Autowired
    private ITbCustomerFollowService tbCustomerFollowService;

    /**
     * 待办统计
     *
     */
    @AddSysLog("待办统计-数量查询")
    @GetMapping(value = "/getBacklogCount")
    public Result<?> getBacklogCount(@CurrentUser JwtUser jwtUser) {
        List<BacklogStatisVO> result = backlogService.getBacklogCount(jwtUser);
        return Result.ok(result);
    }

    /**
     * 待办统计分页
     *
     */
    @AddSysLog("待办统计-分页查询")
    @GetMapping(value = "/getBacklogPage")
    public Result<?> getBacklogPage(@CurrentUser JwtUser jwtUser,
                                    @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                    @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                    @RequestParam(name = "type", required = true) String type) {
        Object result = backlogService.getBacklogPageList(jwtUser,pageNo,pageSize,type);
        return Result.ok(result);
    }
    //1出险 2违章 3事故 4维修 5保养 6年检
    /**
     * 1出险
     *
     */
    @AddSysLog("待办统计-出险查询")
    @GetMapping(value = "/getInsuranceRecordById")
    public Result<?> getInsuranceRecordById(String id) {
        TbInsuranceRecord insuranceRecord = tbInsuranceRecordService.getById(id);
        return Result.ok(insuranceRecord);
    }

    /**
     * 2违章
     *
     */
    @AddSysLog("待办统计-违章查询")
    @GetMapping(value = "/getViolationRecorddById")
    public Result<?> getViolationRecorddById(String id) {
        TbViolationRecord violationRecord = violationRecordService.getById(id);
        return Result.ok(violationRecord);
    }

    /**
     * 3事故
     *
     */
    @AddSysLog("待办统计-事故查询")
    @GetMapping(value = "/getAccidentInfoById")
    public Result<?> getAccidentInfoById(String id) {
        TbAccidentInfo accidentInfo = iTbAccidentInfoService.getTbAccidentInfoById(id);
        return Result.ok(accidentInfo);
    }

    /**
     * 4维修
     *
     */
    @AddSysLog("待办统计-维修查询")
    @GetMapping(value = "/getRepairInfoById")
    public Result<?> getRepairInfoById(@CurrentUser JwtUser jwtUser,
                                            String id) {
        RepairObjectVO obj = repairInfoService.getRepairObjectById(id,jwtUser);
        return Result.ok(obj);
    }

    /**
     * 5保养
     *
     */
    @AddSysLog("待办统计-保养查询")
    @GetMapping(value = "/getUpkeepInfoById")
    public Result<?> getUpkeepInfoById(@CurrentUser JwtUser jwtUser,
                                            String id) {
        UpkeepObjectVO obj = upkeepBillService.getUpkeepObjectById(id,jwtUser);
        return Result.ok(obj);
    }

    /**
     * 6年检
     *
     */
    @AddSysLog("待办统计-年检查询")
    @GetMapping(value = "/getAnnualInspectionById")
    public Result<?> getAnnualInspectionById(String id) {
        TbAnnualInspectionInfo info = iTbAnnualInspectionInfoService.getById(id);
        if(Objects.nonNull(info) && Objects.nonNull(info.getValidityPeriod())){
            info.setValidityPeriod(DateUtil.offset(info.getValidityPeriod(), DateField.YEAR,1));
        }
        return Result.ok(info);
    }

    /**
     * 7 客户回访
     *
     */
    @AddSysLog("待办统计-回访查询")
    @GetMapping(value = "/getFollowInfoById")
    public Result<?> getFollowInfoById(@CurrentUser JwtUser jwtUser,
                                       String id) {
        TbCustomerFollow obj = tbCustomerFollowService.getById(id);
        if(Objects.nonNull(obj)){
            tbCustomerFollowPartService.getPartsByFollowId(obj);
            tbCustomerOpinionService.getOpinionByFollowId(obj);
            tbCustomerFollowService.getInspectionTaskByFollowId(obj);
        }
        return Result.ok(obj);
    }

}
