package com.gci.cdnev.smallprogram.modules.vehicles.controller;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.excel.EasyExcel;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gci.cdnev.smallprogram.annotation.CurrentUser;
import com.gci.cdnev.smallprogram.common.log.annotation.AddSysLog;
import com.gci.cdnev.smallprogram.common.util.MinioUtils;
import com.gci.cdnev.smallprogram.common.util.StringUtil;
import com.gci.cdnev.smallprogram.common.validated.SaveValidated;
import com.gci.cdnev.smallprogram.common.validated.UpdateValidated;
import com.gci.cdnev.smallprogram.common.vo.Result;
import com.gci.cdnev.smallprogram.enums.FileExtension;
import com.gci.cdnev.smallprogram.modules.basedata.service.ITbRepairShopArchivesService;
import com.gci.cdnev.smallprogram.modules.projects.entity.Projects;
import com.gci.cdnev.smallprogram.modules.projects.service.IProjectsService;
import com.gci.cdnev.smallprogram.modules.system.service.ISysDictService;
import com.gci.cdnev.smallprogram.modules.system.service.ISysRoleService;
import com.gci.cdnev.smallprogram.modules.system.service.ISysUserService;
import com.gci.cdnev.smallprogram.modules.system.vo.DictModel;
import com.gci.cdnev.smallprogram.modules.vehicles.entity.TbRepairAssign;
import com.gci.cdnev.smallprogram.modules.vehicles.entity.TbRepairCheck;
import com.gci.cdnev.smallprogram.modules.vehicles.entity.TbRepairInfo;
import com.gci.cdnev.smallprogram.modules.vehicles.mapper.TbRepairInfoMapper;
import com.gci.cdnev.smallprogram.modules.vehicles.mapper.TbViolationRecordMapper;
import com.gci.cdnev.smallprogram.modules.vehicles.service.*;
import com.gci.cdnev.smallprogram.modules.vehicles.vo.*;
import com.gci.cdnev.smallprogram.modules.wx.entity.JwtUser;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

@Api(tags="维修")
@Slf4j
@RestController
@RequestMapping("/api/vehicles/repair")
public class RepairController {

    @Autowired
    private ITbRepairAssignService repairAssignService;
    @Autowired
    private ITbRepairCheckService repairCheckService;
    @Autowired
    private ITbRepairInfoService repairInfoService;
    @Autowired
    private ITbRepairOfferService repairOfferService;
    @Autowired
    private TbViolationRecordMapper violationRecordMapper;
    @Autowired
    private TbRepairInfoMapper repairInfoMapper;
    @Autowired
    private IProjectsService projectsService;
    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private MinioUtils minioUtils;
    @Autowired
    private ISysUserService sysUserService;
    @Autowired
    private ISysRoleService sysRoleService;
    @Autowired
    private ITbRepairShopArchivesService tbRepairShopArchivesService;
    @Autowired
    private ITbUpkeepBillService upkeepBillService;
    @Autowired
    private ISysDictService sysDictService;

    /**
     * 分页列表查询
     *
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AddSysLog("维修-分页列表查询")
    @GetMapping(value = "/list")
    public Result<?> queryPageList(@CurrentUser JwtUser jwtUser,
                                   RepairInfoListVO vo,
                                   @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                   @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                   HttpServletRequest req) {
        if(Objects.nonNull(vo.getFlowStatus()) && vo.getFlowStatus()==-1){
            //适配小程序查全部
            vo.setFlowStatus(null);
        }
        //数据权限
        repairInfoService.dealRepairInfoListVO(jwtUser,  vo);

        Page<RepairInfoListVO> page = new Page<RepairInfoListVO>(pageNo, pageSize);
        IPage<RepairInfoListVO> pageList = repairInfoService.getRepairInfoList(page, vo,jwtUser);
        return Result.ok(pageList);
    }


    /**
     * 获取报修信息所属项目下拉列表
     */
    @AddSysLog("维修-获取报修信息所属项目下拉列表")
    @GetMapping(value = "/queryProjects")
    public Result<?> queryProjects(@CurrentUser JwtUser jwtUser) {
//        List<String> projectIds = jwtUser.getProjectIds();
        Integer departCategory = projectsService.judgeDepartCategory(jwtUser);
        //超级管理员查询全部数据
        List<String> projectIds = new ArrayList<>();
        if(departCategory != 5){
            projectIds = projectsService.queryProjectListOfUser(jwtUser);
            if(CollectionUtil.isEmpty(projectIds) || projectIds.size() == 0 ){
                projectIds.add("no permission");
            }
        }
        List<VehicleProjectVO> list = violationRecordMapper.queryProjects(0,0,projectIds);
        return Result.ok(list);
    }

    /**
     * 获取报修信息车牌号及车辆信息列表
     */
    @AddSysLog("维修-获取报修信息车牌号及车辆信息列表")
    @GetMapping(value = "/queryVehicleInfoByProjectId")
    public Result<?> queryVehicleInfoByProjectId(String projectId) {
        List<VehicleInfoVO> list = repairInfoMapper.findVehicleInfoByProjectId(projectId);
        return Result.ok(list);
    }

    /**
     * 添加
     */
    @AddSysLog("维修-添加编辑维修信息")
    @PostMapping(value = "/addOrEdit")
    public Result<?> add(@CurrentUser JwtUser jwtUser, @RequestBody @Validated(SaveValidated.class)  TbRepairInfo tbRepairInfo) {
        Projects projects = projectsService.getById(tbRepairInfo.getProjectId());
        if(Objects.nonNull(projects)){
            tbRepairInfo.setProjectName(projects.getProjectName());
        }
        if(StrUtil.isNotBlank(tbRepairInfo.getId())){
            repairInfoService.updateById(tbRepairInfo);
            return Result.ok("操作成功！");
        }
        //报修时间--默认上报时间
        if(Objects.nonNull(tbRepairInfo.getRepairTime())){
            tbRepairInfo.setRepairTime(new Date());
        }

        tbRepairInfo.setFlowStatus(2);
        tbRepairInfo.setFlowNode(1);
        tbRepairInfo.setAddType(2);

        //工单号
        String format = DateUtil.format(new Date(),"yyyyMMddHHmmssSSS");
        tbRepairInfo.setOrderNo(format);

        String username = "";
        if(Objects.nonNull(jwtUser)){
            username = jwtUser.getUserName();
        }
        tbRepairInfo.setCreateBy(username);

        repairInfoService.save(tbRepairInfo);
        //补充节点操作人
        TbRepairCheck repairCheck = new TbRepairCheck();
        repairCheck.setRepairId(tbRepairInfo.getId());
        if(Objects.nonNull(jwtUser)){
            repairCheck.setCheckPeople(jwtUser.getRealname());
        }
        repairCheck.setCheckTime(new Date());
        repairCheck.setCheckPeopleType("4");
        repairCheckService.save(repairCheck);
        return Result.ok(tbRepairInfo.getId());
    }

    /**
     * 编辑
     */
    @AddSysLog("维修-编辑维修信息")
    @PutMapping(value = "/edit")
    public Result<?> edit(@RequestBody @Validated(UpdateValidated.class) TbRepairInfo tbRepairInfo) {
        Projects projects = projectsService.getById(tbRepairInfo.getProjectId());
        if(Objects.nonNull(projects)){
            tbRepairInfo.setProjectName(projects.getProjectName());
        }
        repairInfoService.updateById(tbRepairInfo);
        return Result.ok("编辑成功！");
    }


    /**
     * 删除
     */
    @AddSysLog("维修-删除维修信息")
    @DeleteMapping(value = "/realDelete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        repairInfoService.removeById(id);
        return Result.ok("删除成功！");
    }

    /**
     * 作废
     */
    @AddSysLog("维修-作废维修信息")
    @PostMapping(value = "/cancel")
    public Result<?> cancel(@CurrentUser JwtUser jwtUser,
                            @RequestBody CancelVO vo) {
        String arg = StringUtil.checkString(vo.getCancelIdea(),200);
        if(StrUtil.isNotBlank(arg)){
            return Result.error(101,"作废原因 " + arg);
        }
        String realName = "";
        if(StrUtil.isNotBlank(vo.getRealname())){
            realName = vo.getRealname();
        }else{
            if(Objects.nonNull(jwtUser)){
                realName = jwtUser.getRealname();
            }
        }

        repairInfoService.update(new LambdaUpdateWrapper<TbRepairInfo>().eq(TbRepairInfo::getId,vo.getId())
                .set(TbRepairInfo::getFlowStatus,9)
                .set(TbRepairInfo::getFlowNode,4)
                .set(TbRepairInfo::getCancelIdea,vo.getCancelIdea())
                .set(TbRepairInfo::getCancelPeople,realName)
                .set(TbRepairInfo::getCancelTime,new Date()));
        return Result.ok("作废成功！");
    }

    /**
     * 导出
     * @return
     */
    @AddSysLog("维修-导出维修信息")
    @GetMapping("/export")
    public Result<?> export(RepairInfoListVO vo, HttpServletResponse response){
        try (OutputStream output = response.getOutputStream()){

            String fileName = "车辆维修.xlsx";
            try {
                fileName = URLEncoder.encode(fileName, "UTF-8");
            } catch (UnsupportedEncodingException e) {
                e.printStackTrace();
            }
            response.setHeader("Content-disposition", "attachment;filename=" + fileName + ";" + "filename*=utf-8''" + fileName);
            List<RepairInfoListVO> list = repairInfoMapper.findRepairInfoAllList(vo);
            EasyExcel.write(output, RepairInfoListVO.class).sheet("车辆维修").doWrite(list);

        } catch (IOException e) {
            return Result.error("导出错误！");
        }
        return Result.ok();
    }

    /**
     * 根据维修单id获取维修弹窗详情
     */
    @AddSysLog("维修-根据维修单id获取维修弹窗详情")
    @GetMapping(value = "/getRepairObjectById")
    public Result<?> getRepairObjectById(@CurrentUser JwtUser jwtUser,
                                         String id) {
        RepairObjectVO vo = repairInfoService.getRepairObjectById(id,jwtUser);
        return Result.ok(vo);
    }

    /**
     * 数据录入
     */
    @AddSysLog("维修-数据录入")
    @PostMapping(value = "/addDataEntry")
    public Result<?> addDataEntry(@CurrentUser JwtUser jwtUser,
                                  @RequestBody @Validated(SaveValidated.class) RepairDataEntryVO vo) {
        if(Objects.isNull(vo.getRepairInfo().getRepairTime())){
            vo.getRepairInfo().setRepairTime(new Date());
        }
        String id =  repairInfoService.addDataEntry(vo,jwtUser);
        return Result.ok(id);
    }

    /**
     * 维修单流程提交
     */
    @AddSysLog("维修-维修单流程提交")
    @PostMapping(value = "/submitRepairObject")
    public Result<?> submitRepairObject(@CurrentUser JwtUser jwtUser,
                                        @RequestBody @Validated(SaveValidated.class) RepairObjectVO vo, HttpServletRequest request) {

        if(redisTemplate.hasKey(vo.getId()+"_submitRepairObject")){
            return Result.error(101,"该维修单正在被操作中，无法操作，请刷新!");
        }else{
            redisTemplate.opsForValue().set(vo.getId()+"_submitRepairObject", vo.getId(), 10, TimeUnit.SECONDS);
        }
        String result = "";
        try {
            result = repairInfoService.submitRepairObject(vo,jwtUser);
        }catch (Exception e){
            e.printStackTrace();
            return Result.error(101,"提交失败：" + e.getMessage());
        }finally {
            redisTemplate.delete(vo.getId()+"_submitRepairObject");
        }
        if(StrUtil.isNotBlank(result)){
            return Result.error(101,result);
        }else{
            return Result.ok("提交成功！");
        }
    }

    /**
     * 获取指派信息维修厂列表
     */
    @AddSysLog("维修-获取指派信息维修厂列表")
    @GetMapping(value = "/getRepairShop")
    public Result<?> getRepairShop(@CurrentUser JwtUser jwtUser) {
        List<String> repairShopIds = upkeepBillService.getUserListRepairShopIds(jwtUser);
        List<TbRepairAssign> list = repairInfoMapper.findRepairShop(repairShopIds);
        return Result.ok(list);
    }

    /**
     * 文件上传
     */
    @AddSysLog("维修-文件上传")
    @PostMapping("/fileUpload")
    public Result<?> upload(HttpServletRequest request) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        MultipartFile file = multipartRequest.getFile("file");
        if (ObjectUtil.isEmpty(file)) {
            return Result.error("请正确上传文件！");
        }
        List<String> fileTypes = new ArrayList<>();
        for(String type:FileExtension.PIC.getNameList()) {
            fileTypes.add(type);
        }
        for(String type:FileExtension.DOC.getNameList()) {
            fileTypes.add(type);
        }
        return Result.ok(minioUtils.uploadFileV2Valid(file, fileTypes));
    }


    /**
     * 分页列表查询----待办的
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AddSysLog("维修-查询待办列表")
    @GetMapping(value = "/getRepairBacklogList")
    public Result<?> getRepairBacklogList(@CurrentUser JwtUser jwtUser,
                                          RepairInfoListVO vo,
                                          @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                          @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                          HttpServletRequest req) {
        //数据权限
        repairInfoService.dealRepairInfoListVO(jwtUser,  vo);
        Page<RepairInfoListVO> page = new Page<RepairInfoListVO>(pageNo, pageSize);
        IPage<RepairInfoListVO> pageList = repairInfoService.getRepairBacklogList(page, vo,jwtUser);
        return Result.ok(pageList);
    }

    /**
     * 查询--维修列表数量统计
     * @return
     */
    @AddSysLog("维修-查询数量统计")
    @GetMapping(value = "/getRepairPageStatic")
    public Result<?> getRepairPageStatic(@CurrentUser JwtUser jwtUser,
                                         RepairInfoListVO vo) {
        //数据权限
        repairInfoService.dealRepairInfoListVO(jwtUser, vo);
        Page<RepairInfoListVO> page = new Page<RepairInfoListVO>(1, -1);
        List<RepairPageStaticVO> pageList = repairInfoService.getRepairPageStatic(page,vo,jwtUser);
        return Result.ok(pageList);
    }

    /**
     * 查询--维修列表数量统计--维修厂
     * @return
     */
    @AddSysLog("维修-查询维修厂数量统计")
    @GetMapping(value = "/getRepairShopPageStatic")
    public Result<?> getRepairShopPageStatic(@CurrentUser JwtUser jwtUser,
                                             RepairInfoListVO vo) {
        List<Integer> flowStatusList = new ArrayList<>();
        flowStatusList.add(3);
        flowStatusList.add(6);
        vo.setFlowStatusList(flowStatusList);

        //维修
        //数据权限
        repairInfoService.dealRepairInfoListVO(jwtUser, vo);
        Page<RepairPageStaticVO> page = new Page<RepairPageStaticVO>(1, -1);
        List<RepairPageStaticVO> pageList = repairInfoService.getRepairPageStatic(page,vo,jwtUser);
        pageList = pageList.stream().filter(item->(item.getFlowStatus() == 3 || item.getFlowStatus() == 6)).collect(Collectors.toList());
        for (RepairPageStaticVO repair : pageList){
            if(repair.getFlowStatus() == 3){
                repair.setFlowName("维修待报价");
            }
            repair.setRepairOrUpkeep(1);
        }

        //保养
        //维修厂数据权限
        UpkeepInfoListVO upkeepVO = new UpkeepInfoListVO();
        upkeepVO.setFlowStatusList(flowStatusList);
        upkeepVO.setListRepairShop(upkeepBillService.getUserListRepairShopIds(jwtUser));
        Page<UpkeepBillPageStaticVO> upkeepPage = new Page<UpkeepBillPageStaticVO>(1, -1);
        List<UpkeepBillPageStaticVO> upkeepPageList = upkeepBillService.getUpkeepPageStatic(upkeepPage,upkeepVO,jwtUser);
        upkeepPageList = upkeepPageList.stream().filter(item->(item.getFlowStatus() == 3 || item.getFlowStatus() == 6)).collect(Collectors.toList());
        for (UpkeepBillPageStaticVO upkeep : upkeepPageList){
            RepairPageStaticVO one = new RepairPageStaticVO();
            BeanUtil.copyProperties(upkeep,one);
            if(one.getFlowStatus() == 3){
                one.setFlowName("保养待报价");
            }
            one.setRepairOrUpkeep(2);
            pageList.add(one);
        }

        return Result.ok(pageList);
    }


    /**
     * 返回维修分类字典
     * @return
     */
    @AddSysLog("维修-返回维修分类字典")
    @GetMapping(value = "/getRepairClass")
    public Result<?> getRepairClass() {
        List<DictModel> modelList =  sysDictService.queryDictItemsByCode("repair_class");
        return Result.ok(modelList);
    }

    /**
     * 模糊查根据车牌或工单
     */
    /**
     * 模糊查根据车牌或工单
     * @param pageNo
     * @param pageSize
     * @return
     */
    @AddSysLog("维修-根据车牌或工单查询")
    @GetMapping(value = "/getByPlateNumberOrOrderNo")
    public Result<?> getByPlateNumberOrOrderNo(@CurrentUser JwtUser jwtUser,
                                          RepairInfoListVO vo,
                                          @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                          @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize
                                         ) {

        //数据权限
        repairInfoService.dealRepairInfoListVO(jwtUser, vo);
        Page<RepairPageStaticVO> pageCount = new Page<RepairPageStaticVO>(1, -1);
        List<RepairPageStaticVO> pageStatic = repairInfoService.getRepairPageStatic(pageCount,vo,jwtUser);

        Page<RepairInfoListVO> page = new Page<RepairInfoListVO>(pageNo, pageSize);
        IPage<RepairInfoListVO> pageList = repairInfoService.getRepairInfoList(page, vo,jwtUser);

        Map<String,Object> map = new HashMap<>(2);
        map.put("pageStatic",pageStatic);
        map.put("pageList",pageList);

        return Result.ok(map);
    }
}
