package com.gci.cdnev.smallprogram.modules.vehicles.controller;


import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gci.cdnev.smallprogram.annotation.CurrentUser;
import com.gci.cdnev.smallprogram.common.log.annotation.AddSysLog;
import com.gci.cdnev.smallprogram.common.util.MinioUtils;
import com.gci.cdnev.smallprogram.common.validated.SaveValidated;
import com.gci.cdnev.smallprogram.common.validated.UpdateValidated;
import com.gci.cdnev.smallprogram.common.vo.Result;
import com.gci.cdnev.smallprogram.enums.FileExtension;
import com.gci.cdnev.smallprogram.modules.projects.service.IProjectsService;
import com.gci.cdnev.smallprogram.modules.vehicles.entity.TbAccidentInfo;
import com.gci.cdnev.smallprogram.modules.vehicles.query.AccidentInfoQuery;
import com.gci.cdnev.smallprogram.modules.vehicles.service.ITbAccidentInfoService;
import com.gci.cdnev.smallprogram.modules.wx.entity.JwtUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * <p>
 * 事故信息基础表 前端控制器
 * </p>
 *
 * @author 2464
 * @since 2023-09-13
 */
@RestController
@RequestMapping("/api/vehicles/accidentInfo")
public class TbAccidentInfoController {
    @Autowired
    private ITbAccidentInfoService iTbAccidentInfoService;
    @Autowired
    private MinioUtils minioUtils;
    @Autowired
    private IProjectsService projectsService;

    /**
     * 分页列表查询
     *
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AddSysLog("事故信息-分页列表查询")
    @GetMapping(value = "/list")
    public Result<?> queryPageList(AccidentInfoQuery query,
                                   @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                   @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                   HttpServletRequest req) {
        Page<TbAccidentInfo> page = new Page<TbAccidentInfo>(pageNo, pageSize);
        IPage<TbAccidentInfo> pageList = iTbAccidentInfoService.pageBuild(page, query);
        return Result.ok(pageList);
    }

    /**
     * 待办列表
     */
    @AddSysLog("事故信息-待办列表")
    @GetMapping(value = "/todoList")
    public Result<?> todoList(@CurrentUser JwtUser jwtUser,
                              @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                   @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                   HttpServletRequest req) {
        //数据权限
        Integer departCategory = projectsService.judgeDepartCategory(jwtUser);
        //超级管理员查询全部数据
        List<String> projectIds = new ArrayList<>();
        if(departCategory != 5){
            projectIds = projectsService.queryProjectListOfUser(jwtUser);
            if(CollectionUtil.isEmpty(projectIds) || projectIds.size() == 0 ){
                projectIds.add("no permission");
            }
        }
        TbAccidentInfo info = new TbAccidentInfo();
        info.setListProject(projectIds);

        Page<TbAccidentInfo> page = new Page<TbAccidentInfo>(pageNo, pageSize);
        IPage<TbAccidentInfo> pageList = iTbAccidentInfoService.todoList(page,info);
        return Result.ok(pageList);
    }

    /**
     * 添加
     */
    @AddSysLog("事故信息-添加")
    @PostMapping(value = "/add")
    public Result<?> add(@RequestBody @Validated(SaveValidated.class) TbAccidentInfo tbAccidentInfo) {
        iTbAccidentInfoService.saveItem(tbAccidentInfo);
        return Result.ok("添加成功！");
    }

    /**
     * 编辑
     */
    @AddSysLog("事故信息-编辑")
    @PutMapping(value = "/edit")
    public Result<?> edit(@RequestBody @Validated(UpdateValidated.class) TbAccidentInfo tbAccidentInfo) {
        iTbAccidentInfoService.updateItem(tbAccidentInfo);
        return Result.ok("编辑成功！");
    }

    /**
     * 小程序--编辑
     */
    @AddSysLog("事故信息-小程序--编辑")
    @PutMapping(value = "/smallprogramEdit")
    public Result<?> smallprogramEdit(@RequestBody TbAccidentInfo tbAccidentInfo) {
        TbAccidentInfo temp = iTbAccidentInfoService.getTbAccidentInfoById(tbAccidentInfo.getId());
        temp.setAccidentProgress(tbAccidentInfo.getAccidentProgress());
        iTbAccidentInfoService.updateItem(temp);
        return Result.ok("编辑成功！");
    }

    /**
     * 删除
     */
    @AddSysLog("事故信息-删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        iTbAccidentInfoService.removeItem(id);
        return Result.ok("删除成功！");
    }

    /**
     * 批量删除
     */
    @AddSysLog("事故信息-批量删除")
//    @DeleteMapping(value = "/deleteBatch")
    public Result<?> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        iTbAccidentInfoService.removeByIds(Arrays.asList(ids.split(",")));
        return Result.ok("批量删除成功！");
    }

    /**
     * 文件上传
     */
    @AddSysLog("事故信息-文件上传")
    @PostMapping("/upload")
    public Result<?> upload(HttpServletRequest request) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        MultipartFile file = multipartRequest.getFile("file");
        if (ObjectUtil.isEmpty(file)) {
            return Result.error("请正确上传文件！");
        }
        return Result.ok(minioUtils.uploadFileV2Valid(file,FileExtension.PIC.getNameList()));
    }

    /**
     * 文件上传视频
     */
    @AddSysLog("事故信息-文件上传视频")
    @PostMapping("/uploadVideo")
    public Result<?> uploadVideo(HttpServletRequest request) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        MultipartFile file = multipartRequest.getFile("file");
        if (ObjectUtil.isEmpty(file)) {
            return Result.error("请正确上传文件！");
        }
        return Result.ok(minioUtils.uploadFileV2Valid(file,FileExtension.VID.getNameList()));
    }

    /**
     * 获取单个
     */
    @AddSysLog("事故信息-获取单个")
    @GetMapping(value = "/getById")
    public Result<?> getById(String id) {
        TbAccidentInfo info = iTbAccidentInfoService.getTbAccidentInfoById(id);
        return Result.ok(info);
    }
}
