package com.gci.cdnev.smallprogram.modules.vehicles.controller;


import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gci.cdnev.smallprogram.annotation.CurrentUser;
import com.gci.cdnev.smallprogram.common.log.annotation.AddSysLog;
import com.gci.cdnev.smallprogram.common.util.MinioUtils;
import com.gci.cdnev.smallprogram.common.validated.SaveValidated;
import com.gci.cdnev.smallprogram.common.validated.UpdateValidated;
import com.gci.cdnev.smallprogram.common.vo.Result;
import com.gci.cdnev.smallprogram.enums.FileExtension;
import com.gci.cdnev.smallprogram.modules.projects.service.IProjectsService;
import com.gci.cdnev.smallprogram.modules.vehicles.entity.TbAnnualInspectionInfo;
import com.gci.cdnev.smallprogram.modules.vehicles.query.AnnualInspectionInfoQuery;
import com.gci.cdnev.smallprogram.modules.vehicles.service.ITbAnnualInspectionInfoService;
import com.gci.cdnev.smallprogram.modules.wx.entity.JwtUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * <p>
 * 年检信息表 前端控制器
 * </p>
 *
 * @author 2464
 * @since 2023-09-20
 */
@RestController
@RequestMapping("/api/vehicles/annualInspectionInfo")
public class TbAnnualInspectionInfoController {
    @Autowired
    private ITbAnnualInspectionInfoService iTbAnnualInspectionInfoService;
    @Autowired
    private MinioUtils minioUtils;
    @Autowired
    private IProjectsService projectsService;

    /**
     * 分页列表查询
     *
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AddSysLog("年检信息-分页列表查询")
    @GetMapping(value = "/list")
    public Result<?> queryPageList(AnnualInspectionInfoQuery query,
                                   @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                   @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                   HttpServletRequest req) {
        Page<TbAnnualInspectionInfo> page = new Page<TbAnnualInspectionInfo>(pageNo, pageSize);
        IPage<TbAnnualInspectionInfo> pageList = iTbAnnualInspectionInfoService.pageBuild(page, query);
        return Result.ok(pageList);
    }

    @AddSysLog("年检信息-待办")
    @GetMapping(value = "/remindList")
    public Result<?> remindList(@CurrentUser JwtUser jwtUser, AnnualInspectionInfoQuery query,
                                @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                HttpServletRequest req) {

        Integer departCategory = projectsService.judgeDepartCategory(jwtUser);
        //超级管理员查询全部数据
        List<String> projectIds = new ArrayList<>();
        if(departCategory != 5){
            projectIds = projectsService.queryProjectListOfUser(jwtUser);
            if(CollectionUtil.isEmpty(projectIds) || projectIds.size() == 0 ){
                projectIds.add("no permission");
            }
        }
        query.setListProject(projectIds);

        Page<TbAnnualInspectionInfo> page = new Page<TbAnnualInspectionInfo>(pageNo, pageSize);
        IPage<TbAnnualInspectionInfo> pageList = iTbAnnualInspectionInfoService.remindList(page,query);
        return Result.ok(pageList);
    }

    /**
     * 添加
     */
    @AddSysLog("年检信息-添加")
    @PutMapping(value = "/add")
    public Result<?> add(@RequestBody @Validated(SaveValidated.class) TbAnnualInspectionInfo tbAnnualInspectionInfo) {
        tbAnnualInspectionInfo.setId(null);
        if (!tbAnnualInspectionInfo.getMiscellaneousFee().add(tbAnnualInspectionInfo.getServiceFee()).add(tbAnnualInspectionInfo.getTestingFee()).equals(tbAnnualInspectionInfo.getTotalFee())) {
            return Result.error("费用合计错误！");
        }
        iTbAnnualInspectionInfoService.saveItem(tbAnnualInspectionInfo);
        return Result.ok("添加成功！");
    }

    /**
     * 编辑
     */
    @AddSysLog("年检信息-编辑")
    @PutMapping(value = "/edit")
    public Result<?> edit(@RequestBody @Validated(UpdateValidated.class) TbAnnualInspectionInfo tbAnnualInspectionInfo) {
        iTbAnnualInspectionInfoService.save(tbAnnualInspectionInfo);
        return Result.ok("编辑成功！");
    }

    /**
     * 删除
     */
    @AddSysLog("年检信息-删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        iTbAnnualInspectionInfoService.removeById(id);
        return Result.ok("删除成功！");
    }

    /**
     * 批量删除
     */
    @AddSysLog("年检信息-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<?> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        iTbAnnualInspectionInfoService.removeByIds(Arrays.asList(ids.split(",")));
        return Result.ok("批量删除成功！");
    }

    /**
     * 文件上传
     */
    @AddSysLog("年检信息-文件上传")
    @PostMapping("/upload")
    public Result<?> upload(HttpServletRequest request) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        MultipartFile file = multipartRequest.getFile("file");
        if (ObjectUtil.isEmpty(file)) {
            return Result.error("请正确上传文件！");
        }
        ArrayList<String> strings = ListUtil.toList(FileExtension.EXE.getNameList());
        strings.addAll(FileExtension.OTH.getNameList());
        return Result.ok(minioUtils.uploadFileValidNotIn(file, strings));
    }
}
