package com.gci.cdnev.smallprogram.modules.vehicles.controller;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gci.cdnev.smallprogram.annotation.CurrentUser;
import com.gci.cdnev.smallprogram.annotation.Query.handle.QueryGenerator;
import com.gci.cdnev.smallprogram.common.log.annotation.AddSysLog;
import com.gci.cdnev.smallprogram.common.vo.Result;
import com.gci.cdnev.smallprogram.modules.projects.service.IProjectsService;
import com.gci.cdnev.smallprogram.modules.vehicles.entity.TbAnnualInspectionRemind;
import com.gci.cdnev.smallprogram.modules.vehicles.query.AnnualInspectionRemindQuery;
import com.gci.cdnev.smallprogram.modules.vehicles.service.ITbAnnualInspectionRemindService;
import com.gci.cdnev.smallprogram.modules.wx.entity.JwtUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@RestController
@RequestMapping("/vehicles/annualInspectionRemind")
public class TbAnnualInspectionRemindController {
    @Autowired
    private ITbAnnualInspectionRemindService iTbAnnualInspectionRemindService;
    @Autowired
    private IProjectsService projectsService;

    /**
     * 分页列表查询
     *
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AddSysLog(value = "年检提醒-分页列表查询")
    @GetMapping(value = "/list")
    public Result<?> queryPageList(@CurrentUser JwtUser jwtUser, AnnualInspectionRemindQuery query,
                                   @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                   @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                   HttpServletRequest req) {
        Integer departCategory = projectsService.judgeDepartCategory(jwtUser);
        //超级管理员查询全部数据
        List<String> projectIds = new ArrayList<>();
        if(departCategory != 5){
            projectIds = projectsService.queryProjectListOfUser(jwtUser);
            if(CollectionUtil.isEmpty(projectIds) || projectIds.size() == 0 ){
                projectIds.add("no permission");
            }
        }
        query.setListProject(projectIds);
        query.setRemindStatus("1");
        Page<TbAnnualInspectionRemind> page = new Page<TbAnnualInspectionRemind>(pageNo, pageSize);
        LambdaQueryWrapper<TbAnnualInspectionRemind> lambdaQueryWrapper = QueryGenerator.initQueryWrapper(TbAnnualInspectionRemind.class,query);
        IPage<TbAnnualInspectionRemind> pageList = iTbAnnualInspectionRemindService.pageBuild(page, lambdaQueryWrapper);
        return Result.ok(pageList);
    }

    /**
     * 添加
     */
//    @PostMapping(value = "/add")
    public Result<?> add(@RequestBody TbAnnualInspectionRemind tbAnnualInspectionRemind) {
        iTbAnnualInspectionRemindService.save(tbAnnualInspectionRemind);
        return Result.ok("添加成功！");
    }

    /**
     * 编辑
     */
//    @PutMapping(value = "/edit")
    public Result<?> edit(@RequestBody TbAnnualInspectionRemind tbAnnualInspectionRemind) {
        iTbAnnualInspectionRemindService.updateById(tbAnnualInspectionRemind);
        return Result.ok("编辑成功！");
    }

    /**
     * 删除
     */
//    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        iTbAnnualInspectionRemindService.removeById(id);
        return Result.ok("删除成功！");
    }

    /**
     * 批量删除
     */
//    @DeleteMapping(value = "/deleteBatch")
    public Result<?> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        iTbAnnualInspectionRemindService.removeByIds(Arrays.asList(ids.split(",")));
        return Result.ok("批量删除成功！");
    }

}