package com.gci.cdnev.smallprogram.modules.vehicles.controller;


import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import com.gci.cdnev.smallprogram.common.log.annotation.AddSysLog;
import com.gci.cdnev.smallprogram.common.validated.UpdateValidated;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gci.cdnev.smallprogram.annotation.CurrentUser;
import com.gci.cdnev.smallprogram.annotation.Query.handle.QueryGenerator;
import com.gci.cdnev.smallprogram.common.util.MinioUtils;
import com.gci.cdnev.smallprogram.common.validated.SaveValidated;
import com.gci.cdnev.smallprogram.common.vo.Result;
import com.gci.cdnev.smallprogram.enums.FileExtension;
import com.gci.cdnev.smallprogram.modules.vehicles.entity.TbInsuranceRecord;
import com.gci.cdnev.smallprogram.modules.vehicles.service.ITbInsuranceRecordService;
import com.gci.cdnev.smallprogram.modules.wx.entity.JwtUser;

import cn.hutool.core.util.ObjectUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * <p>
 * 出险管理 前端控制器
 * </p>
 *
 * @author xushuhan
 * @since 2023-09-22
 */
@Slf4j
@RestController
@RequestMapping("/api/insuranceRecord")
public class TbInsuranceRecordController {

	@Autowired
	private ITbInsuranceRecordService tbInsuranceRecordService;
	@Autowired
	private MinioUtils minioUtils;

	@AddSysLog("出险管理-列表查询")
	@GetMapping("/list")
	public Result<?> list(TbInsuranceRecord item) {
		LambdaQueryWrapper<TbInsuranceRecord> queryWrapper = QueryGenerator.initQueryWrapper(TbInsuranceRecord.class, item);
		queryWrapper.ge(item.getInsuranceDateStart()!=null,TbInsuranceRecord::getInsuranceDate, item.getInsuranceDateStart());
		queryWrapper.le(item.getInsuranceDateEnd()!=null,TbInsuranceRecord::getInsuranceDate, item.getInsuranceDateEnd());
		queryWrapper.orderByDesc(TbInsuranceRecord::getCreateTime);
		List<TbInsuranceRecord> list = tbInsuranceRecordService.list(queryWrapper);
		return Result.ok(list);
	}

	@AddSysLog("出险管理-分页列表查询")
	@GetMapping("/page")
	public Result<?> page(TbInsuranceRecord item,@RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
						  @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize) {

		LambdaQueryWrapper<TbInsuranceRecord> queryWrapper = QueryGenerator.initQueryWrapper(TbInsuranceRecord.class, item);
		queryWrapper.ge(item.getInsuranceDateStart()!=null,TbInsuranceRecord::getInsuranceDate, item.getInsuranceDateStart());
		queryWrapper.le(item.getInsuranceDateEnd()!=null,TbInsuranceRecord::getInsuranceDate, item.getInsuranceDateEnd());
		queryWrapper.orderByDesc(TbInsuranceRecord::getCreateTime);
		Page<TbInsuranceRecord> page = new Page<TbInsuranceRecord>(pageNo, pageSize);
		IPage<TbInsuranceRecord> pageList = tbInsuranceRecordService.page(page,queryWrapper);
		return Result.ok(pageList);
	}

	@AddSysLog("出险管理-添加")
	@PostMapping("/add")
	public Result<?> add(@CurrentUser JwtUser jwtUser,@RequestBody @Validated(SaveValidated.class) TbInsuranceRecord item){
		item.setSysOrgCode(jwtUser.getSysOrgCode());
		item.setCreateBy(jwtUser.getUserName());
		item.setSourceType("2");
		tbInsuranceRecordService.saveItem(item);
		return Result.ok();
	}

	@AddSysLog("出险管理-编辑")
	@PutMapping("/edit")
	public Result<?> edit(@RequestBody @Validated(UpdateValidated.class) TbInsuranceRecord item){
		tbInsuranceRecordService.updateItem(item);
		return Result.ok();
	}

	/**
	 * 文件上传
	 */
	@AddSysLog("出险管理-文件上传")
	@PostMapping("/upload")
	public Result<?> upload(HttpServletRequest request) {
		MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
		MultipartFile file = multipartRequest.getFile("file");
		if (ObjectUtil.isEmpty(file)) {
			return Result.error("请正确上传文件！");
		}
		return Result.ok(minioUtils.uploadFileValid(file, FileExtension.getCommonFileTypes()));
	}

}
