package com.gci.cdnev.smallprogram.modules.vehicles.controller;


import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.io.IORuntimeException;
import cn.hutool.core.io.resource.ResourceUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gci.cdnev.smallprogram.annotation.CurrentUser;
import com.gci.cdnev.smallprogram.annotation.Query.handle.QueryGenerator;
import com.gci.cdnev.smallprogram.common.log.annotation.AddSysLog;
import com.gci.cdnev.smallprogram.common.util.ValidatorUtil;
import com.gci.cdnev.smallprogram.common.validated.SaveValidated;
import com.gci.cdnev.smallprogram.common.validated.UpdateValidated;
import com.gci.cdnev.smallprogram.common.vo.Result;
import com.gci.cdnev.smallprogram.modules.projects.service.IProjectsService;
import com.gci.cdnev.smallprogram.modules.vehicles.entity.TbRefuelChargeRecord;
import com.gci.cdnev.smallprogram.modules.vehicles.service.ITbRefuelChargeRecordService;
import com.gci.cdnev.smallprogram.modules.vehicles.vo.TbRefuelChargeRecordImportVo;
import com.gci.cdnev.smallprogram.modules.wx.entity.JwtUser;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 加油充电登记 前端控制器
 * </p>
 *
 * @author admin
 * @since 2023-11-30
 */
@Slf4j
@RestController
@RequestMapping("/api/gis/refuelChargeRecord")
public class TbRefuelChargeRecordController {

	@Autowired
	private ITbRefuelChargeRecordService refuelChargeRecordService;
	@Autowired
	private IProjectsService projectsService;

	@AddSysLog(value = "加油充电登记-列表查询")
	@GetMapping("/list")
	public Result<?> list(@CurrentUser JwtUser jwtUser, TbRefuelChargeRecord item) {
		//数据权限
		Integer departCategory = projectsService.judgeDepartCategory(jwtUser);
		//超级管理员查询全部数据
		List<String> projectIds = new ArrayList<>();
		if(departCategory != 5){
			projectIds = projectsService.queryProjectListOfUser(jwtUser);
			if(CollectionUtil.isEmpty(projectIds) || projectIds.size() == 0 ){
				projectIds.add("no permission");
			}
		}
		LambdaQueryWrapper<TbRefuelChargeRecord> queryWrapper = QueryGenerator.initQueryWrapper(TbRefuelChargeRecord.class, item);
		queryWrapper.ge(item.getDateStart()!=null,TbRefuelChargeRecord::getRecordDate, item.getDateStart());
		queryWrapper.le(item.getDateEnd()!=null,TbRefuelChargeRecord::getRecordDate, item.getDateEnd());
		if(CollectionUtil.isNotEmpty(projectIds) && projectIds.size()>0){
			queryWrapper.in(TbRefuelChargeRecord::getProjectId, projectIds);
		}
		queryWrapper.orderByDesc(TbRefuelChargeRecord::getCreateTime);
		List<TbRefuelChargeRecord> list = refuelChargeRecordService.list(queryWrapper);
		return Result.ok(list);
	}

	@AddSysLog(value = "加油充电登记-分页列表查询")
	@GetMapping("/page")
    public Result<?> page(@CurrentUser JwtUser jwtUser,
						  TbRefuelChargeRecord item, @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
						  @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize) {
		//数据权限
		Integer departCategory = projectsService.judgeDepartCategory(jwtUser);
		//超级管理员查询全部数据
		List<String> projectIds = new ArrayList<>();
		if(departCategory != 5){
			projectIds = projectsService.queryProjectListOfUser(jwtUser);
			if(CollectionUtil.isEmpty(projectIds) || projectIds.size() == 0 ){
				projectIds.add("no permission");
			}
		}
		LambdaQueryWrapper<TbRefuelChargeRecord> queryWrapper = QueryGenerator.initQueryWrapper(TbRefuelChargeRecord.class, item);
		queryWrapper.ge(item.getDateStart()!=null,TbRefuelChargeRecord::getRecordDate, item.getDateStart());
		queryWrapper.le(item.getDateEnd()!=null,TbRefuelChargeRecord::getRecordDate, item.getDateEnd());
		if(CollectionUtil.isNotEmpty(projectIds) && projectIds.size()>0){
			queryWrapper.in(TbRefuelChargeRecord::getProjectId, projectIds);
		}
		queryWrapper.orderByDesc(TbRefuelChargeRecord::getCreateTime);
		Page<TbRefuelChargeRecord> page = new Page<TbRefuelChargeRecord>(pageNo, pageSize);
		IPage<TbRefuelChargeRecord> pageList = refuelChargeRecordService.page(page,queryWrapper);
		return Result.ok(pageList);
	}

	@AddSysLog(value = "加油充电登记-添加")
	@PostMapping("/add")
	public Result<?> add(@RequestBody @Validated(SaveValidated.class) TbRefuelChargeRecord item){
		refuelChargeRecordService.saveItem(item);
		return Result.ok();
	}

	@AddSysLog(value = "加油充电登记-编辑")
	@PutMapping("/edit")
	public Result<?> edit(@RequestBody @Validated(UpdateValidated.class) TbRefuelChargeRecord item){
		refuelChargeRecordService.saveItem(item);
		return Result.ok();
	}
	
	@AddSysLog(value = "加油充电登记-删除")
	@DeleteMapping("/delete")
	public Result<?> delete(TbRefuelChargeRecord item){
		refuelChargeRecordService.removeById(item.getId());
		return Result.ok();
	}
}
