package com.gci.cdnev.smallprogram.modules.vehicles.controller;


import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.excel.EasyExcel;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import com.gci.cdnev.smallprogram.annotation.CurrentUser;
import com.gci.cdnev.smallprogram.annotation.Query.handle.QueryGenerator;
import com.gci.cdnev.smallprogram.common.log.annotation.AddSysLog;
import com.gci.cdnev.smallprogram.common.util.MinioUtils;
import com.gci.cdnev.smallprogram.common.util.StringUtil;
import com.gci.cdnev.smallprogram.common.validated.SaveValidated;
import com.gci.cdnev.smallprogram.common.validated.UpdateValidated;
import com.gci.cdnev.smallprogram.common.vo.Result;
import com.gci.cdnev.smallprogram.enums.FileExtension;
import com.gci.cdnev.smallprogram.modules.driver.service.ITbDriverSignInService;
import com.gci.cdnev.smallprogram.modules.projects.entity.Projects;
import com.gci.cdnev.smallprogram.modules.projects.service.IProjectsService;
import com.gci.cdnev.smallprogram.modules.vehicles.entity.TbViolationRecord;
import com.gci.cdnev.smallprogram.modules.vehicles.service.ITbViolationRecordService;
import com.gci.cdnev.smallprogram.modules.vehicles.vo.MessageVO;
import com.gci.cdnev.smallprogram.modules.vehicles.vo.RegexTemplateVO;
import com.gci.cdnev.smallprogram.modules.vehicles.vo.VehicleProjectVO;
import com.gci.cdnev.smallprogram.modules.wx.entity.JwtUser;
import io.jsonwebtoken.lang.Collections;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;

@Api(tags="违章记录表对象")
@Slf4j
@RestController
@RequestMapping("/api/vehicles/violation")
public class TbViolationRecordController {
    @Autowired
    private ITbViolationRecordService violationRecordService;
    @Autowired
    private IProjectsService projectsService;
    @Autowired
    private MinioUtils minioUtils;
    @Autowired
    private ITbDriverSignInService driverSignInService;

    /**
     * 分页列表查询
     *
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AddSysLog("违章记录-分页列表查询")
    @GetMapping(value = "/list")
    public Result<?> queryPageList(@CurrentUser JwtUser jwtUser,
                                   TbViolationRecord violationRecord,
                                   @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                   @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                   HttpServletRequest req) {
        //小程序之查看待办的列表
        violationRecord.setHandleResult(0);
        Page<TbViolationRecord> page = new Page<TbViolationRecord>(pageNo, pageSize);
        LambdaQueryWrapper<TbViolationRecord> lambdaQueryWrapper = QueryGenerator.initQueryWrapper(TbViolationRecord.class, violationRecord);
        lambdaQueryWrapper.in(!Collections.isEmpty(jwtUser.getProjectIds()),TbViolationRecord::getProjectId, jwtUser.getProjectIds());
        IPage<TbViolationRecord> pageList = violationRecordService.page(page, lambdaQueryWrapper);
        return Result.ok(pageList);
    }

    /**
     * 添加
     */
    @AddSysLog("违章记录-添加")
    @PostMapping(value = "/add")
    public Result<?> add(@RequestBody @Validated(SaveValidated.class) TbViolationRecord violationRecord) {
        Projects projects = projectsService.getById(violationRecord.getProjectId());
        if(Objects.nonNull(projects)){
            violationRecord.setProjectName(projects.getProjectName());
        }
        violationRecordService.save(violationRecord);
        return Result.ok("添加成功！");
    }

    /**
     * 编辑
     */
    @AddSysLog("违章记录-编辑")
    @PutMapping(value = "/edit")
    public Result<?> edit(@RequestBody @Validated(UpdateValidated.class) TbViolationRecord violationRecord) {
        Projects projects = projectsService.getById(violationRecord.getProjectId());
        if(Objects.nonNull(projects)){
            violationRecord.setProjectName(projects.getProjectName());
        }
        violationRecordService.updateById(violationRecord);
        return Result.ok("编辑成功！");
    }

    /**
     * 删除
     */
    @AddSysLog("违章记录-删除")
    @DeleteMapping(value = "/realDelete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        violationRecordService.removeById(id);
        return Result.ok("删除成功！");
    }

    /**
     * 作废
     */
    @AddSysLog("违章记录-作废")
    @DeleteMapping(value = "/cancel")
    public Result<?> cancel(@RequestParam(name = "id", required = true) String id) {
        violationRecordService.update(new LambdaUpdateWrapper<TbViolationRecord>().eq(TbViolationRecord::getId,id).set(TbViolationRecord::getCancelFlag,1));
        return Result.ok("作废成功！");
    }

    /**
     * 恢复
     */
    @AddSysLog("违章记录-恢复")
    @DeleteMapping(value = "/unCancel")
    public Result<?> unCancel(@RequestParam(name = "id", required = true) String id) {
        violationRecordService.update(new LambdaUpdateWrapper<TbViolationRecord>().eq(TbViolationRecord::getId,id).set(TbViolationRecord::getCancelFlag,0));
        return Result.ok("作废成功！");
    }

    /**
     * 批量删除
     */
    @AddSysLog("违章记录-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<?> realDeleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        violationRecordService.removeByIds(Arrays.asList(ids.split(",")));
        return Result.ok("批量删除成功！");
    }

    /**
     * 文件上传
     */
    @AddSysLog("违章记录-文件上传")
    @PostMapping("/upload")
    public Result<?> upload(HttpServletRequest request) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        MultipartFile file = multipartRequest.getFile("file");
        if (ObjectUtil.isEmpty(file)) {
            return Result.error("请正确上传文件！");
        }
        return Result.ok(minioUtils.uploadFileV2Valid(file,FileExtension.PIC.getNameList()));
    }

    /**
     * 导出
     * @return
     */
    @AddSysLog("违章记录-导出")
    @GetMapping("/export")
    public Result<?> export(TbViolationRecord violationRecord,HttpServletResponse response){
        try (OutputStream output = response.getOutputStream()){

            String fileName = "违章记录表.xlsx";
            try {
                fileName = URLEncoder.encode(fileName, "UTF-8");
            } catch (UnsupportedEncodingException e) {
                e.printStackTrace();
            }
            response.setHeader("Content-disposition", "attachment;filename=" + fileName + ";" + "filename*=utf-8''" + fileName);

            LambdaQueryWrapper<TbViolationRecord> lambdaQueryWrapper = QueryGenerator.initQueryWrapper(TbViolationRecord.class, violationRecord);
            List<TbViolationRecord> list = violationRecordService.list(lambdaQueryWrapper);
            EasyExcel.write(output, TbViolationRecord.class).sheet("违章记录").doWrite(list);

        } catch (IOException e) {
            return Result.error("导出错误！");
        }
        return Result.ok();
    }

    /**
     * 获取车牌-项目-责任人列表
     */
    @AddSysLog("违章记录-获取车牌-项目-责任人列表")
    @GetMapping(value = "/queryVehicleProject")
    public Result<?> queryVehicleProject(String plateNumber) {
        List<VehicleProjectVO> list = violationRecordService.queryVehicleProject(plateNumber);
        return Result.ok(list);
    }

    /**
     * 获取车牌下拉列表
     */
    @AddSysLog("违章记录-获取车牌下拉列表")
    @GetMapping(value = "/queryAllVehicle")
    public Result<?> queryAllVehicle() {
        List<VehicleProjectVO> list = violationRecordService.queryAllVehicle();
        return Result.ok(list);
    }

    /**
     * 获取项目下拉列表
     */
    @AddSysLog("违章记录-获取项目下拉列表")
    @GetMapping(value = "/queryAllProject")
    public Result<?> queryAllProject(@CurrentUser JwtUser jwtUser) {
        List<VehicleProjectVO> list = violationRecordService.queryAllProject(jwtUser);
        return Result.ok(list);
    }

    /**
     * 解析短信获取车牌、时间、地点
     */
    @AddSysLog("违章记录-解析短信获取车牌、时间、地点")
    @PostMapping(value = "/analyzeMessage")
    public Result<?> analyzeMessage(@RequestBody RegexTemplateVO templateVO) {
        if(Objects.isNull(templateVO) || StrUtil.isBlank(templateVO.getTemplate())){
            return Result.error(101,"违章短信不能为空！");
        }
        String str = StringUtil.checkString2(templateVO.getTemplate(),500);
        if(StrUtil.isNotBlank(str)){
            return Result.error(101,str);
        }
        MessageVO vo = violationRecordService.analyzeMessage(templateVO.getTemplate());
        return Result.ok(vo);
    }

    /**
     * 新增违章行为
     */
    @AddSysLog("违章记录-新增违章行为")
    @PostMapping(value = "/addViolationBehavior")
    public Result<?> addViolationBehavior(@RequestBody MessageVO vo) {
        String str = violationRecordService.addViolationBehavior(vo);
        if(StrUtil.isNotBlank(str) && str.contains("存在")){
            return Result.error(101,str);
        }else{
            return Result.ok(str);
        }
    }

    /**
     * 车牌、时间获取打卡司机
     */
    @AddSysLog("违章记录-车牌、时间获取打卡司机")
    @GetMapping(value = "/queryDriverByPlateNumberAndTime")
    public Result<?> queryDriverByPlateNumberAndTime(String plateNumber, String time) {
        String name = driverSignInService.queryDriverByPlateNumberAndTime(plateNumber, time);
        return Result.ok(name);
    }
}
