package com.gci.cdnev.smallprogram.modules.vehicles.controller;


import cn.hutool.core.util.StrUtil;
import com.alibaba.excel.EasyExcel;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gci.cdnev.smallprogram.annotation.CurrentUser;
import com.gci.cdnev.smallprogram.common.log.annotation.AddSysLog;
import com.gci.cdnev.smallprogram.common.util.StringUtil;
import com.gci.cdnev.smallprogram.common.validated.SaveValidated;
import com.gci.cdnev.smallprogram.common.vo.Result;
import com.gci.cdnev.smallprogram.modules.basedata.entity.TbRepairShopArchives;
import com.gci.cdnev.smallprogram.modules.basedata.service.ITbRepairShopArchivesService;
import com.gci.cdnev.smallprogram.modules.gis.entity.TbGisRunningbus;
import com.gci.cdnev.smallprogram.modules.gis.service.ITbGisRunningbusService;
import com.gci.cdnev.smallprogram.modules.projects.service.IProjectsService;
import com.gci.cdnev.smallprogram.modules.system.service.ISysRoleService;
import com.gci.cdnev.smallprogram.modules.system.service.ISysUserService;
import com.gci.cdnev.smallprogram.modules.vehicles.entity.TbUpkeepBill;
import com.gci.cdnev.smallprogram.modules.vehicles.service.ITbUpkeepBillService;
import com.gci.cdnev.smallprogram.modules.vehicles.vo.*;
import com.gci.cdnev.smallprogram.modules.wx.entity.JwtUser;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

@Api(tags="保养台账")
@Slf4j
@RestController
@RequestMapping("/api/vehicles/upkeepBill")
public class UpKeepBillController {

    @Autowired
    private ITbUpkeepBillService upkeepBillService;
    @Autowired
    private ISysUserService sysUserService;
    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private ITbGisRunningbusService gisRunningbusService;
    @Autowired
    private ISysRoleService sysRoleService;
    @Autowired
    private ITbRepairShopArchivesService tbRepairShopArchivesService;
    @Autowired
    private IProjectsService projectsService;

    /**
     * 数据录入
     */
    @AddSysLog("保养台账-数据录入")
    @PostMapping(value = "/addDataEntry")
    public Result<?> addDataEntry(@RequestBody @Validated(SaveValidated.class) UpkeepDataEntryVO vo) {
        String result = upkeepBillService.addDataEntry(vo);
        if(StrUtil.isNotBlank(result)){
            if(result.contains("ID:")){
                return Result.ok(result.replace("ID:",""));
            }else{
                return Result.error(101,result);
            }
        }
        return Result.ok();
    }

    /**
     * 分页列表查询
     *
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AddSysLog("保养台账-分页列表查询")
    @GetMapping(value = "/list")
    public Result<?> queryPageList(@CurrentUser JwtUser jwtUser,
                                   UpkeepInfoListVO vo,
                                   @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                   @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                   HttpServletRequest req) {

        if(Objects.nonNull(vo.getFlowStatus()) && -1 == vo.getFlowStatus()){
            vo.setFlowStatus(null);
        }

        //识别身份：需处理内部员工身份和维修厂身份的过滤
        Integer departCategory = projectsService.judgeDepartCategory(jwtUser);
        //超级管理员查询全部数据
        if(departCategory != 5){
            //内部员工按项目列表过滤
            if(departCategory == 1 || departCategory == 4){
                List<String> listProjectStaff = projectsService.queryProjectListOfUser(jwtUser);
                if(CollectionUtils.isNotEmpty(listProjectStaff)){
                    vo.setListProject(listProjectStaff);
                }else {
                    vo.setProjectId("no permission");
                }
            }else if(departCategory == 2){
                List<TbRepairShopArchives> listRepairShop = tbRepairShopArchivesService.getRepairShopByUserId(jwtUser.getId());
                List<String> listRepairShopIds = listRepairShop.stream().map(TbRepairShopArchives::getId).collect(Collectors.toList());
                if(CollectionUtils.isNotEmpty(listRepairShopIds)){
                    vo.setListRepairShop(listRepairShopIds);
                }else {
                    vo.setProjectId("no permission");
                }
            }else{
                vo.setProjectId("no permission");
            }
        }

        Page<UpkeepInfoListVO> page = new Page<UpkeepInfoListVO>(pageNo, pageSize);
        IPage<UpkeepInfoListVO> pageList = upkeepBillService.getUpkeepBillList(page, vo,jwtUser);
        return Result.ok(pageList);
    }

    /**
     * 导出
     * @return
     */
    @AddSysLog("保养台账-导出")
    @GetMapping("/export")
    public Result<?> export(UpkeepInfoListVO vo, HttpServletResponse response){
        try (OutputStream output = response.getOutputStream()){
            String fileName = "保养台账表.xlsx";
            try {
                fileName = URLEncoder.encode(fileName, "UTF-8");
            } catch (UnsupportedEncodingException e) {
                e.printStackTrace();
            }
            response.setHeader("Content-disposition", "attachment;filename=" + fileName + ";" + "filename*=utf-8''" + fileName);
            List<UpkeepInfoListVO> list = upkeepBillService.getAllUpkeepBillList(vo);
            EasyExcel.write(output, UpkeepInfoListVO.class).sheet("保养台账").doWrite(list);

        } catch (IOException e) {
            return Result.error("导出错误！");
        }
        return Result.ok();
    }

    /**
     * 作废
     */
    @AddSysLog("保养台账-作废")
    @PostMapping(value = "/cancel")
    public Result<?> cancel(@CurrentUser JwtUser jwtUser,
                            @RequestBody CancelVO vo) {
        String arg = StringUtil.checkString(vo.getCancelIdea(),200);
        if(StrUtil.isNotBlank(arg)){
            return Result.error(101,"作废原因 " + arg);
        }
        String realName = "";
        if(StrUtil.isNotBlank(vo.getRealname())){
            realName = vo.getRealname();
        }else{
            if(Objects.nonNull(jwtUser)){
                realName = jwtUser.getRealname();
            }
        }
        //查当前总里程
        TbUpkeepBill tbUpkeepBill = upkeepBillService.getById(vo.getId());
        LambdaQueryWrapper<TbGisRunningbus> busQuery = new LambdaQueryWrapper<>();
        busQuery.eq(TbGisRunningbus::getPlateNumber,tbUpkeepBill.getPlateNumber()).last(" limit 1 ");
        TbGisRunningbus runningbus = gisRunningbusService.getOne(busQuery);

        upkeepBillService.update(new LambdaUpdateWrapper<TbUpkeepBill>().eq(TbUpkeepBill::getId,vo.getId())
                .set(TbUpkeepBill::getFlowStatus,9)
                .set(TbUpkeepBill::getFlowNode,4)
                .set(TbUpkeepBill::getCancelIdea,vo.getCancelIdea())
                .set(TbUpkeepBill::getCancelPeople,realName)
                .set(TbUpkeepBill::getCancelTime,new Date()));
        return Result.ok("作废成功！");
    }

    /**
     * 根据保养单id获取保养弹窗详情
     */
    @AddSysLog("保养台账-根据保养单id获取保养弹窗详情")
    @GetMapping(value = "/getUpkeepObjectById")
    public Result<?> getUpkeepObjectById(@CurrentUser JwtUser jwtUser,String id) {
        UpkeepObjectVO vo = upkeepBillService.getUpkeepObjectById(id,jwtUser);
        return Result.ok(vo);
    }

    /**
     * 保养台账流程提交
     */
    @AddSysLog("保养台账-保养台账流程提交")
    @PostMapping(value = "/submitUpkeepObject")
    public Result<?> submitUpkeepObject(@CurrentUser JwtUser jwtUser,
                                        @RequestBody @Validated(SaveValidated.class) UpkeepObjectVO vo) {

        if(redisTemplate.hasKey(vo.getId()+"_submitRepairObject")){
            return Result.error(101,"该保养单正在被操作中，无法操作，请刷新!");
        }else{
            redisTemplate.opsForValue().set(vo.getId()+"_submitUpkeepObject", vo.getId(), 10, TimeUnit.SECONDS);
        }

        String result = "";
        try {
            result = upkeepBillService.submitUpkeepObject(vo,jwtUser);
        }catch (Exception e){
            e.printStackTrace();
            return Result.error(101,"提交失败：" + e.getMessage());
        }finally {
            redisTemplate.delete(vo.getId()+"_submitUpkeepObject");
        }
        if(StrUtil.isNotBlank(result)){
            return Result.error(101,result);
        }
        return Result.ok("提交成功！");
    }

    /**
     * 查询--保养台账列表数量统计
     * @return
     */
    @AddSysLog("保养台账-查询--保养台账列表数量统计")
    @GetMapping(value = "/getUpkeepPageStatic")
    public Result<?> getUpkeepPageStatic(@CurrentUser JwtUser jwtUser,
                                         UpkeepInfoListVO vo) {
        //识别身份：需处理内部员工身份和维修厂身份的过滤
        Integer departCategory = projectsService.judgeDepartCategory(jwtUser);
        //超级管理员查询全部数据
        if(departCategory != 5){
            //内部员工按项目列表过滤
            if(departCategory == 1 || departCategory == 4){
                List<String> listProjectStaff = projectsService.queryProjectListOfUser(jwtUser);
                if(CollectionUtils.isNotEmpty(listProjectStaff)){
                    vo.setListProject(listProjectStaff);
                }else {
                    vo.setProjectId("no permission");
                }
            }else if(departCategory == 2){
                List<TbRepairShopArchives> listRepairShop = tbRepairShopArchivesService.getRepairShopByUserId(jwtUser.getId());
                List<String> listRepairShopIds = listRepairShop.stream().map(TbRepairShopArchives::getId).collect(Collectors.toList());
                if(CollectionUtils.isNotEmpty(listRepairShopIds)){
                    vo.setListRepairShop(listRepairShopIds);
                }else {
                    vo.setProjectId("no permission");
                }
            }else{
                vo.setProjectId("no permission");
            }
        }

        Page<UpkeepBillPageStaticVO> page = new Page<UpkeepBillPageStaticVO>(1, -1);
        List<UpkeepBillPageStaticVO> pageList = upkeepBillService.getUpkeepPageStatic(page,vo,jwtUser);
        return Result.ok(pageList);
    }
}
