package com.gci.cdnev.smallprogram.modules.vehicles.controller;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import com.alibaba.excel.EasyExcel;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gci.cdnev.smallprogram.annotation.CurrentUser;
import com.gci.cdnev.smallprogram.common.log.annotation.AddSysLog;
import com.gci.cdnev.smallprogram.common.util.StringUtil;
import com.gci.cdnev.smallprogram.common.validated.SaveValidated;
import com.gci.cdnev.smallprogram.common.validated.UpdateValidated;
import com.gci.cdnev.smallprogram.common.vo.Result;
import com.gci.cdnev.smallprogram.modules.projects.entity.Projects;
import com.gci.cdnev.smallprogram.modules.projects.service.IProjectsService;
import com.gci.cdnev.smallprogram.modules.system.service.ISysUserService;
import com.gci.cdnev.smallprogram.modules.vehicles.entity.TbUpkeepPlan;
import com.gci.cdnev.smallprogram.modules.vehicles.mapper.TbUpkeepPlanMapper;
import com.gci.cdnev.smallprogram.modules.vehicles.service.ITbUpkeepPlanService;
import com.gci.cdnev.smallprogram.modules.vehicles.vo.*;
import com.gci.cdnev.smallprogram.modules.wx.entity.JwtUser;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.TimeUnit;

@Api(tags="保养计划")
@Slf4j
@RestController
@RequestMapping("/api/vehicles/upkeepPlan")
public class UpKeepPlanController {

    @Autowired
    private ITbUpkeepPlanService upkeepPlanService;
    @Autowired
    private IProjectsService projectsService;
    @Autowired
    private ISysUserService sysUserService;
    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private TbUpkeepPlanMapper upkeepPlanMapper;

    /**
     * 分页列表查询
     *
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AddSysLog("保养计划-分页列表查询")
    @GetMapping(value = "/list")
    public Result<?> queryPageList(@CurrentUser JwtUser jwtUser,
                                   TbUpkeepPlan tbUpkeepPlan,
                                   @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                   @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                   HttpServletRequest req) {
        if(Objects.nonNull(tbUpkeepPlan.getAuditStatus()) && "-1".equals(tbUpkeepPlan.getAuditStatus())){
            //适配小程序查全部
            tbUpkeepPlan.setAuditStatus(null);
        }
        //项目权限
        Integer departCategory = projectsService.judgeDepartCategory(jwtUser);
        //超级管理员查询全部数据
        List<String> projectIds = new ArrayList<>();
        if(departCategory != 5){
            projectIds = projectsService.queryProjectListOfUser(jwtUser);
            if(CollectionUtil.isEmpty(projectIds) || projectIds.size() == 0 ){
                projectIds.add("no permission");
            }
        }
        tbUpkeepPlan.setListProject(projectIds);

        Page<TbUpkeepPlan> page = new Page<TbUpkeepPlan>(pageNo, pageSize);
        IPage<TbUpkeepPlan> pageList = upkeepPlanService.getUpkeepPlanList(page, tbUpkeepPlan,jwtUser);
        return Result.ok(pageList);
    }

    /**
     * 添加
     */
    @AddSysLog("保养计划-添加")
    @PostMapping(value = "/add")
    public Result<?> add(@CurrentUser JwtUser jwtUser, @RequestBody @Validated(SaveValidated.class) TbUpkeepPlan tbUpkeepPlan) {
        Projects projects = projectsService.getById(tbUpkeepPlan.getProjectId());
        if(Objects.nonNull(projects)){
            tbUpkeepPlan.setProjectName(projects.getProjectName());
        }
        //手动生成
        tbUpkeepPlan.setSaveType(2);
        tbUpkeepPlan.setAuditStatus("1");
        tbUpkeepPlan.setPlanSaveTime(new Date());

        String username = "";
        if(Objects.nonNull(jwtUser)){
            username = jwtUser.getUserName();
        }
        if(Objects.nonNull(tbUpkeepPlan.getAttachmentList()) && tbUpkeepPlan.getAttachmentList().size()>0){
            tbUpkeepPlan.setAttachmentJson(JSONUtil.toJsonPrettyStr(tbUpkeepPlan.getAttachmentList()));
        }
        String format = DateUtil.format(new Date(),"yyyyMMddHHmmssSSS");
        tbUpkeepPlan.setOrderNo(format);

        tbUpkeepPlan.setCreateBy(username);
        upkeepPlanService.save(tbUpkeepPlan);
        return Result.ok(tbUpkeepPlan.getId());
    }

    /**
     * 编辑
     */
    @AddSysLog("保养计划-编辑")
    @PutMapping(value = "/edit")
    public Result<?> edit(@RequestBody @Validated(UpdateValidated.class) TbUpkeepPlan tbUpkeepPlan) {
        Projects projects = projectsService.getById(tbUpkeepPlan.getProjectId());
        if(Objects.nonNull(projects)){
            tbUpkeepPlan.setProjectName(projects.getProjectName());
        }
        //审核未通过的编辑-》状态跳回未审核？
        TbUpkeepPlan one = upkeepPlanService.getById(tbUpkeepPlan.getId());

        if("3".equals(one.getAuditStatus())){
            tbUpkeepPlan.setAuditStatus("1");
        }
        if(Objects.nonNull(tbUpkeepPlan.getAttachmentList()) && tbUpkeepPlan.getAttachmentList().size()>0){
            tbUpkeepPlan.setAttachmentJson(JSONUtil.toJsonPrettyStr(tbUpkeepPlan.getAttachmentList()));
        }
        upkeepPlanService.updateById(tbUpkeepPlan);
        return Result.ok("编辑成功！");
    }

    /**
     * 删除
     */
    @AddSysLog("保养计划-删除")
    @DeleteMapping(value = "/realDelete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        upkeepPlanService.removeById(id);
        return Result.ok("删除成功！");
    }

    /**
     * 作废
     */
    @AddSysLog("保养计划-作废")
    @PostMapping(value = "/cancel")
    public Result<?> cancel(@CurrentUser JwtUser jwtUser,
                            @RequestBody CancelVO vo) {
        String arg = StringUtil.checkString(vo.getCancelIdea(),200);
        if(StrUtil.isNotBlank(arg)){
            return Result.error(101,"作废原因 " + arg);
        }
        String realName = "";
        if(StrUtil.isNotBlank(vo.getRealname())){
            realName = vo.getRealname();
        }else{
            if(Objects.nonNull(jwtUser)){
                realName = jwtUser.getRealname();
            }
        }

        upkeepPlanService.update(new LambdaUpdateWrapper<TbUpkeepPlan>().eq(TbUpkeepPlan::getId,vo.getId())
                .set(TbUpkeepPlan::getAuditStatus,"4")
                .set(TbUpkeepPlan::getCancelIdea,vo.getCancelIdea())
                .set(TbUpkeepPlan::getCancelPeople,realName)
                .set(TbUpkeepPlan::getCancelTime,new Date()));
        return Result.ok("作废成功！");
    }


    /**
     * 导出
     * @return
     */
    @AddSysLog("保养计划-导出")
    @GetMapping("/export")
    public Result<?> export(TbUpkeepPlan tbUpkeepPlan, HttpServletResponse response){
        try (OutputStream output = response.getOutputStream()){

            String fileName = "保养计划表.xlsx";
            try {
                fileName = URLEncoder.encode(fileName, "UTF-8");
            } catch (UnsupportedEncodingException e) {
                e.printStackTrace();
            }
            response.setHeader("Content-disposition", "attachment;filename=" + fileName + ";" + "filename*=utf-8''" + fileName);

            List<TbUpkeepPlan> list = upkeepPlanMapper.findUpkeepAllList(tbUpkeepPlan);
            EasyExcel.write(output, TbUpkeepPlan.class).sheet("保养计划").doWrite(list);

        } catch (IOException e) {
            return Result.error("导出错误！");
        }
        return Result.ok();
    }

    /**
     * 根据保养单id获取保养计划弹窗详情
     */
    @AddSysLog("保养计划-根据保养单id获取保养计划弹窗详情")
    @GetMapping(value = "/getUpkeepPlanObjectById")
    public Result<?> getUpkeepPlanObjectById(@CurrentUser JwtUser jwtUser,
                                             String id) {
        UpkeepPlanObjectVO vo = upkeepPlanService.getUpkeepPlanObjectById(id,jwtUser);
        return Result.ok(vo);
    }

    /**
     * 保养计划审核提交
     */
    @AddSysLog("保养计划-保养计划审核提交")
    @PostMapping(value = "/submitUpkeepPlan")
    public Result<?> submitUpkeepPlan(@CurrentUser JwtUser jwtUser,
                                      @RequestBody @Validated(SaveValidated.class) UpkeepPlanObjectVO vo) {

        if(redisTemplate.hasKey(vo.getId()+"_submitUpkeepPlan")){
            return Result.error(101,"该保养计划正在被操作中，无法操作，请刷新!");
        }else{
            redisTemplate.opsForValue().set(vo.getId()+"_submitUpkeepPlan", vo.getId(), 10, TimeUnit.SECONDS);
        }

        String result = "";
        try {
            result = upkeepPlanService.submitUpkeepPlan(vo,jwtUser);
        }catch (Exception e){
            e.printStackTrace();
            return Result.error(101,"提交失败：" + e.getMessage());
        }finally {
            redisTemplate.delete(vo.getId()+"_submitUpkeepPlan");
        }
        if(StrUtil.isNotBlank(result)){
            return Result.error(101,result);
        }
        return Result.ok("提交成功！");
    }

    /**
     * 规则生成保养计划
     */
    @AddSysLog("保养计划-规则生成保养计划")
    @GetMapping(value = "/ruleCreatePlan")
    public Result<?> ruleCreatePlan() {
        upkeepPlanService.ruleCreatePlan();
        return Result.ok();
    }

//    /**
//     * 定时任务 规则生成保养计划 每个月最后一天五点
//     */
//    @Scheduled(cron = "0 0 5 L * ?")
//    public void scheduledRuleCreatePlan() {
//        upkeepPlanService.ruleCreatePlan();
//    }


    /**
     * 查询--保养计划列表数量统计--只有维保部看的
     * @return
     */
    @AddSysLog("保养计划-查询--保养计划列表数量统计--只有维保部看的")
    @GetMapping(value = "/getUpkeepPageStatic")
    public Result<?> getUpkeepPageStatic(@CurrentUser JwtUser jwtUser,
                                         TbUpkeepPlan tbUpkeepPlan) {

        Page<UpkeepPlanPageStaticVO> page = new Page<UpkeepPlanPageStaticVO>(1, -1);
        List<UpkeepPlanPageStaticVO> pageList = upkeepPlanService.getUpkeepPlanPageStatic(page,tbUpkeepPlan,jwtUser);
        return Result.ok(pageList);
    }
}
