package com.gci.cdnev.smallprogram.modules.vehicles.entity;

import com.baomidou.mybatisplus.annotation.*;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.gci.cdnev.smallprogram.annotation.fileUrl.FileUrl;
import com.gci.cdnev.smallprogram.annotation.fileUrl.handle.FileUrlHandle;
import com.gci.cdnev.smallprogram.common.dict.annotation.Dict;
import com.gci.cdnev.smallprogram.common.validated.SaveValidated;
import com.gci.cdnev.smallprogram.common.validated.UpdateValidated;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.Digits;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;

/**
 * <p>
 * 事故保险信息
 * </p>
 *
 * @author 2464
 * @since 2023-09-13
 */
@Data
@AllArgsConstructor
@NoArgsConstructor
@TableName("tb_accident_insurance_info")
@ApiModel(value = "TbAccidentInsuranceInfo对象", description = "事故保险信息")
public class TbAccidentInsuranceInfo implements Serializable {

    private static final long serialVersionUID = 1L;

    @ApiModelProperty("id")
    @TableId(value = "id", type = IdType.ASSIGN_ID)
    private String id;

    @ApiModelProperty("所属机构")
    private String sysOrgCode;

    @ApiModelProperty("创建时间")
    private Date createTime;

    @ApiModelProperty("创建人")
    private String createBy;

    @ApiModelProperty("更新时间")
    private Date updateTime;

    @ApiModelProperty("更新人")
    private String updateBy;

    @ApiModelProperty("事故信息主表id")
    private String infoId;

    @ApiModelProperty("交案回执单号")
    @Size(max = 32,message = "交案回执单号不能超过32位字符！",groups = {SaveValidated.class, UpdateValidated.class})
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private String caseReceiptNumber;

    @ApiModelProperty("确认事故责任")
    @Dict(dicCode = "liability_type")
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private String liabilityConfirm;

    @ApiModelProperty("确认事故性质")
    @Dict(dicCode = "nature_type")
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private String natureConfirm;

    @ApiModelProperty("标的车损失金额")
    @Digits(integer = 9,fraction = 2,message = "标的车损失金额整数部分不能大于9位，小数部分不能大于2位",groups = {SaveValidated.class, UpdateValidated.class})
    private BigDecimal vehicleLossAmount;

    @ApiModelProperty("标的车人伤金额")
    @Digits(integer = 9,fraction = 2,message = "标的车人伤金额整数部分不能大于9位，小数部分不能大于2位",groups = {SaveValidated.class, UpdateValidated.class})
    @Min(value = 0,message = "标的车人伤金额最小值为0！",groups = {SaveValidated.class,UpdateValidated.class})
    private BigDecimal vehiclePersonalInjuryAmount;

    @ApiModelProperty("第三方物损金额")
    @Digits(integer = 9,fraction = 2,message = "第三方物损金额整数部分不能大于9位，小数部分不能大于2位",groups = {SaveValidated.class, UpdateValidated.class})
    @Min(value = 0,message = "第三方物损金额最小值为0！",groups = {SaveValidated.class,UpdateValidated.class})
    private BigDecimal thirdPropertyDamageAmount;

    @ApiModelProperty("第三方人伤金额")
    @Digits(integer = 9,fraction = 2,message = "第三方人伤金额整数部分不能大于9位，小数部分不能大于2位",groups = {SaveValidated.class, UpdateValidated.class})
    @Min(value = 0,message = "第三方人伤金额最小值为0！",groups = {SaveValidated.class,UpdateValidated.class})
    private BigDecimal thirdPersonalInjuryAmount;

    @ApiModelProperty("预估损失总金额")
    @Digits(integer = 10,fraction = 2,message = "预估损失总金额整数部分不能大于10位，小数部分不能大于2位",groups = {SaveValidated.class, UpdateValidated.class})
    @Min(value = 0,message = "预估损失总金额最小值为0！",groups = {SaveValidated.class,UpdateValidated.class})
    private BigDecimal estimatedTotalLossAmount;

    @ApiModelProperty("理赔至公司账户日期")
    @JsonFormat(timezone = "GMT+8",pattern = "yyyy-MM-dd")
    @DateTimeFormat(pattern="yyyy-MM-dd")
    private Date claimToCompanyAccountDate;

    @ApiModelProperty("理赔至公司账户金额")
    @Digits(integer = 9,fraction = 2,message = "理赔至公司账户金额整数部分不能大于9位，小数部分不能大于2位",groups = {SaveValidated.class, UpdateValidated.class})
    @Min(value = 0,message = "理赔至公司账户金额最小值为0！",groups = {SaveValidated.class,UpdateValidated.class})
    private BigDecimal claimToCompanyAccountAmount;

    @ApiModelProperty("理赔至第三方账户日期")
    @JsonFormat(timezone = "GMT+8",pattern = "yyyy-MM-dd")
    @DateTimeFormat(pattern="yyyy-MM-dd")
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private Date claimToThirdAccountDate;

    @ApiModelProperty("理赔至第三方账户金额")
    @Digits(integer = 9,fraction = 2,message = "理赔至第三方账户金额整数部分不能大于9位，小数部分不能大于2位",groups = {SaveValidated.class, UpdateValidated.class})
    @Min(value = 0,message = "理赔至第三方账户金额最小值为0！",groups = {SaveValidated.class,UpdateValidated.class})
    private BigDecimal claimToThirdAccountAmount;

    @ApiModelProperty("赔偿单据图片")
    @JsonDeserialize(using = FileUrlHandle.class)
    @FileUrl
    private String documentImage;

    @ApiModelProperty("出险赔偿总金额")
    @Digits(integer = 10,fraction = 2,message = "出险赔偿总金额部分不能大于10位，小数部分不能大于2位",groups = {SaveValidated.class, UpdateValidated.class})
    @Min(value = 0,message = "出险赔偿总金额最小值为0！",groups = {SaveValidated.class,UpdateValidated.class})
    private BigDecimal totalCompensationAmount;

    @ApiModelProperty("出险未赔偿金额")
    @Digits(integer = 9,fraction = 2,message = "出险未赔偿金额整数部分不能大于9位，小数部分不能大于2位",groups = {SaveValidated.class, UpdateValidated.class})
    @Min(value = 0,message = "出险未赔偿金额最小值为0！",groups = {SaveValidated.class,UpdateValidated.class})
    private BigDecimal uncompensatedAmount;

    @ApiModelProperty("案件实际发生总金额")
    @Digits(integer = 10,fraction = 2,message = "案件实际发生总金额整数部分不能大于10位，小数部分不能大于2位",groups = {SaveValidated.class, UpdateValidated.class})
    @Min(value = 0,message = "案件实际发生总金额最小值为0！",groups = {SaveValidated.class,UpdateValidated.class})
    private BigDecimal totalAmountActuallyIncurred;

    @ApiModelProperty("是否同步出险")
    @Dict(dicCode = "yes_no")
    @NotNull(message = "是否同步出险不能为空！",groups = {SaveValidated.class,UpdateValidated.class})
    private String isRecord;

    @ApiModelProperty("保险公司")
    @Dict(dictTable = "tb_insurance_company_archives", dicText = "name", dicCode = "id")
    private String companyId;

    @ApiModelProperty("出险日期")
    @JsonFormat(timezone = "GMT+8",pattern = "yyyy-MM-dd")
    @DateTimeFormat(pattern="yyyy-MM-dd")
    private Date insuranceDate;
}
