package com.gci.cdnev.smallprogram.modules.vehicles.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gci.cdnev.smallprogram.annotation.Query.handle.QueryGenerator;
import com.gci.cdnev.smallprogram.constant.CommonConstant;
import com.gci.cdnev.smallprogram.modules.basedata.entity.TbRepairShopArchives;
import com.gci.cdnev.smallprogram.modules.basedata.entity.TbVehicle;
import com.gci.cdnev.smallprogram.modules.basedata.service.ITbRepairShopArchivesService;
import com.gci.cdnev.smallprogram.modules.basedata.service.ITbVehicleService;
import com.gci.cdnev.smallprogram.modules.customer.entity.TbCustomerFollow;
import com.gci.cdnev.smallprogram.modules.customer.service.ITbCustomerFollowPartService;
import com.gci.cdnev.smallprogram.modules.customer.service.ITbCustomerFollowService;
import com.gci.cdnev.smallprogram.modules.customer.service.ITbCustomerOpinionService;
import com.gci.cdnev.smallprogram.modules.projects.service.IProjectsService;
import com.gci.cdnev.smallprogram.modules.system.entity.SysPermission;
import com.gci.cdnev.smallprogram.modules.system.service.ISysDictService;
import com.gci.cdnev.smallprogram.modules.system.service.ISysPermissionService;
import com.gci.cdnev.smallprogram.modules.system.vo.DictModel;
import com.gci.cdnev.smallprogram.modules.vehicles.entity.*;
import com.gci.cdnev.smallprogram.modules.vehicles.enums.BacklogEnum;
import com.gci.cdnev.smallprogram.modules.vehicles.enums.BacklogTabEnum;
import com.gci.cdnev.smallprogram.modules.vehicles.query.AnnualInspectionRemindQuery;
import com.gci.cdnev.smallprogram.modules.vehicles.service.*;
import com.gci.cdnev.smallprogram.modules.vehicles.vo.*;
import com.gci.cdnev.smallprogram.modules.wx.entity.JwtUser;
import io.jsonwebtoken.lang.Collections;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.shiro.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

@Service
@Slf4j
public class BacklogServiceImpl implements BacklogService {
    @Autowired
    private ITbViolationRecordService violationRecordService;
    @Autowired
    private ITbRepairInfoService repairInfoService;
    @Autowired
    private ITbUpkeepBillService upkeepBillService;
    @Autowired
    private ITbAccidentInfoService iTbAccidentInfoService;
    @Autowired
    private ITbInsuranceRecordService tbInsuranceRecordService;
    @Autowired
    private ITbAnnualInspectionInfoService iTbAnnualInspectionInfoService;
    @Autowired
    private ITbVehicleService vehicleService;
    @Autowired
    private ISysDictService sysDictService;
    @Autowired
    private CommonService commonService;
    @Autowired
    private ITbRepairShopArchivesService tbRepairShopArchivesService;
    @Autowired
    private IProjectsService projectsService;
    @Autowired
    private ITbAnnualInspectionRemindService iTbAnnualInspectionRemindService;
    @Autowired
    private ITbCustomerFollowPartService tbCustomerFollowPartService;
    @Autowired
    private ITbCustomerOpinionService tbCustomerOpinionService;
    @Autowired
    private ITbCustomerFollowService tbCustomerFollowService;


    @Override
    public List<BacklogStatisVO> getBacklogCount(JwtUser jwtUser) {
        //数据权限
        Integer departCategory = projectsService.judgeDepartCategory(jwtUser);
        //超级管理员查询全部数据
        List<String> projectIds = new ArrayList<>();
        if(departCategory != 5){
            projectIds = projectsService.queryProjectListOfUser(jwtUser);
            if(CollectionUtil.isEmpty(projectIds) || projectIds.size() == 0 ){
                projectIds.add("no permission");
            }
        }

        //1-司机，2-维修厂，3-司机&维修厂，4-维保管理员，5-维保领导，6-维保管理员&维保领导
        List<BacklogStatisVO> result = new ArrayList<>();
        // 需要处理的 BacklogEnum 列表
        List<BacklogTabEnum> backlogTabEnums = Arrays.asList(
                BacklogTabEnum.VIOLATION,
                BacklogTabEnum.REPAIR,
                BacklogTabEnum.UPKEEP,
                BacklogTabEnum.INSURANCE,
                BacklogTabEnum.ACCIDENNT,
                BacklogTabEnum.AS,
                BacklogTabEnum.FOLLOW
        );

        for (BacklogTabEnum backlogTabEnum : backlogTabEnums) {
            BacklogStatisVO backlogVo = new BacklogStatisVO();
            backlogVo.setName(backlogTabEnum.getName());
            backlogVo.setType(backlogTabEnum.getType());
            backlogVo.setCount(getPageCount(jwtUser, backlogTabEnum,projectIds));
            result.add(backlogVo);
        }
        return result;
    }

    @Override
    public Object getBacklogPageList(JwtUser jwtUser, Integer pageNo, Integer pageSize, String type) {
        //数据权限
        Integer departCategory = projectsService.judgeDepartCategory(jwtUser);
        //超级管理员查询全部数据
        List<String> projectIds = new ArrayList<>();
        if(departCategory != 5){
            projectIds = projectsService.queryProjectListOfUser(jwtUser);
            if(CollectionUtil.isEmpty(projectIds) || projectIds.size() == 0 ){
                projectIds.add("no permission");
            }
        }
        IPage<BacklogListVO> result =  new Page<>(pageNo, pageSize);
        List<BacklogListVO> list = new ArrayList<>();
        //1出险 2违章 3事故 4维修 5保养 6年检 7客户回访
        if("1".equals(type)){
            IPage<TbInsuranceRecord> insuranceRecordPageList = getInsurancePage(jwtUser,pageNo,pageSize,projectIds);
            List<TbInsuranceRecord> insuranceRecordList = insuranceRecordPageList.getRecords();
            insuranceRecordList.stream().forEach(item->{
                BacklogListVO vo = new BacklogListVO();
                vo.setId(item.getId());
                TbVehicle tbVehicle = vehicleService.getById(item.getVehicleId());
                if(Objects.nonNull(tbVehicle)){
                    vo.setPlateNumber(tbVehicle.getPlateNumber());
                }
                vo.setProjectId(item.getProjectId());
                vo.setTime(DateUtil.format(item.getInsuranceDate(),"yyyy-MM-dd"));
                vo.setType(type);
                vo.setName(BacklogTabEnum.getEnumByType(type).getName());
                list.add(vo);
            });
            result.setTotal(insuranceRecordPageList.getTotal());
            result.setRecords(list);
        }
        if("2".equals(type)){
            IPage<TbViolationRecord> violationPageList = getViolationPage(jwtUser,pageNo,pageSize,projectIds);
            List<TbViolationRecord> violationRecordList = violationPageList.getRecords();
            violationRecordList.stream().forEach(item->{
                BacklogListVO vo = new BacklogListVO();
                vo.setId(item.getId());
                vo.setPlateNumber(item.getPlateNumber());
                vo.setProjectId(item.getProjectId());
                vo.setTime(DateUtil.format(item.getViolationTime(),"yyyy-MM-dd HH:mm:ss"));
                vo.setViolationAct(item.getViolationAct());
                vo.setType(type);
                vo.setName(BacklogTabEnum.getEnumByType(type).getName());
                list.add(vo);
            });
            result.setTotal(violationPageList.getTotal());
            result.setRecords(list);
        }
        if("3".equals(type)){
            IPage<TbAccidentInfo> accidentpageList = getAccidentPage(jwtUser,pageNo,pageSize,projectIds);
            List<TbAccidentInfo> accidentInfoList = accidentpageList.getRecords();
            accidentInfoList.stream().forEach(item->{
                BacklogListVO vo = new BacklogListVO();
                vo.setId(item.getId());
                vo.setPlateNumber(item.getPlateNumber());
                vo.setProjectId(item.getProjectId());
                vo.setTime(DateUtil.format(item.getAccidentTime(),"yyyy-MM-dd"));
                vo.setNatureEstimate(item.getNatureEstimate());
                vo.setType(type);
                vo.setName(BacklogTabEnum.getEnumByType(type).getName());
                list.add(vo);
            });
            result.setTotal(accidentpageList.getTotal());
            result.setRecords(list);
        }
        if("4".equals(type)){
            List<DictModel> vehicleTypeList =  sysDictService.queryDictItemsByCode("repair_status");
            Map<String, String> repairMap = vehicleTypeList.stream().collect(Collectors.toMap(DictModel::getValue, DictModel::getText));

            IPage<RepairInfoListVO> repairpageList = getRepairPage(jwtUser,pageNo,pageSize);
            List<RepairInfoListVO> repairList = repairpageList.getRecords();
            repairList.stream().forEach(item->{
                BacklogListVO vo = new BacklogListVO();
                vo.setId(item.getId());
                vo.setPlateNumber(item.getPlateNumber());
                vo.setProjectId(item.getProjectId());
                //格式年月日
                vo.setTime(DateUtil.format(item.getRepairTime(),"yyyy-MM-dd"));
                vo.setFlowStatus(item.getFlowStatus());
                vo.setOrderNo(item.getOrderNo());
                vo.setFlowStatusName(repairMap.getOrDefault(item.getFlowStatus().toString(),""));
                vo.setType(type);
                vo.setName(BacklogTabEnum.getEnumByType(type).getName());
                list.add(vo);
            });
            result.setTotal(repairpageList.getTotal());
            result.setRecords(list);
        }
        if("5".equals(type)){
            IPage<UpkeepInfoListVO> upkeepPageList = getUpkeepPage(jwtUser,pageNo,pageSize);
            List<UpkeepInfoListVO> upkeepList = upkeepPageList.getRecords();

            List<DictModel> vehicleTypeList =  sysDictService.queryDictItemsByCode("upkeep_status");
            Map<String, String> upkeepMap = vehicleTypeList.stream().collect(Collectors.toMap(DictModel::getValue, DictModel::getText));
            upkeepList.stream().forEach(item->{
                BacklogListVO vo = new BacklogListVO();
                vo.setId(item.getId());
                vo.setPlateNumber(item.getPlateNumber());
                vo.setProjectId(item.getProjectId());
                vo.setTime(DateUtil.format(item.getPredictTime(),"yyyy-MM-dd"));
                vo.setFlowStatus(item.getFlowStatus());
                vo.setOrderNo(item.getOrderNo());
                vo.setType(type);
                vo.setName(BacklogTabEnum.getEnumByType(type).getName());
                vo.setPlanOrBill(item.getPlanOrBill());
                if("bill".equals(item.getPlanOrBill())){
                    vo.setFlowStatusName(upkeepMap.getOrDefault(item.getFlowStatus().toString(),""));
                }else{
                    vo.setFlowStatusName("未审批");
                    vo.setFlowStatus(4);
                }
                list.add(vo);
            });
            result.setTotal(upkeepPageList.getTotal());
            result.setRecords(list);
        }
        if("6".equals(type)){
            IPage<TbAnnualInspectionRemind> asPageList  = getASPage(jwtUser,pageNo,pageSize,projectIds);
            List<TbAnnualInspectionRemind> asList = asPageList.getRecords();

            List<DictModel> vehicleTypeList =  sysDictService.queryDictItemsByCode("overdue_type");
            Map<String, String> vehicleTypeMap = vehicleTypeList.stream().collect(Collectors.toMap(DictModel::getValue, DictModel::getText));

            asList.stream().forEach(item->{
                BacklogListVO vo = new BacklogListVO();
                vo.setId(item.getInfoId());
                vo.setPlateNumber(item.getPlateNumber());
                vo.setProjectId(item.getProjectId());
                vo.setType(type);
                vo.setTime(DateUtil.format(item.getValidityPeriod(),"yyyy-MM-dd"));
                vo.setName(BacklogTabEnum.getEnumByType(type).getName());
                vo.setOverdueType(vehicleTypeMap.getOrDefault(item.getOverdueType(),null));
                vo.setOverdueDays(item.getOverdueDays());
                vo.setDaysToOverdue(item.getDaysToOverdue());
                list.add(vo);
            });
            result.setTotal(asPageList.getTotal());
            result.setRecords(list);
        }
        if("7".equals(type)){
            IPage<TbCustomerFollow> followIPage = getFollowPage(jwtUser,pageNo,pageSize,projectIds);
            List<TbCustomerFollow> followList = followIPage.getRecords();
            //回访方式
            List<DictModel> reviewTypeList =  sysDictService.queryDictItemsByCode("review_type");
            Map<String, String> reviewTypeMap = reviewTypeList.stream().collect(Collectors.toMap(DictModel::getValue, DictModel::getText));
            //回访结果
            List<DictModel> resultTypeList =  sysDictService.queryDictItemsByCode("result_status");
            Map<String, String> resultTypeMap = resultTypeList.stream().collect(Collectors.toMap(DictModel::getValue, DictModel::getText));

            followList.stream().forEach(item->{
                BacklogListVO vo = new BacklogListVO();
                vo.setId(item.getId());
                vo.setPlateNumber(item.getFollowNo());
                vo.setProjectId(item.getProjectId());
                vo.setTime(DateUtil.format(item.getReviewTime(),"yyyy-MM-dd"));
                vo.setType(type);
                vo.setName(BacklogTabEnum.getEnumByType(type).getName());
                vo.setFollowType(reviewTypeMap.getOrDefault(item.getReviewType(),null));
                vo.setFollowStatus(resultTypeMap.getOrDefault(item.getResultStatus(),null));
                list.add(vo);
            });
            result.setTotal(followIPage.getTotal());
            result.setRecords(list);
        }
        return result;
    }

    @Override
    public IPage<TbViolationRecord> getViolationPage(JwtUser jwtUser,Integer pageNo,Integer pageSize,List<String> projectIds) {
        TbViolationRecord violationRecord = new TbViolationRecord();
        violationRecord.setHandleResult(0);
        Page<TbViolationRecord> violationPage = new Page<TbViolationRecord>(pageNo, pageSize);
        LambdaQueryWrapper<TbViolationRecord> lambdaQueryWrapper = QueryGenerator.initQueryWrapper(TbViolationRecord.class, violationRecord);
        lambdaQueryWrapper.in(!Collections.isEmpty(projectIds),TbViolationRecord::getProjectId, projectIds);
        lambdaQueryWrapper.orderByDesc(TbViolationRecord::getViolationTime);
        IPage<TbViolationRecord> violationPageList = violationRecordService.page(violationPage, lambdaQueryWrapper);
        return violationPageList;
    }

    @Override
    public IPage<RepairInfoListVO> getRepairPage(JwtUser jwtUser,Integer pageNo,Integer pageSize) {
        RepairInfoListVO repairInfo = new RepairInfoListVO();
        repairInfoService.dealRepairInfoListVO(jwtUser, repairInfo);
        Page<RepairInfoListVO> repairPage = new Page<RepairInfoListVO>(pageNo, pageSize);
        IPage<RepairInfoListVO> repairpageList = repairInfoService.getRepairBacklogList(repairPage,repairInfo,jwtUser);
        return repairpageList;
    }

    @Override
    public IPage<UpkeepInfoListVO> getUpkeepPage(JwtUser jwtUser,Integer pageNo,Integer pageSize) {
        String authorityType = commonService.getAuthority(jwtUser.getUserName());
        List<Integer> flowStatusList = new ArrayList<>();
//        if("2".equals(authorityType) || "3".equals(authorityType)){
//            flowStatusList.add(3); flowStatusList.add(6);
//        }
//        if("4".equals(authorityType) || "5".equals(authorityType) || "6".equals(authorityType)){
//            flowStatusList.add(2); flowStatusList.add(3);
//            flowStatusList.add(4); flowStatusList.add(6); flowStatusList.add(7);
//        }
        //获取用户对应待办按钮
        Map<String, List<Integer>> map = commonService.getUserBacklogButton(jwtUser);
        flowStatusList = map.getOrDefault(BacklogTabEnum.UPKEEP.getName(),new ArrayList<>());
        //没权限的调接口返回空
        if(flowStatusList.size() == 0){
            return new Page<>();
        }

        UpkeepInfoListVO  upkeep = new UpkeepInfoListVO();
        upkeep.setFlowStatusList(flowStatusList);
        //识别身份：需处理内部员工身份和维修厂身份的过滤
        //识别当前用户的身份（所属的机构类别 1普通部门，2维修厂，3客户公司（租赁方），0根节点）5为超管
        Integer departCategory = projectsService.judgeDepartCategory(jwtUser);
        //超级管理员查询全部数据
        if(departCategory != 5){
            //内部员工按项目列表过滤
            if(departCategory == 1 || departCategory == 4){
                List<String> listProjectStaff = projectsService.queryProjectListOfUser(jwtUser);
                if(CollectionUtils.isNotEmpty(listProjectStaff)){
                    upkeep.setListProject(listProjectStaff);
                }else {
                    upkeep.setProjectId("no permission");
                }
            }else if(departCategory == 2){
                List<TbRepairShopArchives> listRepairShop = tbRepairShopArchivesService.getRepairShopByUserId(jwtUser.getId());
                List<String> listRepairShopIds = listRepairShop.stream().map(TbRepairShopArchives::getId).collect(Collectors.toList());
                if(CollectionUtils.isNotEmpty(listRepairShopIds)){
                    upkeep.setListRepairShop(listRepairShopIds);
                }else {
                    upkeep.setProjectId("no permission");
                }
            }else{
                upkeep.setProjectId("no permission");
            }
        }

        Page<UpkeepInfoListVO> page = new Page<UpkeepInfoListVO>(pageNo, pageSize);
//        //领导多返回计划待办-->保养计划根据是否有权限返回
        if(map.containsKey(BacklogEnum.UPKEEP_PLAN_LEADER_CHECK_OFFER.getMenu())){
            IPage<UpkeepInfoListVO> upkeepPageList = upkeepBillService.getUpkeepPlanAndBillList(page, upkeep,jwtUser);
            return upkeepPageList;
        }else{
            IPage<UpkeepInfoListVO> upkeepPageList = upkeepBillService.getUpkeepBillList(page, upkeep,jwtUser);
            return upkeepPageList;
        }
    }

    @Override
    public IPage<TbAccidentInfo> getAccidentPage(JwtUser jwtUser,Integer pageNo,Integer pageSize,List<String> projectIds) {
        TbAccidentInfo info = new TbAccidentInfo();
        info.setListProject(projectIds);

        String authorityType = commonService.getAuthority(jwtUser.getUserName());
        if("4".equals(authorityType) || "5".equals(authorityType) || "6".equals(authorityType)){
            Page<TbAccidentInfo> accidentPage = new Page<TbAccidentInfo>(pageNo, pageSize);
            IPage<TbAccidentInfo> accidentpageList = iTbAccidentInfoService.todoList(accidentPage,info);
            return accidentpageList;
        }else{
            return new Page<>();
        }
    }

    @Override
    public IPage<TbInsuranceRecord> getInsurancePage(JwtUser jwtUser,Integer pageNo,Integer pageSize,List<String> projectIds) {
        String authorityType = commonService.getAuthority(jwtUser.getUserName());
        TbInsuranceRecord insuranceRecord = new TbInsuranceRecord();
        insuranceRecord.setHandleStatus("0");
        LambdaQueryWrapper<TbInsuranceRecord> queryWrapper = QueryGenerator.initQueryWrapper(TbInsuranceRecord.class, insuranceRecord);
        queryWrapper.in(!Collections.isEmpty(projectIds),TbInsuranceRecord::getProjectId,projectIds);
        queryWrapper.orderByDesc(TbInsuranceRecord::getInsuranceDate);
        if("4".equals(authorityType) || "5".equals(authorityType) || "6".equals(authorityType)){
            Page<TbInsuranceRecord> insuranceRecordPage = new Page<TbInsuranceRecord>(pageNo, pageSize);
            IPage<TbInsuranceRecord> insuranceRecordPageList = tbInsuranceRecordService.page(insuranceRecordPage,queryWrapper);
            return insuranceRecordPageList;
        }else{
            return new Page<>();
        }
    }

    @Override
    public IPage<TbAnnualInspectionRemind> getASPage(JwtUser jwtUser,Integer pageNo,Integer pageSize,List<String> projectIds) {
        String authorityType = commonService.getAuthority(jwtUser.getUserName());
        AnnualInspectionRemindQuery query = new AnnualInspectionRemindQuery();
        query.setListProject(projectIds);
        query.setRemindStatus("1");
        if("4".equals(authorityType) || "5".equals(authorityType) || "6".equals(authorityType)){
            Page<TbAnnualInspectionRemind> page = new Page<TbAnnualInspectionRemind>(pageNo, pageSize);
            LambdaQueryWrapper<TbAnnualInspectionRemind> lambdaQueryWrapper = QueryGenerator.initQueryWrapper(TbAnnualInspectionRemind.class,query);
            IPage<TbAnnualInspectionRemind> pageList = iTbAnnualInspectionRemindService.pageBuild(page, lambdaQueryWrapper);
            return pageList;
        }else{
            return new Page<>();
        }
    }

    @Override
    public IPage<TbCustomerFollow> getFollowPage(JwtUser jwtUser, Integer pageNo, Integer pageSize, List<String> projectIds) {
        String authorityType = commonService.getAuthority(jwtUser.getUserName());
        if("4".equals(authorityType) || "5".equals(authorityType) || "6".equals(authorityType)){
            TbCustomerFollow item = new TbCustomerFollow();
            LambdaQueryWrapper<TbCustomerFollow> queryWrapper = QueryGenerator.initQueryWrapper(TbCustomerFollow.class, item);
            //回访结果（0未开始，1未完成，2已完成）
            queryWrapper.in(TbCustomerFollow::getResultStatus,"0","1");
            if(CollectionUtil.isNotEmpty(projectIds)){
                queryWrapper.in(TbCustomerFollow::getProjectId,projectIds);
            }
            queryWrapper.orderByDesc(TbCustomerFollow::getReviewTime);
            Page<TbCustomerFollow> page = new Page<TbCustomerFollow>(pageNo, pageSize);
            IPage<TbCustomerFollow> pageList = tbCustomerFollowService.page(page,queryWrapper);
            for(TbCustomerFollow template:pageList.getRecords()) {
                tbCustomerFollowPartService.getPartsByFollowId(template);
                tbCustomerOpinionService.getOpinionByFollowId(template);
                tbCustomerFollowService.getInspectionTaskByFollowId(template);
            }
            return pageList;
        }else{
            return new Page<>();
        }
    }

    private long getPageCount(JwtUser jwtUser, BacklogTabEnum backlogTabEnum, List<String> projectIds) {
        switch (backlogTabEnum) {
            case VIOLATION:
                return getViolationPage(jwtUser, 1, 1,projectIds).getTotal();
            case REPAIR:
                return getRepairPage(jwtUser, 1, 1).getTotal();
            case UPKEEP:
                return getUpkeepPage(jwtUser, 1, 1).getTotal();
            case INSURANCE:
                return getInsurancePage(jwtUser, 1, 1,projectIds).getTotal();
            case ACCIDENNT:
                return getAccidentPage(jwtUser, 1, 1,projectIds).getTotal();
            case AS:
                return getASPage(jwtUser, 1, 1,projectIds).getTotal();
            case FOLLOW:
                return getFollowPage(jwtUser, 1, 1,projectIds).getTotal();
            default:
                return 0;
        }
    }

    private int getPageTotal(JwtUser jwtUser, String type,List<String> projectIds) {
        switch (type) {
            case "2":
                return getViolationPage(jwtUser, 1, -1,projectIds).getRecords().size();
            case "4":
                return getRepairPage(jwtUser, 1, -1).getRecords().size();
            case "5":
                return getUpkeepPage(jwtUser, 1, -1).getRecords().size();
            case "1":
                return getInsurancePage(jwtUser, 1, -1,projectIds).getRecords().size();
            case "3":
                return getAccidentPage(jwtUser, 1, -1,projectIds).getRecords().size();
            case "6":
                return getASPage(jwtUser, 1, -1,projectIds).getRecords().size();
            case "7":
                return getFollowPage(jwtUser, 1, -1,projectIds).getRecords().size();
            default:
                return 0;
        }
    }

}
