package com.gci.cdnev.smallprogram.modules.vehicles.service.impl;

import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gci.cdnev.smallprogram.exception.GciBootException;
import com.gci.cdnev.smallprogram.modules.vehicles.entity.TbAnnualInspectionInfo;
import com.gci.cdnev.smallprogram.modules.vehicles.entity.TbAnnualInspectionRemind;
import com.gci.cdnev.smallprogram.modules.vehicles.mapper.TbAnnualInspectionInfoMapper;
import com.gci.cdnev.smallprogram.modules.vehicles.query.AnnualInspectionInfoQuery;
import com.gci.cdnev.smallprogram.modules.vehicles.service.ITbAnnualInspectionInfoService;
import com.gci.cdnev.smallprogram.modules.vehicles.service.ITbAnnualInspectionRemindService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;

/**
 * <p>
 * 年检信息表 服务实现类
 * </p>
 *
 * @author 2464
 * @since 2023-09-20
 */
@Service
public class TbAnnualInspectionInfoServiceImpl extends ServiceImpl<TbAnnualInspectionInfoMapper, TbAnnualInspectionInfo> implements ITbAnnualInspectionInfoService {

    @Autowired
    private TbAnnualInspectionInfoMapper annualInspectionInfoMapper;
    @Autowired
    private ITbAnnualInspectionRemindService annualInspectionRemindService;

    @Override
    public IPage<TbAnnualInspectionInfo> pageBuild(Page<TbAnnualInspectionInfo> page, AnnualInspectionInfoQuery query) {
        checkTime(query);
        return annualInspectionInfoMapper.pageBuild(page,query);
    }

    private void checkTime(AnnualInspectionInfoQuery query) {
        List<Date> time = query.getTime();
        if (ObjectUtil.isNotEmpty(time)) {
            if (time.size()!=2) {
                throw new GciBootException("年审有效期必须包括起止时间！");
            }
            if (DateUtil.between(time.get(0), time.get(1), DateUnit.MS, false) < 0) {
                throw new GciBootException("开始时间不能大于结束时间！");
            }
        }
    }

    @Override
    public IPage<TbAnnualInspectionInfo> remindList(Page<TbAnnualInspectionInfo> page, AnnualInspectionInfoQuery query) {
        checkTime(query);
        IPage<TbAnnualInspectionInfo> tbAnnualInspectionInfoIPage = annualInspectionInfoMapper.remindList(page, query);
        for (TbAnnualInspectionInfo record : tbAnnualInspectionInfoIPage.getRecords()) {
            Integer days = record.getDays();
            if (days>0) {
                record.setDaysToOverdue(days);
            } else {
                record.setOverdueDays(-days);
            }
        }
        return tbAnnualInspectionInfoIPage;
    }

    @Override
    @Transactional(rollbackFor = RuntimeException.class)
    public void saveItem(TbAnnualInspectionInfo tbAnnualInspectionInfo) {
        save(tbAnnualInspectionInfo);
        TbAnnualInspectionRemind annualInspectionRemind = annualInspectionRemindService.buildRemind(tbAnnualInspectionInfo);
        annualInspectionRemindService.save(annualInspectionRemind);
        annualInspectionRemindService.recalculate(tbAnnualInspectionInfo.getPlateNumber());
    }
}
