package com.gci.cdnev.smallprogram.modules.vehicles.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONObject;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.write.style.column.LongestMatchColumnWidthStyleStrategy;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gci.cdnev.smallprogram.common.util.DateUtils;
import com.gci.cdnev.smallprogram.common.vo.Result;
import com.gci.cdnev.smallprogram.constant.CommonConstant;
import com.gci.cdnev.smallprogram.modules.basedata.entity.TbCustomerArchives;
import com.gci.cdnev.smallprogram.modules.basedata.entity.TbVehicle;
import com.gci.cdnev.smallprogram.modules.basedata.service.ITbCustomerArchivesService;
import com.gci.cdnev.smallprogram.modules.basedata.service.ITbVehicleService;
import com.gci.cdnev.smallprogram.modules.projects.service.IProjectsService;
import com.gci.cdnev.smallprogram.modules.system.service.impl.SysDictServiceImpl;
import com.gci.cdnev.smallprogram.modules.system.vo.DictModel;
import com.gci.cdnev.smallprogram.modules.vehicles.entity.TbRefuelChargeRecord;
import com.gci.cdnev.smallprogram.modules.vehicles.mapper.TbRefuelChargeRecordMapper;
import com.gci.cdnev.smallprogram.modules.vehicles.service.ITbRefuelChargeRecordService;
import com.gci.cdnev.smallprogram.modules.vehicles.vo.TbRefuelChargeRecordImportVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.math.BigDecimal;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * <p>
 * 加油充电记录 服务实现类
 * </p>
 *
 * @author admin
 * @since 2023-11-30
 */
@Service
public class TbRefuelChargeRecordServiceImpl extends ServiceImpl<TbRefuelChargeRecordMapper, TbRefuelChargeRecord> implements ITbRefuelChargeRecordService {

	@Autowired
	private ITbVehicleService vehicleService;
	@Autowired
	private ITbCustomerArchivesService customerArchivesService;
	@Autowired
	private IProjectsService projectsService;
	@Autowired
	private SysDictServiceImpl sysDictService;
	
	@Override
	public void saveItem(TbRefuelChargeRecord item) {
		initImte(item);
		saveOrUpdate(item);
	}
	
	public int getByOilCard(String oilCard) {
		if(!StringUtils.hasLength(oilCard)) {
			return 0;
		}
		LambdaQueryWrapper<TbVehicle> query = new LambdaQueryWrapper<>();
		query.eq(TbVehicle::getOilCard, oilCard);
		return vehicleService.count(query);
	}
	
	public void initImte(TbRefuelChargeRecord item) {
		int has = getByOilCard(item.getOilCard());
		if(has>0) {
			item.setRefuelType("1");
		}else {
			item.setRefuelType("2");
		}
		if(!StringUtils.hasLength(item.getOrderNo())) {
			item.setOrderNo("RC"+DateUtils.formatDate(new Date(), DateUtils.HYPHENS_DATETIME4));
		}
		item.setPrice(null);
		if(item.getTotalPrice()!=null&&item.getAmount()!=null&&!item.getAmount().equals(BigDecimal.ZERO)) {
			item.setPrice(item.getTotalPrice().divide(item.getAmount(),2,BigDecimal.ROUND_HALF_UP));
		}
		TbCustomerArchives com = customerArchivesService.getByProjectId(item.getProjectId());
		if(com!=null) {
			item.setSysOrgCode(com.getSysOrgCode());
		}
		//开始结束时间年月日处理
		Calendar recordDateCalendar = Calendar.getInstance();
		recordDateCalendar.setTime(item.getRecordDate());
		if(Objects.nonNull(item.getStartTime())){
			Calendar startTimeCalendar = Calendar.getInstance();
			startTimeCalendar.setTime(item.getStartTime());
			startTimeCalendar.set(Calendar.YEAR, recordDateCalendar.get(Calendar.YEAR));
			startTimeCalendar.set(Calendar.MONTH, recordDateCalendar.get(Calendar.MONTH));
			startTimeCalendar.set(Calendar.DAY_OF_MONTH, recordDateCalendar.get(Calendar.DAY_OF_MONTH));
			item.setStartTime(startTimeCalendar.getTime());
		}
		if(Objects.nonNull(item.getEndTime())){
			Calendar endTimeCalendar = Calendar.getInstance();
			endTimeCalendar.setTime(item.getEndTime());
			endTimeCalendar.set(Calendar.YEAR, recordDateCalendar.get(Calendar.YEAR));
			endTimeCalendar.set(Calendar.MONTH, recordDateCalendar.get(Calendar.MONTH));
			endTimeCalendar.set(Calendar.DAY_OF_MONTH, recordDateCalendar.get(Calendar.DAY_OF_MONTH));
			item.setEndTime(endTimeCalendar.getTime());
		}

	}
	
}
