package com.gci.cdnev.smallprogram.modules.vehicles.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import com.gci.cdnev.smallprogram.modules.projects.service.IProjectsService;
import com.gci.cdnev.smallprogram.modules.system.entity.SysDict;
import com.gci.cdnev.smallprogram.modules.system.entity.SysDictItem;
import com.gci.cdnev.smallprogram.modules.system.service.ISysDictItemService;
import com.gci.cdnev.smallprogram.modules.system.service.ISysDictService;
import com.gci.cdnev.smallprogram.modules.system.vo.DictModel;
import com.gci.cdnev.smallprogram.modules.vehicles.entity.TbViolationRecord;
import com.gci.cdnev.smallprogram.modules.vehicles.mapper.TbViolationRecordMapper;
import com.gci.cdnev.smallprogram.modules.vehicles.service.ITbViolationRecordService;
import com.gci.cdnev.smallprogram.modules.vehicles.vo.MessageVO;
import com.gci.cdnev.smallprogram.modules.vehicles.vo.VehicleProjectVO;
import com.gci.cdnev.smallprogram.modules.wx.entity.JwtUser;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

@Service
@Slf4j
public class TbViolationRecordServiceImpl extends ServiceImpl<TbViolationRecordMapper, TbViolationRecord> implements ITbViolationRecordService {

    @Autowired
    private TbViolationRecordMapper violationRecordMapper;
    @Autowired
    private ISysDictService sysDictService;
    @Autowired
    private ISysDictItemService sysDictItemService;
    @Autowired
    private IProjectsService projectsService;

    @Override
    public List<VehicleProjectVO> queryVehicleProject(String plateNumber) {
        return violationRecordMapper.queryVehicleProject(plateNumber);
    }

    @Override
    public List<VehicleProjectVO> queryAllVehicle() {
        return violationRecordMapper.queryAllVehicle();
    }

    @Override
    public List<VehicleProjectVO> queryAllProject(JwtUser jwtUser) {
//        List<String> projectIds = jwtUser.getProjectIds();
        Integer departCategory = projectsService.judgeDepartCategory(jwtUser);
        //超级管理员查询全部数据
        List<String> projectIds = new ArrayList<>();
        if(departCategory != 5){
            projectIds = projectsService.queryProjectListOfUser(jwtUser);
            if(CollectionUtil.isEmpty(projectIds) || projectIds.size() == 0 ){
                projectIds.add("no permission");
            }
        }
        return violationRecordMapper.queryProjects(0,0,projectIds);
    }

    @Override
    public MessageVO analyzeMessage(String template) {
        log.info("识别短信 {}", template);
        //模板1: 【四川交警】您的小型新能源汽车川AAP0633于2023-08-19 09:10在成都市二环路西二段与战旗东路交叉路口，被交通技术监控设备记录了『不按导向车道行驶』的违法行为。请于收到本告知之日起30日内接受处理。
        //模板2: 【成都交警】川AAP8022于2023-10-30在瑞星路与琼楼路交叉路口实施机动车通过有灯控路口时，不按所需行进方向驶入导向车道的违法行为，请三十日内到交警部门接受处理。该车有18笔违法记录未处理，请及时处理。(更多请关注蓉E行)
        MessageVO vo = new MessageVO();
        // 定义车牌号的正则表达式
        String regexPlateNumber = "[京津沪渝冀豫云辽黑湘皖鲁新苏浙赣鄂桂甘晋蒙陕吉闽贵粤青藏川宁琼使领][A-HJ-NP-Z][A-HJ-NP-Z0-9]{4,5}[A-HJ-NP-Z0-9挂学警港澳]*";
        // 定义时间的正则表达式--模板1
        String regexTime = "\\d{4}-\\d{2}-\\d{2} \\d{2}:\\d{2}";
        // 定义地点的正则表达式--模板1
        String regexLocation = "(\\d{4}-\\d{2}-\\d{2} \\d{2}:\\d{2})在(.+?)，";
        //违章行为--模板1
        String regexViolationAct = "『([^』]+)』";

        Pattern patternPlateNumber = Pattern.compile(regexPlateNumber);
        Matcher matcherPlateNumber = patternPlateNumber.matcher(template);
        if (matcherPlateNumber.find()) {
            vo.setPlateNumber(matcherPlateNumber.group());
        }

        Pattern patternTime = Pattern.compile(regexTime);
        Matcher matcherTime = patternTime.matcher(template);
        if (matcherTime.find()) {
            vo.setViolationTime(matcherTime.group());
        }else{
            //时间模板2处理
            Matcher matcherTime2 = Pattern.compile("\\d{4}-\\d{2}-\\d{2}").matcher(template);
            if (matcherTime2.find()) {
                vo.setViolationTime(matcherTime2.group() + " 00:00");
            }
        }

        Pattern patternAddress = Pattern.compile(regexLocation);
        Matcher matcherAddress = patternAddress.matcher(template);
        if (matcherAddress.find()) {
            vo.setViolationAddress(matcherAddress.group(2));
        }else{
            //地点模板2处理
            Matcher matcherAddress2 = Pattern.compile("(\\d{4}-\\d{2}-\\d{2})在(.+?)实施").matcher(template);
            if (matcherAddress2.find()) {
                vo.setViolationAddress(matcherAddress2.group(2));
            }
        }

        Pattern pattern = Pattern.compile(regexViolationAct);
        Matcher matcher = pattern.matcher(template);
        if (matcher.find()) {
            String violation = matcher.group(1);
            log.info("违章行为 {}", violation);
            //查字典是否存在，没有存在的话加到字典里面去
            //违章行为字典
            List<DictModel> violationList =  sysDictService.queryDictItemsByCode("violation_behavior");
            Map<String, String> violationMap = violationList.stream().collect(Collectors.toMap(DictModel::getValue, DictModel::getText));
            String act = "";
            for (String key : violationMap.keySet()) {
                String value = violationMap.getOrDefault(key,"");
                if(StrUtil.isNotBlank(value) && violation.equals(value)){
                    act = key;
                    break;
                }
            }
            if(StrUtil.isNotBlank(act)){
                vo.setViolationAct(act);
            }else{
                vo.setIsNewBehavior(true);
                vo.setNewBehavior(violation);
            }
        }else{
            //违章行为模板2处理
            Matcher matcher2 = Pattern.compile("实施(.+?)的违法行为").matcher(template);
            if (matcher2.find()) {
                String violation = matcher2.group(1);
                log.info("违章行为 {}", violation);
                //查字典是否存在，没有存在的话加到字典里面去
                //违章行为字典
                List<DictModel> violationList =  sysDictService.queryDictItemsByCode("violation_behavior");
                Map<String, String> violationMap = violationList.stream().collect(Collectors.toMap(DictModel::getValue, DictModel::getText));
                String act = "";
                for (String key : violationMap.keySet()) {
                    String value = violationMap.getOrDefault(key,"");
                    if(StrUtil.isNotBlank(value) && violation.equals(value)){
                        act = key;
                        break;
                    }
                }
                if(StrUtil.isNotBlank(act)){
                    vo.setViolationAct(act);
                }else{
                    vo.setIsNewBehavior(true);
                    vo.setNewBehavior(violation);
                }
            }else{
                vo.setIsNewBehavior(null);
            }
        }

        return vo;
    }

    @Override
    public String addViolationBehavior(MessageVO vo) {
        //违章行为字典
        List<DictModel> violationList =  sysDictService.queryDictItemsByCode("violation_behavior");
        List<String> distTexts = violationList.stream().map(DictModel::getText).collect(Collectors.toList());
        if(distTexts.contains(vo.getNewBehavior())){
            return "已存在该违章行为!";
        }
        //保存到字典
        LambdaQueryWrapper<SysDict> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(SysDict::getDictCode,"violation_behavior").last(" limit 1 ");
        SysDict sysDict = sysDictService.getOne(queryWrapper);

        SysDictItem dictItem = new SysDictItem();
        dictItem.setDictId(sysDict.getId());
        dictItem.setItemText(vo.getNewBehavior());
        dictItem.setItemValue(String.valueOf(getMaxValue(violationList)));
        dictItem.setStatus(1);
        sysDictItemService.save(dictItem);
        return dictItem.getItemValue();
    }

    public int getMaxValue(List<DictModel> violationList) {
        int maxValue = Integer.MIN_VALUE;
        for (DictModel dictModel : violationList) {
            String valueStr = dictModel.getValue();
            int value = Integer.parseInt(valueStr);
            if (value > maxValue) {
                maxValue = value;
            }
        }
        return maxValue + 1;
    }
}