package com.gci.cdnev.smallprogram.modules.wx.controller;


import java.nio.charset.StandardCharsets;
import java.util.Base64;
import java.util.HashMap;
import java.util.Map;

import com.gci.cdnev.smallprogram.common.log.annotation.AddSysLog;
import com.gci.cdnev.smallprogram.modules.system.entity.LoginTimesValidate;
import com.gci.cdnev.smallprogram.modules.system.service.LoginTimesValidateService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.gci.cdnev.smallprogram.common.util.JsonUtil;
import com.gci.cdnev.smallprogram.common.vo.Result;
import com.gci.cdnev.smallprogram.modules.system.entity.SysUser;
import com.gci.cdnev.smallprogram.modules.system.service.ISysUserService;
import com.gci.cdnev.smallprogram.modules.system.util.AesEncryptUtil;
import com.gci.cdnev.smallprogram.modules.system.util.PasswordUtil;
import com.gci.cdnev.smallprogram.modules.wx.entity.LoginDto;
import com.gci.cdnev.smallprogram.modules.wx.service.WxLoginService;

import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

import javax.crypto.Cipher;
import javax.crypto.spec.SecretKeySpec;
import java.util.Base64;

@RestController
@RequestMapping("/api/user")
@Slf4j
public class WxLoginController {

    /**
     * 解码次数
     */
    private static final int decodeCount = 3;

    @Autowired
    private WxLoginService wxLoginService;
    @Autowired
    private ISysUserService sysUserService;

    @Autowired
    private LoginTimesValidateService loginTimesValidateService;



    @PostMapping("/loginV2")
    public Result<?> loginV2(@RequestBody LoginDto loginDto) {
        if (StrUtil.isBlank(loginDto.getUserName()) || StrUtil.isBlank(loginDto.getPassword())) {
            return Result.error("用户名或密码不能为空");
        }
        if (StrUtil.isBlank(loginDto.getCode())) {
            return Result.error("小程序参数不能为空");
        }
        log.info("小程序登录loginDto:{}",JsonUtil.object2Json(loginDto));
        String userName = loginDto.getUserName().trim();
        SysUser sysUser = sysUserService.getByUsername(userName);
        if(sysUser==null){
            return Result.error("用户不存在");
        }
        log.info("小程序登录sysUser:{}",JsonUtil.object2Json(sysUser));

        LoginTimesValidate validate = loginTimesValidateService.getManageTimes(userName);
        if (validate.nonTimes()) {
            return Result.error("连续输入错误密码次数超限，请稍后重试");
        }
        //2. 校验用户名或密码是否正确
		try {
			//TODO
			//String password = AesEncryptUtil.desEncrypt(loginDto.getPassword().replaceAll("%2B", "\\+")).trim();
//            String password = loginDto.getPassword();
//            int i=0;
//            while (++i <= decodeCount) {
//                password = new String(Base64.getDecoder().decode(password.getBytes(StandardCharsets.UTF_8)));
//            }
			String password = loginDto.getPassword().replaceAll("%2B", "\\+");
			//解密,连续三次解密,同步小程序端三次加密
			password = decode(decode(decode(password)));
			String userpassword = PasswordUtil.encrypt(loginDto.getUserName(), password, sysUser.getSalt());
			String syspassword = sysUser.getPassword();
			if(!userpassword.equals(syspassword)){
                loginTimesValidateService.saveManageTimes(validate);
				return Result.error("用户名或密码错误！");
			}
		} catch (Exception e) {
            log.info("小程序登录验证密码出错",e);
			return Result.error("小程序登录失败");
		}
		if(!StringUtils.hasLength(sysUser.getType())){
			return Result.error("暂无小程序登录权限");
		}
		if(sysUser.getStatus()!=1){
			return Result.error("账号已冻结");
		}
//        if(!StringUtils.hasLength(sysUser.getOpenid())){
//        	String openid = wxLoginService.obtainOpenId(loginDto.getCode());
//        	sysUser.setOpenid(openid);
//            if (!StringUtils.hasLength(openid)) {
//            	log.info("绑定小程序失败openid：{}",JsonUtil.object2Json(sysUser));
//                return Result.error("绑定小程序失败");
//            }else {
//            	log.info("小程序登录绑定的openid：{}",JsonUtil.object2Json(sysUser));
//            }
//            //一个微信用户只能绑定一个账号
//            SysUser user = sysUserService.getByOpenid(openid);
//            if (user == null) {
//            	sysUserService.updateById(sysUser);
//            }
//        }

        HashMap<String, Object> userMap = wxLoginService.creatUserToken(sysUser);
        return Result.ok(userMap);
    }

    @AddSysLog(value = "用户登录", logType = 1)
    @PostMapping("/login")
    public Result<?> login(@RequestBody LoginDto loginDto) {
        if (StrUtil.isBlank(loginDto.getUserName()) || StrUtil.isBlank(loginDto.getPassword())) {
            return Result.error("用户名或密码不能为空");
        }
        if (StrUtil.isBlank(loginDto.getCode())) {
            return Result.error("小程序参数不能为空");
        }
        log.info("小程序登录loginDto:{}",JsonUtil.object2Json(loginDto));
        String userName = loginDto.getUserName().trim();
        SysUser sysUser = sysUserService.getByUsername(userName);
        if(sysUser==null){
            return Result.error("用户名或密码错误！");
        }
        log.info("小程序登录sysUser:{}",JsonUtil.object2Json(sysUser));

        LoginTimesValidate validate = loginTimesValidateService.getManageTimes(userName);
        if (validate.nonTimes()) {
            return Result.error("连续输入错误密码次数超限，请稍后重试");
        }
        //2. 校验用户名或密码是否正确
        try {
            //TODO
            //String password = AesEncryptUtil.desEncrypt(loginDto.getPassword().replaceAll("%2B", "\\+")).trim();
//            String password = loginDto.getPassword();
//            int i=0;
//            while (++i <= decodeCount) {
//                password = new String(Base64.getDecoder().decode(password.getBytes(StandardCharsets.UTF_8)));
//            }
            String password = AesEncryptUtil.desEncrypt(loginDto.getPassword().replaceAll("%2B", "\\+")).trim();
            String userpassword = PasswordUtil.encrypt(loginDto.getUserName(), password, sysUser.getSalt());
            String syspassword = sysUser.getPassword();
            if(!userpassword.equals(syspassword)){
                loginTimesValidateService.saveManageTimes(validate);
                return Result.error("用户名或密码错误！");
            }
        } catch (Exception e) {
            log.info("小程序登录验证密码出错",e);
            return Result.error("小程序登录失败");
        }
		if(!StringUtils.hasLength(sysUser.getType())){
			return Result.error("暂无小程序登录权限");
		}
		if(sysUser.getStatus()!=1){
			return Result.error("账号已冻结");
		}
        //todo 处理 首次登录是需要绑定openid，绑定后就需要判断账号和openid一致
        //1、账号未绑定openid,登录微信用户openid未给其它用户绑定--绑定;
        //2、账号未绑定openid,登录微信用户openid被其它用户绑定---提示错误;
        //3、账号绑定openid,校验是否与登录微信用户openid一致
        String openid = wxLoginService.obtainOpenId(loginDto.getCode());
        if (!StringUtils.hasLength(openid)) {
            log.info("绑定小程序失败openid：{}",JsonUtil.object2Json(sysUser));
            return Result.error("绑定小程序失败,openid为NULL");
        }
        if(!StringUtils.hasLength(sysUser.getOpenid())){
            //未绑定
            sysUser.setOpenid(openid);
            //一个微信用户只能绑定一个账号
            SysUser user = sysUserService.getByOpenid(openid);
            if (user == null) {
                sysUserService.updateById(sysUser);
            }else{
                return Result.error("登录失败,该用户已绑定其它账号");
            }
        }else{
            //已绑定的情况
            if(!sysUser.getOpenid().equals(openid)){
                return Result.error("登录失败,该账号已绑定其它用户");
            }

        }

        HashMap<String, Object> userMap = wxLoginService.creatUserToken(sysUser);
        return Result.ok(userMap);
    }

    /**
     * 获取加密字符串
     *
     * @return
     */
    @GetMapping("/getEncryptedString")
    public Result<Map<String, String>> getEncryptedString() {
        Result<Map<String, String>> result = new Result<Map<String, String>>();
        Map<String, String> map = new HashMap<String, String>();
        map.put("key", AesEncryptUtil.key);
        map.put("iv", AesEncryptUtil.iv);
        result.setResult(map);
        return result;
    }

    /**
     * 小程序登录
     * @param code
     * @return
     */
    @GetMapping("/wxlogin")
    @AddSysLog(value = "用户登录", logType = 1)
    public Result<?> wxlogin(String code) {
        if (StrUtil.isBlank(code)) {
            return Result.error("小程序参数不能为空");
        }
        String openid = wxLoginService.obtainOpenId(code);
        if (!StringUtils.hasLength(openid)) {
            return Result.error("获取用户信息失败");
        }
        log.info("小程序登录openid:{}",openid);
        SysUser user = sysUserService.getByOpenid(openid);
        if (user == null) {
            log.error("未绑定小程序openid:{}",openid);
            return Result.error("未绑定小程序,请先用账号登录,绑定小程序");
        }
        if(!StringUtils.hasLength(user.getType())){
            return Result.error("暂无小程序登录权限");
        }
        if(user.getStatus()!=1){
            return Result.error("账号已冻结");
        }
        HashMap<String, Object> userMap = wxLoginService.creatUserToken(user);
        return Result.ok(userMap);
    }

    @GetMapping("/getByRoleCode")
    public Result<?> getByRoleCode(String roleCode) {
        return Result.ok(sysUserService.getByRoleCode(roleCode));
    }

    final static Base64.Encoder encoder = Base64.getEncoder();
    final static Base64.Decoder decoder = Base64.getDecoder();

    public static String encode(String text) {
        return encoder.encodeToString(text.getBytes(StandardCharsets.UTF_8));
    }

    public static String decode(String encodedText) {
        return new String(decoder.decode(encodedText), StandardCharsets.UTF_8);
    }

    public static void main(String[] args) {
        String str = "Liqiao@123";

        System.out.println("编码后的字符串为：");
        String strEncode = encode(encode(encode(str)));
        System.out.println(strEncode);

        System.out.println("解码后的字符串为：");
        String strDecode = decode(decode(decode(strEncode)));
        System.out.println(strDecode);

        String strDecode2 = decode(decode(decode("VVZSRmVVMTZVVEZPYWpnOQ==")));
        System.out.println(strDecode2);
    }


}
