package com.gci.cdnev.smallprogram.modules.wx.service.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

import com.gci.cdnev.smallprogram.modules.system.entity.SysPermission;
import com.gci.cdnev.smallprogram.modules.system.service.ISysPermissionService;
import com.gci.cdnev.smallprogram.modules.system.service.ISysUserService;
import com.gci.cdnev.smallprogram.modules.vehicles.service.CommonService;
import com.gci.cdnev.smallprogram.properties.GciSecurityProperties;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import com.gci.cdnev.smallprogram.common.util.JsonUtil;
import com.gci.cdnev.smallprogram.common.util.JwtUtils;
import com.gci.cdnev.smallprogram.config.JwtConfig;
import com.gci.cdnev.smallprogram.constant.CommonConstant;
import com.gci.cdnev.smallprogram.modules.system.entity.SysDepart;
import com.gci.cdnev.smallprogram.modules.system.entity.SysUser;
import com.gci.cdnev.smallprogram.modules.system.service.ISysDepartService;
import com.gci.cdnev.smallprogram.modules.wx.entity.JwtUser;
import com.gci.cdnev.smallprogram.modules.wx.service.WxLoginService;

import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import lombok.extern.slf4j.Slf4j;

@Service
@Slf4j
public class WxLoginServiceImpl implements WxLoginService{


    @Autowired
    private JwtConfig jwtConfig;


    @Autowired
    private RestTemplate restTemplate;

    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    @Autowired
    private ISysDepartService sysDepartService;
    @Autowired
    private CommonService commonService;
    @Autowired
    private ISysUserService sysUserService;
    @Autowired
    private ISysPermissionService sysPermissionService;

    @Override
    public String obtainOpenId(String code) {
        StringBuilder url = new StringBuilder(CommonConstant.REQUEST_URL);
        url.append("?grant_type=authorization_code");
        url.append("&appid=" + jwtConfig.getAppId());
        url.append("&secret=" + jwtConfig.getDecSecret());
        url.append("&js_code=" + code);
        String resp = restTemplate.getForObject(url.toString(), String.class);

        JSONObject json = JSONUtil.parseObj(resp);

        log.error("获取小程序openid，响应: {}", resp);
        String openid = json.getStr("openid");

        return openid;
    }


    @Override
    public HashMap<String, Object> creatUserToken(SysUser sysUser) {
        JwtUser jwtUser = new JwtUser();
        jwtUser.setId(sysUser.getId());
        jwtUser.setUserName(sysUser.getUsername());
        jwtUser.setRealname(sysUser.getRealname());
        jwtUser.setEmployeeId(sysUser.getEmployeeId());
        jwtUser.setEmployeeName(sysUser.getEmployeeName());
        jwtUser.setType(sysUser.getType());
        jwtUser.setSysOrgCode(sysUser.getOrgCode());
        jwtUser.setProjectIds(sysUser.getProjectIds());
        //获取所有机构信息
        SysDepart orgDto = sysDepartService.getByUserId(sysUser.getId());
        if(orgDto!=null) {
        	jwtUser.setSysOrgId(orgDto.getId());
        	jwtUser.setSysOrgName(orgDto.getDepartName());
        }

        String token = JwtUtils.createJwtToken(jwtUser);
        HashMap<String, Object> userMap = new HashMap<>();
        userMap.put("id", sysUser.getId());
        userMap.put("userName", sysUser.getUsername());
        userMap.put("realname", sysUser.getRealname());
        userMap.put("token", token);
        userMap.put("sysOrgName",jwtUser.getSysOrgName());
        userMap.put("type", sysUser.getType());
        userMap.put("employeeId", sysUser.getEmployeeId());
        userMap.put("employeeName", sysUser.getEmployeeName());
        //补充返回用户角色，给到小程序端用于维修保养流程处理
        userMap.put("userType",commonService.getAuthority(sysUser.getUsername()));
        userMap.put("wx-openId",sysUser.getOpenid());
        //userMap.put("orgCode", sysUser.getOrgCode());
        //登录成功给redis放着续期token
        log.info("用户登录后给用户续期,时间为token有效期的两倍,token有效期:{}毫秒", JwtUtils.expireTime);
        String tokenKey = String.join("", CommonConstant.SYS_USER_MOBILE_LOGIN, token);
        redisTemplate.opsForValue().set(tokenKey, JsonUtil.object2Json(jwtUser), JwtUtils.expireTime * 2, TimeUnit.MILLISECONDS);
        //补充菜单权限
        List<SysPermission> metaList = sysPermissionService.queryByUser(sysUser.getUsername());
        List<Map<String, Object>> authArray = new ArrayList();
        sysUserService.getAuthArray(authArray, metaList);
        //按钮权限（用户拥有的权限集合）
        userMap.put("auth", authArray);
        return userMap;
    }

    public void jwtTokenRefresh(String token,JwtUser jwtUser){
        log.info("——————————用户在线操作，更新token保证不掉线—————————jwtTokenRefresh——————— "+ token);
        String tokenKey = String.join("", CommonConstant.SYS_USER_MOBILE_LOGIN, token);
        redisTemplate.opsForValue().set(tokenKey, JsonUtil.object2Json(jwtUser), JwtUtils.expireTime * 2, TimeUnit.MILLISECONDS);
    }


}
